! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
! 
! Linear Algebra Data and Routines File
! 
! Generated by KPP-2.2.1_rs5 symbolic chemistry Kinetics PreProcessor
!       (http://www.cs.vt.edu/~asandu/Software/KPP)
! KPP is distributed under GPL, the general public licence
!       (http://www.gnu.org/copyleft/gpl.html)
! (C) 1995-1997, V. Damian & A. Sandu, CGRER, Univ. Iowa
! (C) 1997-2005, A. Sandu, Michigan Tech, Virginia Tech
!     With important contributions from:
!        M. Damian, Villanova University, USA
!        R. Sander, Max-Planck Institute for Chemistry, Mainz, Germany
! 
! File                 : gckpp_LinearAlgebra.f90
! Time                 : Mon Oct 19 13:24:39 2015
! Working directory    : /home/mpayer/KPP
! Equation file        : gckpp.kpp
! Output root filename : gckpp
! 
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~



MODULE gckpp_LinearAlgebra

  USE gckpp_Parameters
  USE gckpp_JacobianSP

  IMPLICIT NONE

CONTAINS


! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
! 
! SPARSE_UTIL - SPARSE utility functions
!   Arguments :
! 
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~


! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
SUBROUTINE KppDecomp( JVS, IER )
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!        Sparse LU factorization
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

  USE gckpp_Parameters
  USE gckpp_JacobianSP

      INTEGER  :: IER
      REAL(kind=dp) :: JVS(LU_NONZERO), W(NVAR), a
      INTEGER  :: k, kk, j, jj

      a = 0. ! mz_rs_20050606
      IER = 0
      DO k=1,NVAR
        ! mz_rs_20050606: don't check if real value == 0
        ! IF ( JVS( LU_DIAG(k) ) .EQ. 0. ) THEN
        IF ( ABS(JVS(LU_DIAG(k))) < TINY(a) ) THEN
            IER = k
            RETURN
        END IF
        DO kk = LU_CROW(k), LU_CROW(k+1)-1
              W( LU_ICOL(kk) ) = JVS(kk)
        END DO
        DO kk = LU_CROW(k), LU_DIAG(k)-1
            j = LU_ICOL(kk)
            a = -W(j) / JVS( LU_DIAG(j) )
            W(j) = -a
            DO jj = LU_DIAG(j)+1, LU_CROW(j+1)-1
               W( LU_ICOL(jj) ) = W( LU_ICOL(jj) ) + a*JVS(jj)
            END DO
         END DO
         DO kk = LU_CROW(k), LU_CROW(k+1)-1
            JVS(kk) = W( LU_ICOL(kk) )
         END DO
      END DO
      
END SUBROUTINE KppDecomp


! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
SUBROUTINE KppDecompCmplx( JVS, IER )
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!        Sparse LU factorization, complex
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

  USE gckpp_Parameters
  USE gckpp_JacobianSP

      INTEGER        :: IER
      DOUBLE COMPLEX :: JVS(LU_NONZERO), W(NVAR), a
      REAL(kind=dp)  :: b = 0.0
      INTEGER        :: k, kk, j, jj

      IER = 0
      DO k=1,NVAR
        IF ( ABS(JVS(LU_DIAG(k))) < TINY(b) ) THEN
            IER = k
            RETURN
        END IF
        DO kk = LU_CROW(k), LU_CROW(k+1)-1
              W( LU_ICOL(kk) ) = JVS(kk)
        END DO
        DO kk = LU_CROW(k), LU_DIAG(k)-1
            j = LU_ICOL(kk)
            a = -W(j) / JVS( LU_DIAG(j) )
            W(j) = -a
            DO jj = LU_DIAG(j)+1, LU_CROW(j+1)-1
               W( LU_ICOL(jj) ) = W( LU_ICOL(jj) ) + a*JVS(jj)
            END DO
         END DO
         DO kk = LU_CROW(k), LU_CROW(k+1)-1
            JVS(kk) = W( LU_ICOL(kk) )
         END DO
      END DO
      
END SUBROUTINE KppDecompCmplx


! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
SUBROUTINE KppDecompCmplxR( JVSR, JVSI, IER )
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!    Sparse LU factorization, complex
!   (Real and Imaginary parts are used instead of complex data type)     
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

  USE gckpp_Parameters
  USE gckpp_JacobianSP

      INTEGER       :: IER
      REAL(kind=dp) :: JVSR(LU_NONZERO), JVSI(LU_NONZERO) 
      REAL(kind=dp) :: WR(NVAR), WI(NVAR), ar, ai, den
      INTEGER       :: k, kk, j, jj

      IER = 0
      ar  = 0.0
      DO k=1,NVAR
        IF (  ( ABS(JVSR(LU_DIAG(k))) < TINY(ar) ) .AND. &
              ( ABS(JVSI(LU_DIAG(k))) < TINY(ar) ) )  THEN
            IER = k
            RETURN
        END IF
        DO kk = LU_CROW(k), LU_CROW(k+1)-1
              WR( LU_ICOL(kk) ) = JVSR(kk)
              WI( LU_ICOL(kk) ) = JVSI(kk)
        END DO
        DO kk = LU_CROW(k), LU_DIAG(k)-1
            j = LU_ICOL(kk)
            den = JVSR(LU_DIAG(j))**2 + JVSI(LU_DIAG(j))**2
            ar = -(WR(j)*JVSR(LU_DIAG(j)) + WI(j)*JVSI(LU_DIAG(j)))/den
            ai = -(WI(j)*JVSR(LU_DIAG(j)) - WR(j)*JVSI(LU_DIAG(j)))/den
            WR(j) = -ar
            WI(j) = -ai
            DO jj = LU_DIAG(j)+1, LU_CROW(j+1)-1
               WR( LU_ICOL(jj) ) = WR( LU_ICOL(jj) ) + ar*JVSR(jj) - ai*JVSI(jj)
               WI( LU_ICOL(jj) ) = WI( LU_ICOL(jj) ) + ar*JVSI(jj) + ai*JVSR(jj)
            END DO
         END DO
         DO kk = LU_CROW(k), LU_CROW(k+1)-1
            JVSR(kk) = WR( LU_ICOL(kk) )
            JVSI(kk) = WI( LU_ICOL(kk) )
         END DO
      END DO

END SUBROUTINE KppDecompCmplxR


! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
SUBROUTINE KppSolveIndirect( JVS, X )
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!        Sparse solve subroutine using indirect addressing
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

  USE gckpp_Parameters
  USE gckpp_JacobianSP

      INTEGER  :: i, j
      REAL(kind=dp) :: JVS(LU_NONZERO), X(NVAR), sum

      DO i=1,NVAR
         DO j = LU_CROW(i), LU_DIAG(i)-1 
             X(i) = X(i) - JVS(j)*X(LU_ICOL(j));
         END DO  
      END DO

      DO i=NVAR,1,-1
        sum = X(i);
        DO j = LU_DIAG(i)+1, LU_CROW(i+1)-1
          sum = sum - JVS(j)*X(LU_ICOL(j));
        END DO
        X(i) = sum/JVS(LU_DIAG(i));
      END DO
      
END SUBROUTINE KppSolveIndirect


! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
SUBROUTINE KppSolveTRIndirect( JVS, X )
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!        Complex sparse solve transpose subroutine using indirect addressing
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

  USE gckpp_Parameters
  USE gckpp_JacobianSP

      INTEGER       :: i, j
      REAL(kind=dp) :: JVS(LU_NONZERO), X(NVAR)

      DO i=1,NVAR
        X(i) = X(i)/JVS(LU_DIAG(i))
        ! subtract all nonzero elements in row i of JVS from X
        DO j=LU_DIAG(i)+1,LU_CROW(i+1)-1
          X(LU_ICOL(j)) = X(LU_ICOL(j))-JVS(j)*X(i)
        END DO
      END DO

      DO i=NVAR, 1, -1
        ! subtract all nonzero elements in row i of JVS from X
        DO j=LU_CROW(i),LU_DIAG(i)-1
          X(LU_ICOL(j)) = X(LU_ICOL(j))-JVS(j)*X(i)
        END DO
      END DO
      
END SUBROUTINE KppSolveTRIndirect


! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
SUBROUTINE KppSolveCmplx( JVS, X )
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!        Complex sparse solve subroutine using indirect addressing
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

  USE gckpp_Parameters
  USE gckpp_JacobianSP

      INTEGER        :: i, j
      DOUBLE COMPLEX :: JVS(LU_NONZERO), X(NVAR), sum

      DO i=1,NVAR
         DO j = LU_CROW(i), LU_DIAG(i)-1 
             X(i) = X(i) - JVS(j)*X(LU_ICOL(j));
         END DO  
      END DO

      DO i=NVAR,1,-1
        sum = X(i);
        DO j = LU_DIAG(i)+1, LU_CROW(i+1)-1
          sum = sum - JVS(j)*X(LU_ICOL(j));
        END DO
        X(i) = sum/JVS(LU_DIAG(i));
      END DO
      
END SUBROUTINE KppSolveCmplx

! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
SUBROUTINE KppSolveCmplxR( JVSR, JVSI, XR, XI )
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!   Complex sparse solve subroutine using indirect addressing
!   (Real and Imaginary parts are used instead of complex data type)     
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

  USE gckpp_Parameters
  USE gckpp_JacobianSP

      INTEGER       ::  i, j
      REAL(kind=dp) ::  JVSR(LU_NONZERO), JVSI(LU_NONZERO), XR(NVAR), XI(NVAR), sumr, sumi, den

      DO i=1,NVAR
         DO j = LU_CROW(i), LU_DIAG(i)-1 
             XR(i) = XR(i) - (JVSR(j)*XR(LU_ICOL(j)) - JVSI(j)*XI(LU_ICOL(j)))
             XI(i) = XI(i) - (JVSR(j)*XI(LU_ICOL(j)) + JVSI(j)*XR(LU_ICOL(j)))
         END DO  
      END DO

      DO i=NVAR,1,-1
        sumr = XR(i); sumi = XI(i)
        DO j = LU_DIAG(i)+1, LU_CROW(i+1)-1
            sumr = sumr - (JVSR(j)*XR(LU_ICOL(j)) - JVSI(j)*XI(LU_ICOL(j)))
            sumi = sumi - (JVSR(j)*XI(LU_ICOL(j)) + JVSI(j)*XR(LU_ICOL(j)))
        END DO
        den   = JVSR(LU_DIAG(i))**2 + JVSI(LU_DIAG(i))**2
        XR(i) = (sumr*JVSR(LU_DIAG(i)) + sumi*JVSI(LU_DIAG(i)))/den
        XI(i) = (sumi*JVSR(LU_DIAG(i)) - sumr*JVSI(LU_DIAG(i)))/den
      END DO
      
END SUBROUTINE KppSolveCmplxR


! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
SUBROUTINE KppSolveTRCmplx( JVS, X )
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!        Complex sparse solve transpose subroutine using indirect addressing
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

  USE gckpp_Parameters
  USE gckpp_JacobianSP

      INTEGER        :: i, j
      DOUBLE COMPLEX :: JVS(LU_NONZERO), X(NVAR)

      DO i=1,NVAR
        X(i) = X(i)/JVS(LU_DIAG(i))
        ! subtract all nonzero elements in row i of JVS from X
        DO j=LU_DIAG(i)+1,LU_CROW(i+1)-1
          X(LU_ICOL(j)) = X(LU_ICOL(j))-JVS(j)*X(i)
        END DO
      END DO

      DO i=NVAR, 1, -1
        ! subtract all nonzero elements in row i of JVS from X
        DO j=LU_CROW(i),LU_DIAG(i)-1
          X(LU_ICOL(j)) = X(LU_ICOL(j))-JVS(j)*X(i)
        END DO
      END DO
      
END SUBROUTINE KppSolveTRCmplx


! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
SUBROUTINE KppSolveTRCmplxR( JVSR, JVSI, XR, XI )
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!   Complex sparse solve transpose subroutine using indirect addressing
!   (Real and Imaginary parts are used instead of complex data type)     
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

  USE gckpp_Parameters
  USE gckpp_JacobianSP

      INTEGER       ::  i, j
      REAL(kind=dp) ::  JVSR(LU_NONZERO), JVSI(LU_NONZERO), XR(NVAR), XI(NVAR), den

      DO i=1,NVAR
        den   = JVSR(LU_DIAG(i))**2 + JVSI(LU_DIAG(i))**2
        XR(i) = (XR(i)*JVSR(LU_DIAG(i)) + XI(i)*JVSI(LU_DIAG(i)))/den
        XI(i) = (XI(i)*JVSR(LU_DIAG(i)) - XR(i)*JVSI(LU_DIAG(i)))/den
        ! subtract all nonzero elements in row i of JVS from X
        DO j=LU_DIAG(i)+1,LU_CROW(i+1)-1
          XR(LU_ICOL(j)) = XR(LU_ICOL(j))-(JVSR(j)*XR(i) - JVSI(j)*XI(i))
          XI(LU_ICOL(j)) = XI(LU_ICOL(j))-(JVSI(j)*XR(i) + JVSR(j)*XI(i))
        END DO
      END DO

      DO i=NVAR, 1, -1
        ! subtract all nonzero elements in row i of JVS from X
        DO j=LU_CROW(i),LU_DIAG(i)-1
          XR(LU_ICOL(j)) = XR(LU_ICOL(j))-(JVSR(j)*XR(i) - JVSI(j)*XI(i))
          XI(LU_ICOL(j)) = XI(LU_ICOL(j))-(JVSI(j)*XR(i) + JVSR(j)*XI(i))
        END DO
      END DO
      
END SUBROUTINE KppSolveTRCmplxR


!
! Next few commented subroutines perform sparse big linear algebra
!
!! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!SUBROUTINE KppDecompBig( JVS, IP, IER )
!! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!!        Sparse LU factorization
!!        for the Runge Kutta (3n)x(3n) linear system
!! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!
!  USE gckpp_Parameters
!  USE gckpp_JacobianSP
!
!      INTEGER  :: IP3(3), IER, IP(3,NVAR)
!      REAL(kind=dp) :: JVS(3,3,LU_NONZERO), W(3,3,NVAR), a(3,3), E(3,3)
!      INTEGER  :: k, kk, j, jj
!
!      a = 0.0d0
!      IER = 0
!      DO k=1,NVAR
!        DO kk = LU_CROW(k), LU_CROW(k+1)-1
!              W( 1:3,1:3,LU_ICOL(kk) ) = JVS(1:3,1:3,kk)
!        END DO
!        DO kk = LU_CROW(k), LU_DIAG(k)-1
!            j = LU_ICOL(kk)
!            E(1:3,1:3) = JVS( 1:3,1:3,LU_DIAG(j) )
!            ! CALL DGETRF(3,3,E,3,IP3,IER) 
!            CALL FAC3(E,IP3,IER)
!            IF ( IER /= 0 )  RETURN
!            ! a = W(j) / JVS( LU_DIAG(j) )
!            a(1:3,1:3) = W( 1:3,1:3,j )
!            ! CALL DGETRS ('N',3,3,E,3,IP3,a,3,IER) 
!            CALL SOL3('N',E,IP3,a(1,1))
!            CALL SOL3('N',E,IP3,a(1,2))
!            CALL SOL3('N',E,IP3,a(1,3))
!            W(1:3,1:3,j) = a(1:3,1:3)
!            DO jj = LU_DIAG(j)+1, LU_CROW(j+1)-1
!               W( 1:3,1:3,LU_ICOL(jj) ) = W( 1:3,1:3,LU_ICOL(jj) ) &
!                        - MATMUL( a(1:3,1:3) , JVS(1:3,1:3,jj) )
!            END DO
!         END DO
!         DO kk = LU_CROW(k), LU_CROW(k+1)-1
!            JVS(1:3,1:3,kk) = W( 1:3,1:3,LU_ICOL(kk) )
!         END DO
!      END DO
!
!      DO k=1,NVAR
!         ! CALL WGEFA(JVS(1,1,LU_DIAG(k)),3,3,IP(1,k),IER)
!         ! CALL DGETRF(3,3,JVS(1,1,LU_DIAG(k)),3,IP(1,k),IER)
!         CALL FAC3(JVS(1,1,LU_DIAG(k)),IP(1,k),IER)
!         IF ( IER /= 0 )  RETURN
!      END DO 
!      
!END SUBROUTINE KppDecompBig
!
!
!! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!SUBROUTINE KppSolveBig( JVS, IP, X )
!! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!!        Sparse solve subroutine using indirect addressing
!!        for the Runge Kutta (3n)x(3n) linear system
!! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!
!  USE gckpp_Parameters
!  USE gckpp_JacobianSP
!
!      INTEGER  :: i, j, k, m, IP3(3), IP(3,NVAR), IER
!      REAL(kind=dp) :: JVS(3,3,LU_NONZERO), X(3,NVAR), sum(3)
!
!      DO i=1,NVAR
!        DO j = LU_CROW(i), LU_DIAG(i)-1 
!          !X(1:3,i) = X(1:3,i) - MATMUL(JVS(1:3,1:3,j),X(1:3,LU_ICOL(j)));
!          DO k=1,3
!            DO m=1,3
!              X(k,i) = X(k,i) - JVS(k,m,j)*X(m,LU_ICOL(j))
!            END DO
!          END DO
!        END DO  
!      END DO
!
!      DO i=NVAR,1,-1
!        sum(1:3) = X(1:3,i);
!        DO j = LU_DIAG(i)+1, LU_CROW(i+1)-1
!          !sum(1:3) = sum(1:3) - MATMUL(JVS(1:3,1:3,j),X(1:3,LU_ICOL(j)));
!          DO k=1,3
!            DO m=1,3
!              sum(k) = sum(k) - JVS(k,m,j)*X(m,LU_ICOL(j))
!            END DO
!          END DO
!        END DO
!        ! X(i) = sum/JVS(LU_DIAG(i));
!        ! CALL DGETRS ('N',3,1,JVS(1:3,1:3,LU_DIAG(i)),3,IP(1,i),sum,3,0) 
!        ! CALL WGESL('N',JVS(1,1,LU_DIAG(i)),3,3,IP(1,i),sum)
!        CALL SOL3('N',JVS(1,1,LU_DIAG(i)),IP(1,i),sum)
!        X(1:3,i) = sum(1:3)
!      END DO
!      
!END SUBROUTINE KppSolveBig
!
!
!! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!SUBROUTINE KppSolveBigTR( JVS, IP, X )
!! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!!        Big sparse transpose solve using indirect addressing
!! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!
!  USE gckpp_Parameters
!  USE gckpp_JacobianSP
!
!      INTEGER       :: i, j, k, m, IP(3,NVAR)
!      REAL(kind=dp) :: JVS(3,3,LU_NONZERO), X(3,NVAR)
!
!      DO i=1,NVAR
!        ! X(i) = X(i)/JVS(LU_DIAG(i))
!        CALL SOL3('T',JVS(1,1,LU_DIAG(i)),IP(1,i),X(1,i))
!        DO j=LU_DIAG(i)+1,LU_CROW(i+1)-1
!         !X(1:3,LU_ICOL(j)) = X(1:3,LU_ICOL(j)) &
!          !    - MATMUL( TRANSPOSE(JVS(1:3,1:3,j)), X(1:3,i) )
!          DO k=1,3
!            DO m=1,3
!              X(k,LU_ICOL(j)) = X(k,LU_ICOL(j)) - JVS(m,k,j)*X(m,i)
!            END DO
!          END DO
!       END DO
!      END DO
!
!      DO i=NVAR, 1, -1
!        DO j=LU_CROW(i),LU_DIAG(i)-1
!         !X(1:3,LU_ICOL(j)) = X(1:3,LU_ICOL(j)) &
!          !   - MATMUL( TRANSPOSE(JVS(1:3,1:3,j)), X(1:3,i) )
!          DO k=1,3
!            DO m=1,3
!              X(k,LU_ICOL(j)) = X(k,LU_ICOL(j)) - JVS(m,k,j)*X(m,i)
!            END DO
!          END DO
!       END DO
!      END DO
!      
!END SUBROUTINE KppSolveBigTR
!
!
!
!! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!SUBROUTINE FAC3(A,IPVT,INFO)
!! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!!     FAC3 FACTORS THE MATRIX A (3,3) BY
!!           GAUSS ELIMINATION WITH PARTIAL PIVOTING
!!     LINPACK - LIKE 
!!
!!     Remove comments to perform pivoting
!! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!!
!      REAL(kind=dp) :: A(3,3)
!      INTEGER       :: IPVT(3),INFO
!!      INTEGER       :: L
!!      REAL(kind=dp) :: t, dmax, da, TMP(3)
!      REAL(kind=dp), PARAMETER :: ZERO = 0.0, ONE = 1.0
!
!      info = 0
!!      t = TINY(da)
!!      
!!      da = ABS(A(1,1)); L = 1
!!      IF ( ABS(A(2,1))>da ) THEN
!!        da = ABS(A(2,1)); L = 2
!!        IF ( ABS(A(3,1))>da ) THEN
!!          L = 3
!!        END IF  
!!      END IF  
!!      IPVT(1)  = L
!!      IF (L /=1 ) THEN
!!         TMP(1:3) = A(L,1:3)
!!         A(L,1:3) = A(1,1:3)
!!         A(1,1:3) = TMP(1:3)
!!      END IF
!!      IF (ABS(A(1,1)) < t) THEN
!!         info = 1
!!         return
!!      END IF   
!!
!      A(2,1) = A(2,1)/A(1,1)
!      A(2,2) = A(2,2) - A(2,1)*A(1,2)
!      A(2,3) = A(2,3) - A(2,1)*A(1,3)
!      A(3,1) = A(3,1)/A(1,1)
!      A(3,2) = A(3,2) - A(3,1)*A(1,2)
!      A(3,3) = A(3,3) - A(3,1)*A(1,3)
!      
!!      IPVT(2)  = 2
!!      IF (ABS(A(3,2))>ABS(A(2,2))) THEN
!!         IPVT(2)  = 3
!!         TMP(2:3) = A(3,2:3)
!!         A(3,2:3) = A(2,2:3)
!!         A(2,2:3) = TMP(2:3)
!!      END IF
!!      IF (ABS(A(2,2)) < t) THEN
!!         info = 1
!!         return
!!      END IF   
!!      
!      A(3,2)   = A(3,2)/A(2,2)
!      A(3,3)   = A(3,3) - A(3,2)*A(2,3)
!      IPVT(3)  = 3
!      
!END SUBROUTINE FAC3
!
!
!! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!SUBROUTINE SOL3(Trans,A,IPVT,b)
!! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!!     SOL3 solves the system 3x3
!!     A * x = b  or  trans(a) * x = b
!!     using the factors computed by WGEFA.
!!
!!     Trans      = 'N'   to solve  A*x = b ,
!!                = 'T'   to solve  transpose(A)*x = b
!!     LINPACK - LIKE 
!!
!!     Remove comments to use pivoting
!! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!
!      CHARACTER     :: Trans
!      REAL(kind=dp) :: a(3,3),b(3)
!      INTEGER       :: IPVT(3)
!!      INTEGER       :: L
!!      REAL(kind=dp) :: TMP
!      
!      SELECT CASE (Trans)
!
!      CASE ('n','N')  !  Solve  A * x = b
!
!!     Solve  L*y = b
!!         L = IPVT(1)
!!         IF (L /= 1) THEN
!!            TMP = B(1); B(1) = B(L); B(L) = TMP
!!         END IF
!         b(2) = b(2)-A(2,1)*b(1)
!         b(3) = b(3)-A(3,1)*b(1)
!         
!!         L = IPVT(2)
!!         IF (L /= 2) THEN
!!            TMP = B(2); B(2) = B(L); B(L) = TMP
!!         END IF
!         b(3) = b(3)-A(3,2)*b(2)
!
!!     Solve  U*x = y
!         b(3) = b(3)/A(3,3)
!         b(2) = (b(2)-A(2,3)*b(3))/A(2,2)
!         b(1) = (b(1)-A(1,3)*b(3)-A(1,2)*b(2))/A(1,1)
!      
!      
!      CASE ('t','T')  !  Solve transpose(A) * x = b
!
!!      Solve transpose(U)*y = b
!         b(1) = b(1)/A(1,1)
!         b(2) = (b(2)-A(1,2)*b(1))/A(2,2)
!         b(3) = (b(3)-A(1,3)*b(1)-A(2,3)*b(2))/A(3,3)
!
!!      Solve transpose(L)*x = y
!         b(2) = b(2)-A(3,2)*b(3)
!!         L = ipvt(2)
!!         IF (L /= 2) THEN
!!            TMP = B(2); B(2) = B(L); B(L) = TMP
!!         END IF
!         b(1) = b(1)-A(3,1)*b(3)-A(2,1)*b(2)
!!         L = ipvt(1)
!!         IF (L /= 1) THEN
!!            TMP = B(1); B(1) = B(L); B(L) = TMP
!!         END IF
!   
!      END SELECT
!
!END SUBROUTINE SOL3

! End of SPARSE_UTIL function
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~


! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
! 
! KppSolve - sparse back substitution
!   Arguments :
!      JVS       - sparse Jacobian of variables
!      X         - Vector for variables
! 
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

SUBROUTINE KppSolve ( JVS, X )

! JVS - sparse Jacobian of variables
  REAL(kind=dp) :: JVS(LU_NONZERO)
! X - Vector for variables
  REAL(kind=dp) :: X(NVAR)

  X(58) = X(58)-JVS(183)*X(53)
  X(59) = X(59)-JVS(188)*X(48)
  X(60) = X(60)-JVS(193)*X(49)
  X(75) = X(75)-JVS(250)*X(74)
  X(76) = X(76)-JVS(258)*X(70)
  X(91) = X(91)-JVS(337)*X(89)
  X(96) = X(96)-JVS(367)*X(56)-JVS(368)*X(88)
  X(97) = X(97)-JVS(374)*X(57)
  X(98) = X(98)-JVS(381)*X(92)
  X(104) = X(104)-JVS(429)*X(95)
  X(106) = X(106)-JVS(445)*X(67)
  X(110) = X(110)-JVS(475)*X(109)
  X(112) = X(112)-JVS(489)*X(79)
  X(113) = X(113)-JVS(498)*X(84)
  X(116) = X(116)-JVS(523)*X(61)-JVS(524)*X(63)-JVS(525)*X(68)-JVS(526)*X(79)-JVS(527)*X(82)-JVS(528)*X(98)-JVS(529)&
             &*X(100)-JVS(530)*X(112)
  X(117) = X(117)-JVS(553)*X(69)-JVS(554)*X(70)
  X(118) = X(118)-JVS(564)*X(96)-JVS(565)*X(100)-JVS(566)*X(107)-JVS(567)*X(115)
  X(119) = X(119)-JVS(580)*X(77)
  X(121) = X(121)-JVS(595)*X(95)
  X(122) = X(122)-JVS(605)*X(109)
  X(123) = X(123)-JVS(614)*X(81)-JVS(615)*X(110)
  X(126) = X(126)-JVS(639)*X(67)-JVS(640)*X(70)
  X(127) = X(127)-JVS(650)*X(84)-JVS(651)*X(110)
  X(129) = X(129)-JVS(669)*X(71)-JVS(670)*X(82)
  X(130) = X(130)-JVS(684)*X(87)-JVS(685)*X(89)-JVS(686)*X(96)-JVS(687)*X(100)-JVS(688)*X(104)-JVS(689)*X(105)-JVS(690)&
             &*X(120)
  X(131) = X(131)-JVS(708)*X(89)
  X(132) = X(132)-JVS(716)*X(87)-JVS(717)*X(109)
  X(133) = X(133)-JVS(727)*X(54)-JVS(728)*X(78)-JVS(729)*X(124)
  X(134) = X(134)-JVS(742)*X(85)-JVS(743)*X(106)-JVS(744)*X(126)-JVS(745)*X(128)
  X(135) = X(135)-JVS(757)*X(94)-JVS(758)*X(125)
  X(137) = X(137)-JVS(778)*X(77)-JVS(779)*X(101)-JVS(780)*X(124)-JVS(781)*X(136)
  X(138) = X(138)-JVS(793)*X(68)-JVS(794)*X(102)-JVS(795)*X(103)-JVS(796)*X(105)-JVS(797)*X(108)-JVS(798)*X(109)&
             &-JVS(799)*X(110)-JVS(800)*X(111)-JVS(801)*X(114)-JVS(802)*X(122)
  X(139) = X(139)-JVS(813)*X(85)-JVS(814)*X(87)-JVS(815)*X(89)-JVS(816)*X(96)-JVS(817)*X(97)-JVS(818)*X(100)-JVS(819)&
             &*X(104)-JVS(820)*X(111)-JVS(821)*X(113)-JVS(822)*X(114)-JVS(823)*X(120)-JVS(824)*X(121)-JVS(825)*X(125)&
             &-JVS(826)*X(127)-JVS(827)*X(128)-JVS(828)*X(131)-JVS(829)*X(132)-JVS(830)*X(134)-JVS(831)*X(138)
  X(140) = X(140)-JVS(855)*X(93)-JVS(856)*X(107)-JVS(857)*X(109)-JVS(858)*X(115)-JVS(859)*X(136)
  X(141) = X(141)-JVS(868)*X(88)-JVS(869)*X(109)
  X(142) = X(142)-JVS(878)*X(88)-JVS(879)*X(95)-JVS(880)*X(120)-JVS(881)*X(140)-JVS(882)*X(141)
  X(143) = X(143)-JVS(891)*X(1)-JVS(892)*X(66)-JVS(893)*X(68)-JVS(894)*X(73)-JVS(895)*X(77)-JVS(896)*X(80)-JVS(897)&
             &*X(81)-JVS(898)*X(85)-JVS(899)*X(87)-JVS(900)*X(88)-JVS(901)*X(90)-JVS(902)*X(95)-JVS(903)*X(96)-JVS(904)&
             &*X(97)-JVS(905)*X(101)-JVS(906)*X(104)-JVS(907)*X(107)-JVS(908)*X(108)-JVS(909)*X(109)-JVS(910)*X(110)&
             &-JVS(911)*X(113)-JVS(912)*X(115)-JVS(913)*X(117)-JVS(914)*X(118)-JVS(915)*X(119)-JVS(916)*X(120)-JVS(917)&
             &*X(121)-JVS(918)*X(122)-JVS(919)*X(123)-JVS(920)*X(124)-JVS(921)*X(125)-JVS(922)*X(126)-JVS(923)*X(127)&
             &-JVS(924)*X(128)-JVS(925)*X(129)-JVS(926)*X(130)-JVS(927)*X(131)-JVS(928)*X(132)-JVS(929)*X(133)-JVS(930)&
             &*X(134)-JVS(931)*X(135)-JVS(932)*X(136)-JVS(933)*X(137)-JVS(934)*X(138)-JVS(935)*X(140)-JVS(936)*X(141)&
             &-JVS(937)*X(142)
  X(144) = X(144)-JVS(960)*X(90)-JVS(961)*X(122)-JVS(962)*X(138)-JVS(963)*X(142)
  X(145) = X(145)-JVS(976)*X(94)-JVS(977)*X(95)-JVS(978)*X(125)-JVS(979)*X(141)
  X(146) = X(146)-JVS(990)*X(94)-JVS(991)*X(120)-JVS(992)*X(125)-JVS(993)*X(140)-JVS(994)*X(141)-JVS(995)*X(145)
  X(147) = X(147)-JVS(1005)*X(81)-JVS(1006)*X(89)-JVS(1007)*X(90)-JVS(1008)*X(91)-JVS(1009)*X(96)-JVS(1010)*X(97)&
             &-JVS(1011)*X(100)-JVS(1012)*X(101)-JVS(1013)*X(105)-JVS(1014)*X(119)-JVS(1015)*X(120)-JVS(1016)*X(121)&
             &-JVS(1017)*X(122)-JVS(1018)*X(123)-JVS(1019)*X(124)-JVS(1020)*X(131)-JVS(1021)*X(134)-JVS(1022)*X(136)&
             &-JVS(1023)*X(137)-JVS(1024)*X(138)-JVS(1025)*X(140)-JVS(1026)*X(141)-JVS(1027)*X(142)-JVS(1028)*X(144)&
             &-JVS(1029)*X(145)-JVS(1030)*X(146)
  X(148) = X(148)-JVS(1047)*X(63)-JVS(1048)*X(86)-JVS(1049)*X(98)-JVS(1050)*X(100)-JVS(1051)*X(131)-JVS(1052)*X(135)&
             &-JVS(1053)*X(141)-JVS(1054)*X(142)-JVS(1055)*X(145)-JVS(1056)*X(146)
  X(149) = X(149)-JVS(1067)*X(71)-JVS(1068)*X(78)-JVS(1069)*X(81)-JVS(1070)*X(103)-JVS(1071)*X(110)-JVS(1072)*X(123)&
             &-JVS(1073)*X(124)-JVS(1074)*X(127)-JVS(1075)*X(128)-JVS(1076)*X(129)-JVS(1077)*X(133)-JVS(1078)*X(135)&
             &-JVS(1079)*X(136)-JVS(1080)*X(138)-JVS(1081)*X(140)-JVS(1082)*X(142)-JVS(1083)*X(145)-JVS(1084)*X(146)&
             &-JVS(1085)*X(148)
  X(150) = X(150)-JVS(1102)*X(93)-JVS(1103)*X(107)-JVS(1104)*X(115)-JVS(1105)*X(124)-JVS(1106)*X(125)-JVS(1107)*X(128)&
             &-JVS(1108)*X(134)-JVS(1109)*X(135)-JVS(1110)*X(136)-JVS(1111)*X(140)-JVS(1112)*X(145)-JVS(1113)*X(146)&
             &-JVS(1114)*X(148)
  X(151) = X(151)-JVS(1127)*X(88)-JVS(1128)*X(109)-JVS(1129)*X(122)-JVS(1130)*X(138)-JVS(1131)*X(140)-JVS(1132)*X(141)&
             &-JVS(1133)*X(142)
  X(152) = X(152)-JVS(1145)*X(88)-JVS(1146)*X(102)-JVS(1147)*X(109)-JVS(1148)*X(138)-JVS(1149)*X(140)-JVS(1150)*X(141)&
             &-JVS(1151)*X(142)-JVS(1152)*X(151)
  X(153) = X(153)-JVS(1163)*X(87)-JVS(1164)*X(91)-JVS(1165)*X(92)-JVS(1166)*X(93)-JVS(1167)*X(107)-JVS(1168)*X(115)&
             &-JVS(1169)*X(119)-JVS(1170)*X(128)-JVS(1171)*X(131)-JVS(1172)*X(132)-JVS(1173)*X(134)-JVS(1174)*X(136)&
             &-JVS(1175)*X(137)-JVS(1176)*X(140)-JVS(1177)*X(141)-JVS(1178)*X(142)-JVS(1179)*X(144)-JVS(1180)*X(146)&
             &-JVS(1181)*X(148)-JVS(1182)*X(150)-JVS(1183)*X(151)-JVS(1184)*X(152)
  X(154) = X(154)-JVS(1196)*X(50)-JVS(1197)*X(51)-JVS(1198)*X(52)-JVS(1199)*X(55)-JVS(1200)*X(70)-JVS(1201)*X(74)&
             &-JVS(1202)*X(75)-JVS(1203)*X(76)-JVS(1204)*X(79)-JVS(1205)*X(82)-JVS(1206)*X(106)-JVS(1207)*X(112)-JVS(1208)&
             &*X(126)-JVS(1209)*X(143)-JVS(1210)*X(144)-JVS(1211)*X(145)-JVS(1212)*X(146)-JVS(1213)*X(147)-JVS(1214)*X(148)&
             &-JVS(1215)*X(149)-JVS(1216)*X(150)-JVS(1217)*X(151)-JVS(1218)*X(152)-JVS(1219)*X(153)
  X(155) = X(155)-JVS(1231)*X(62)-JVS(1232)*X(65)-JVS(1233)*X(77)-JVS(1234)*X(85)-JVS(1235)*X(87)-JVS(1236)*X(101)&
             &-JVS(1237)*X(104)-JVS(1238)*X(106)-JVS(1239)*X(109)-JVS(1240)*X(114)-JVS(1241)*X(117)-JVS(1242)*X(119)&
             &-JVS(1243)*X(123)-JVS(1244)*X(124)-JVS(1245)*X(125)-JVS(1246)*X(126)-JVS(1247)*X(127)-JVS(1248)*X(128)&
             &-JVS(1249)*X(129)-JVS(1250)*X(131)-JVS(1251)*X(132)-JVS(1252)*X(133)-JVS(1253)*X(134)-JVS(1254)*X(135)&
             &-JVS(1255)*X(136)-JVS(1256)*X(137)-JVS(1257)*X(138)-JVS(1258)*X(139)-JVS(1259)*X(140)-JVS(1260)*X(141)&
             &-JVS(1261)*X(142)-JVS(1262)*X(144)-JVS(1263)*X(145)-JVS(1264)*X(146)-JVS(1265)*X(147)-JVS(1266)*X(148)&
             &-JVS(1267)*X(149)-JVS(1268)*X(150)-JVS(1269)*X(151)-JVS(1270)*X(152)-JVS(1271)*X(153)-JVS(1272)*X(154)
  X(156) = X(156)-JVS(1283)*X(70)-JVS(1284)*X(82)-JVS(1285)*X(89)-JVS(1286)*X(90)-JVS(1287)*X(91)-JVS(1288)*X(96)&
             &-JVS(1289)*X(101)-JVS(1290)*X(102)-JVS(1291)*X(103)-JVS(1292)*X(105)-JVS(1293)*X(106)-JVS(1294)*X(109)&
             &-JVS(1295)*X(110)-JVS(1296)*X(111)-JVS(1297)*X(113)-JVS(1298)*X(114)-JVS(1299)*X(118)-JVS(1300)*X(120)&
             &-JVS(1301)*X(121)-JVS(1302)*X(122)-JVS(1303)*X(124)-JVS(1304)*X(125)-JVS(1305)*X(126)-JVS(1306)*X(127)&
             &-JVS(1307)*X(130)-JVS(1308)*X(131)-JVS(1309)*X(132)-JVS(1310)*X(136)-JVS(1311)*X(137)-JVS(1312)*X(138)&
             &-JVS(1313)*X(139)-JVS(1314)*X(140)-JVS(1315)*X(141)-JVS(1316)*X(142)-JVS(1317)*X(143)-JVS(1318)*X(144)&
             &-JVS(1319)*X(145)-JVS(1320)*X(146)-JVS(1321)*X(147)-JVS(1322)*X(148)-JVS(1323)*X(149)-JVS(1324)*X(150)&
             &-JVS(1325)*X(151)-JVS(1326)*X(152)-JVS(1327)*X(153)-JVS(1328)*X(154)-JVS(1329)*X(155)
  X(157) = X(157)-JVS(1339)*X(92)-JVS(1340)*X(95)-JVS(1341)*X(101)-JVS(1342)*X(109)-JVS(1343)*X(110)-JVS(1344)*X(112)&
             &-JVS(1345)*X(120)-JVS(1346)*X(124)-JVS(1347)*X(131)-JVS(1348)*X(133)-JVS(1349)*X(136)-JVS(1350)*X(137)&
             &-JVS(1351)*X(140)-JVS(1352)*X(141)-JVS(1353)*X(142)-JVS(1354)*X(151)-JVS(1355)*X(152)-JVS(1356)*X(153)&
             &-JVS(1357)*X(154)-JVS(1358)*X(155)-JVS(1359)*X(156)
  X(158) = X(158)-JVS(1368)*X(62)-JVS(1369)*X(65)-JVS(1370)*X(66)-JVS(1371)*X(68)-JVS(1372)*X(73)-JVS(1373)*X(106)&
             &-JVS(1374)*X(107)-JVS(1375)*X(109)-JVS(1376)*X(110)-JVS(1377)*X(113)-JVS(1378)*X(114)-JVS(1379)*X(115)&
             &-JVS(1380)*X(117)-JVS(1381)*X(119)-JVS(1382)*X(120)-JVS(1383)*X(123)-JVS(1384)*X(125)-JVS(1385)*X(126)&
             &-JVS(1386)*X(127)-JVS(1387)*X(128)-JVS(1388)*X(129)-JVS(1389)*X(131)-JVS(1390)*X(132)-JVS(1391)*X(133)&
             &-JVS(1392)*X(134)-JVS(1393)*X(135)-JVS(1394)*X(136)-JVS(1395)*X(137)-JVS(1396)*X(138)-JVS(1397)*X(140)&
             &-JVS(1398)*X(141)-JVS(1399)*X(142)-JVS(1400)*X(144)-JVS(1401)*X(145)-JVS(1402)*X(146)-JVS(1403)*X(147)&
             &-JVS(1404)*X(148)-JVS(1405)*X(149)-JVS(1406)*X(150)-JVS(1407)*X(151)-JVS(1408)*X(152)-JVS(1409)*X(153)&
             &-JVS(1410)*X(154)-JVS(1411)*X(155)-JVS(1412)*X(156)-JVS(1413)*X(157)
  X(159) = X(159)-JVS(1421)*X(58)-JVS(1422)*X(59)-JVS(1423)*X(60)-JVS(1424)*X(66)-JVS(1425)*X(67)-JVS(1426)*X(69)&
             &-JVS(1427)*X(71)-JVS(1428)*X(72)-JVS(1429)*X(73)-JVS(1430)*X(78)-JVS(1431)*X(81)-JVS(1432)*X(83)-JVS(1433)&
             &*X(84)-JVS(1434)*X(86)-JVS(1435)*X(87)-JVS(1436)*X(88)-JVS(1437)*X(89)-JVS(1438)*X(90)-JVS(1439)*X(91)&
             &-JVS(1440)*X(92)-JVS(1441)*X(93)-JVS(1442)*X(94)-JVS(1443)*X(95)-JVS(1444)*X(96)-JVS(1445)*X(97)-JVS(1446)&
             &*X(98)-JVS(1447)*X(99)-JVS(1448)*X(100)-JVS(1449)*X(102)-JVS(1450)*X(103)-JVS(1451)*X(105)-JVS(1452)*X(107)&
             &-JVS(1453)*X(108)-JVS(1454)*X(109)-JVS(1455)*X(110)-JVS(1456)*X(111)-JVS(1457)*X(112)-JVS(1458)*X(114)&
             &-JVS(1459)*X(115)-JVS(1460)*X(117)-JVS(1461)*X(118)-JVS(1462)*X(119)-JVS(1463)*X(120)-JVS(1464)*X(121)&
             &-JVS(1465)*X(122)-JVS(1466)*X(123)-JVS(1467)*X(124)-JVS(1468)*X(125)-JVS(1469)*X(126)-JVS(1470)*X(127)&
             &-JVS(1471)*X(128)-JVS(1472)*X(129)-JVS(1473)*X(130)-JVS(1474)*X(131)-JVS(1475)*X(132)-JVS(1476)*X(133)&
             &-JVS(1477)*X(134)-JVS(1478)*X(135)-JVS(1479)*X(136)-JVS(1480)*X(137)-JVS(1481)*X(138)-JVS(1482)*X(139)&
             &-JVS(1483)*X(140)-JVS(1484)*X(141)-JVS(1485)*X(142)-JVS(1486)*X(143)-JVS(1487)*X(144)-JVS(1488)*X(145)&
             &-JVS(1489)*X(146)-JVS(1490)*X(147)-JVS(1491)*X(148)-JVS(1492)*X(149)-JVS(1493)*X(150)-JVS(1494)*X(151)&
             &-JVS(1495)*X(152)-JVS(1496)*X(153)-JVS(1497)*X(154)-JVS(1498)*X(155)-JVS(1499)*X(156)-JVS(1500)*X(157)&
             &-JVS(1501)*X(158)
  X(160) = X(160)-JVS(1508)*X(69)-JVS(1509)*X(84)-JVS(1510)*X(86)-JVS(1511)*X(93)-JVS(1512)*X(94)-JVS(1513)*X(98)&
             &-JVS(1514)*X(99)-JVS(1515)*X(100)-JVS(1516)*X(107)-JVS(1517)*X(115)-JVS(1518)*X(117)-JVS(1519)*X(119)&
             &-JVS(1520)*X(123)-JVS(1521)*X(125)-JVS(1522)*X(126)-JVS(1523)*X(127)-JVS(1524)*X(128)-JVS(1525)*X(131)&
             &-JVS(1526)*X(132)-JVS(1527)*X(135)-JVS(1528)*X(136)-JVS(1529)*X(137)-JVS(1530)*X(140)-JVS(1531)*X(141)&
             &-JVS(1532)*X(142)-JVS(1533)*X(144)-JVS(1534)*X(145)-JVS(1535)*X(146)-JVS(1536)*X(148)-JVS(1537)*X(150)&
             &-JVS(1538)*X(151)-JVS(1539)*X(152)-JVS(1540)*X(153)-JVS(1541)*X(154)-JVS(1542)*X(155)-JVS(1543)*X(156)&
             &-JVS(1544)*X(157)-JVS(1545)*X(158)-JVS(1546)*X(159)
  X(161) = X(161)-JVS(1552)*X(48)-JVS(1553)*X(49)-JVS(1554)*X(50)-JVS(1555)*X(51)-JVS(1556)*X(52)-JVS(1557)*X(53)&
             &-JVS(1558)*X(56)-JVS(1559)*X(63)-JVS(1560)*X(64)-JVS(1561)*X(65)-JVS(1562)*X(67)-JVS(1563)*X(68)-JVS(1564)&
             &*X(69)-JVS(1565)*X(70)-JVS(1566)*X(71)-JVS(1567)*X(72)-JVS(1568)*X(73)-JVS(1569)*X(74)-JVS(1570)*X(75)&
             &-JVS(1571)*X(76)-JVS(1572)*X(77)-JVS(1573)*X(78)-JVS(1574)*X(79)-JVS(1575)*X(80)-JVS(1576)*X(81)-JVS(1577)&
             &*X(82)-JVS(1578)*X(83)-JVS(1579)*X(84)-JVS(1580)*X(85)-JVS(1581)*X(86)-JVS(1582)*X(87)-JVS(1583)*X(88)&
             &-JVS(1584)*X(89)-JVS(1585)*X(90)-JVS(1586)*X(91)-JVS(1587)*X(92)-JVS(1588)*X(93)-JVS(1589)*X(94)-JVS(1590)&
             &*X(95)-JVS(1591)*X(96)-JVS(1592)*X(97)-JVS(1593)*X(98)-JVS(1594)*X(99)-JVS(1595)*X(100)-JVS(1596)*X(101)&
             &-JVS(1597)*X(102)-JVS(1598)*X(103)-JVS(1599)*X(104)-JVS(1600)*X(105)-JVS(1601)*X(106)-JVS(1602)*X(107)&
             &-JVS(1603)*X(108)-JVS(1604)*X(109)-JVS(1605)*X(110)-JVS(1606)*X(111)-JVS(1607)*X(112)-JVS(1608)*X(113)&
             &-JVS(1609)*X(114)-JVS(1610)*X(115)-JVS(1611)*X(116)-JVS(1612)*X(117)-JVS(1613)*X(118)-JVS(1614)*X(119)&
             &-JVS(1615)*X(120)-JVS(1616)*X(121)-JVS(1617)*X(122)-JVS(1618)*X(123)-JVS(1619)*X(124)-JVS(1620)*X(125)&
             &-JVS(1621)*X(126)-JVS(1622)*X(127)-JVS(1623)*X(128)-JVS(1624)*X(129)-JVS(1625)*X(130)-JVS(1626)*X(131)&
             &-JVS(1627)*X(132)-JVS(1628)*X(133)-JVS(1629)*X(134)-JVS(1630)*X(135)-JVS(1631)*X(136)-JVS(1632)*X(137)&
             &-JVS(1633)*X(138)-JVS(1634)*X(139)-JVS(1635)*X(140)-JVS(1636)*X(141)-JVS(1637)*X(142)-JVS(1638)*X(143)&
             &-JVS(1639)*X(144)-JVS(1640)*X(145)-JVS(1641)*X(146)-JVS(1642)*X(147)-JVS(1643)*X(148)-JVS(1644)*X(149)&
             &-JVS(1645)*X(150)-JVS(1646)*X(151)-JVS(1647)*X(152)-JVS(1648)*X(153)-JVS(1649)*X(154)-JVS(1650)*X(155)&
             &-JVS(1651)*X(156)-JVS(1652)*X(157)-JVS(1653)*X(158)-JVS(1654)*X(159)-JVS(1655)*X(160)
  X(162) = X(162)-JVS(1660)*X(54)-JVS(1661)*X(55)-JVS(1662)*X(57)-JVS(1663)*X(61)-JVS(1664)*X(62)-JVS(1665)*X(64)&
             &-JVS(1666)*X(66)-JVS(1667)*X(72)-JVS(1668)*X(79)-JVS(1669)*X(80)-JVS(1670)*X(84)-JVS(1671)*X(93)-JVS(1672)&
             &*X(94)-JVS(1673)*X(95)-JVS(1674)*X(96)-JVS(1675)*X(97)-JVS(1676)*X(98)-JVS(1677)*X(100)-JVS(1678)*X(101)&
             &-JVS(1679)*X(102)-JVS(1680)*X(103)-JVS(1681)*X(104)-JVS(1682)*X(105)-JVS(1683)*X(107)-JVS(1684)*X(108)&
             &-JVS(1685)*X(109)-JVS(1686)*X(110)-JVS(1687)*X(111)-JVS(1688)*X(112)-JVS(1689)*X(113)-JVS(1690)*X(114)&
             &-JVS(1691)*X(115)-JVS(1692)*X(116)-JVS(1693)*X(117)-JVS(1694)*X(118)-JVS(1695)*X(119)-JVS(1696)*X(120)&
             &-JVS(1697)*X(121)-JVS(1698)*X(122)-JVS(1699)*X(123)-JVS(1700)*X(124)-JVS(1701)*X(125)-JVS(1702)*X(126)&
             &-JVS(1703)*X(127)-JVS(1704)*X(128)-JVS(1705)*X(129)-JVS(1706)*X(130)-JVS(1707)*X(131)-JVS(1708)*X(132)&
             &-JVS(1709)*X(133)-JVS(1710)*X(134)-JVS(1711)*X(135)-JVS(1712)*X(136)-JVS(1713)*X(137)-JVS(1714)*X(138)&
             &-JVS(1715)*X(139)-JVS(1716)*X(140)-JVS(1717)*X(141)-JVS(1718)*X(142)-JVS(1719)*X(143)-JVS(1720)*X(144)&
             &-JVS(1721)*X(145)-JVS(1722)*X(146)-JVS(1723)*X(147)-JVS(1724)*X(148)-JVS(1725)*X(149)-JVS(1726)*X(150)&
             &-JVS(1727)*X(151)-JVS(1728)*X(152)-JVS(1729)*X(153)-JVS(1730)*X(154)-JVS(1731)*X(155)-JVS(1732)*X(156)&
             &-JVS(1733)*X(157)-JVS(1734)*X(158)-JVS(1735)*X(159)-JVS(1736)*X(160)-JVS(1737)*X(161)
  X(163) = X(163)-JVS(1741)*X(58)-JVS(1742)*X(59)-JVS(1743)*X(60)-JVS(1744)*X(64)-JVS(1745)*X(96)-JVS(1746)*X(97)&
             &-JVS(1747)*X(98)-JVS(1748)*X(100)-JVS(1749)*X(102)-JVS(1750)*X(103)-JVS(1751)*X(105)-JVS(1752)*X(107)&
             &-JVS(1753)*X(108)-JVS(1754)*X(109)-JVS(1755)*X(110)-JVS(1756)*X(111)-JVS(1757)*X(112)-JVS(1758)*X(114)&
             &-JVS(1759)*X(115)-JVS(1760)*X(117)-JVS(1761)*X(119)-JVS(1762)*X(121)-JVS(1763)*X(122)-JVS(1764)*X(123)&
             &-JVS(1765)*X(124)-JVS(1766)*X(125)-JVS(1767)*X(126)-JVS(1768)*X(127)-JVS(1769)*X(128)-JVS(1770)*X(129)&
             &-JVS(1771)*X(131)-JVS(1772)*X(132)-JVS(1773)*X(133)-JVS(1774)*X(134)-JVS(1775)*X(135)-JVS(1776)*X(136)&
             &-JVS(1777)*X(137)-JVS(1778)*X(138)-JVS(1779)*X(140)-JVS(1780)*X(141)-JVS(1781)*X(142)-JVS(1782)*X(144)&
             &-JVS(1783)*X(145)-JVS(1784)*X(146)-JVS(1785)*X(148)-JVS(1786)*X(150)-JVS(1787)*X(151)-JVS(1788)*X(152)&
             &-JVS(1789)*X(153)-JVS(1790)*X(154)-JVS(1791)*X(155)-JVS(1792)*X(156)-JVS(1793)*X(157)-JVS(1794)*X(158)&
             &-JVS(1795)*X(159)-JVS(1796)*X(160)-JVS(1797)*X(161)-JVS(1798)*X(162)
  X(164) = X(164)-JVS(1801)*X(61)-JVS(1802)*X(62)-JVS(1803)*X(63)-JVS(1804)*X(66)-JVS(1805)*X(68)-JVS(1806)*X(72)&
             &-JVS(1807)*X(79)-JVS(1808)*X(82)-JVS(1809)*X(97)-JVS(1810)*X(101)-JVS(1811)*X(102)-JVS(1812)*X(103)-JVS(1813)&
             &*X(104)-JVS(1814)*X(105)-JVS(1815)*X(108)-JVS(1816)*X(109)-JVS(1817)*X(110)-JVS(1818)*X(111)-JVS(1819)*X(112)&
             &-JVS(1820)*X(114)-JVS(1821)*X(115)-JVS(1822)*X(116)-JVS(1823)*X(118)-JVS(1824)*X(119)-JVS(1825)*X(121)&
             &-JVS(1826)*X(122)-JVS(1827)*X(124)-JVS(1828)*X(130)-JVS(1829)*X(131)-JVS(1830)*X(132)-JVS(1831)*X(136)&
             &-JVS(1832)*X(137)-JVS(1833)*X(138)-JVS(1834)*X(139)-JVS(1835)*X(140)-JVS(1836)*X(141)-JVS(1837)*X(142)&
             &-JVS(1838)*X(143)-JVS(1839)*X(144)-JVS(1840)*X(145)-JVS(1841)*X(146)-JVS(1842)*X(147)-JVS(1843)*X(148)&
             &-JVS(1844)*X(149)-JVS(1845)*X(150)-JVS(1846)*X(151)-JVS(1847)*X(152)-JVS(1848)*X(153)-JVS(1849)*X(154)&
             &-JVS(1850)*X(155)-JVS(1851)*X(156)-JVS(1852)*X(157)-JVS(1853)*X(158)-JVS(1854)*X(159)-JVS(1855)*X(160)&
             &-JVS(1856)*X(161)-JVS(1857)*X(162)-JVS(1858)*X(163)
  X(164) = X(164)/JVS(1859)
  X(163) = (X(163)-JVS(1800)*X(164))/(JVS(1799))
  X(162) = (X(162)-JVS(1739)*X(163)-JVS(1740)*X(164))/(JVS(1738))
  X(161) = (X(161)-JVS(1657)*X(162)-JVS(1658)*X(163)-JVS(1659)*X(164))/(JVS(1656))
  X(160) = (X(160)-JVS(1548)*X(161)-JVS(1549)*X(162)-JVS(1550)*X(163)-JVS(1551)*X(164))/(JVS(1547))
  X(159) = (X(159)-JVS(1503)*X(160)-JVS(1504)*X(161)-JVS(1505)*X(162)-JVS(1506)*X(163)-JVS(1507)*X(164))/(JVS(1502))
  X(158) = (X(158)-JVS(1415)*X(159)-JVS(1416)*X(160)-JVS(1417)*X(161)-JVS(1418)*X(162)-JVS(1419)*X(163)-JVS(1420)&
             &*X(164))/(JVS(1414))
  X(157) = (X(157)-JVS(1361)*X(158)-JVS(1362)*X(159)-JVS(1363)*X(160)-JVS(1364)*X(161)-JVS(1365)*X(162)-JVS(1366)*X(163)&
             &-JVS(1367)*X(164))/(JVS(1360))
  X(156) = (X(156)-JVS(1331)*X(157)-JVS(1332)*X(158)-JVS(1333)*X(159)-JVS(1334)*X(160)-JVS(1335)*X(161)-JVS(1336)*X(162)&
             &-JVS(1337)*X(163)-JVS(1338)*X(164))/(JVS(1330))
  X(155) = (X(155)-JVS(1274)*X(156)-JVS(1275)*X(157)-JVS(1276)*X(158)-JVS(1277)*X(159)-JVS(1278)*X(160)-JVS(1279)*X(161)&
             &-JVS(1280)*X(162)-JVS(1281)*X(163)-JVS(1282)*X(164))/(JVS(1273))
  X(154) = (X(154)-JVS(1221)*X(155)-JVS(1222)*X(156)-JVS(1223)*X(157)-JVS(1224)*X(158)-JVS(1225)*X(159)-JVS(1226)*X(160)&
             &-JVS(1227)*X(161)-JVS(1228)*X(162)-JVS(1229)*X(163)-JVS(1230)*X(164))/(JVS(1220))
  X(153) = (X(153)-JVS(1186)*X(154)-JVS(1187)*X(155)-JVS(1188)*X(156)-JVS(1189)*X(157)-JVS(1190)*X(158)-JVS(1191)*X(159)&
             &-JVS(1192)*X(161)-JVS(1193)*X(162)-JVS(1194)*X(163)-JVS(1195)*X(164))/(JVS(1185))
  X(152) = (X(152)-JVS(1154)*X(155)-JVS(1155)*X(156)-JVS(1156)*X(157)-JVS(1157)*X(158)-JVS(1158)*X(159)-JVS(1159)*X(161)&
             &-JVS(1160)*X(162)-JVS(1161)*X(163)-JVS(1162)*X(164))/(JVS(1153))
  X(151) = (X(151)-JVS(1135)*X(152)-JVS(1136)*X(155)-JVS(1137)*X(156)-JVS(1138)*X(157)-JVS(1139)*X(158)-JVS(1140)*X(159)&
             &-JVS(1141)*X(161)-JVS(1142)*X(162)-JVS(1143)*X(163)-JVS(1144)*X(164))/(JVS(1134))
  X(150) = (X(150)-JVS(1116)*X(151)-JVS(1117)*X(153)-JVS(1118)*X(154)-JVS(1119)*X(155)-JVS(1120)*X(157)-JVS(1121)*X(158)&
             &-JVS(1122)*X(159)-JVS(1123)*X(161)-JVS(1124)*X(162)-JVS(1125)*X(163)-JVS(1126)*X(164))/(JVS(1115))
  X(149) = (X(149)-JVS(1087)*X(150)-JVS(1088)*X(151)-JVS(1089)*X(152)-JVS(1090)*X(153)-JVS(1091)*X(154)-JVS(1092)*X(155)&
             &-JVS(1093)*X(156)-JVS(1094)*X(157)-JVS(1095)*X(158)-JVS(1096)*X(159)-JVS(1097)*X(160)-JVS(1098)*X(161)&
             &-JVS(1099)*X(162)-JVS(1100)*X(163)-JVS(1101)*X(164))/(JVS(1086))
  X(148) = (X(148)-JVS(1058)*X(150)-JVS(1059)*X(151)-JVS(1060)*X(155)-JVS(1061)*X(157)-JVS(1062)*X(158)-JVS(1063)*X(159)&
             &-JVS(1064)*X(161)-JVS(1065)*X(163)-JVS(1066)*X(164))/(JVS(1057))
  X(147) = (X(147)-JVS(1032)*X(148)-JVS(1033)*X(150)-JVS(1034)*X(151)-JVS(1035)*X(152)-JVS(1036)*X(153)-JVS(1037)*X(154)&
             &-JVS(1038)*X(155)-JVS(1039)*X(156)-JVS(1040)*X(157)-JVS(1041)*X(158)-JVS(1042)*X(159)-JVS(1043)*X(161)&
             &-JVS(1044)*X(162)-JVS(1045)*X(163)-JVS(1046)*X(164))/(JVS(1031))
  X(146) = (X(146)-JVS(997)*X(151)-JVS(998)*X(155)-JVS(999)*X(157)-JVS(1000)*X(158)-JVS(1001)*X(159)-JVS(1002)*X(161)&
             &-JVS(1003)*X(163)-JVS(1004)*X(164))/(JVS(996))
  X(145) = (X(145)-JVS(981)*X(146)-JVS(982)*X(151)-JVS(983)*X(155)-JVS(984)*X(157)-JVS(985)*X(158)-JVS(986)*X(159)&
             &-JVS(987)*X(161)-JVS(988)*X(163)-JVS(989)*X(164))/(JVS(980))
  X(144) = (X(144)-JVS(965)*X(151)-JVS(966)*X(152)-JVS(967)*X(155)-JVS(968)*X(156)-JVS(969)*X(157)-JVS(970)*X(158)&
             &-JVS(971)*X(159)-JVS(972)*X(161)-JVS(973)*X(162)-JVS(974)*X(163)-JVS(975)*X(164))/(JVS(964))
  X(143) = (X(143)-JVS(939)*X(144)-JVS(940)*X(145)-JVS(941)*X(146)-JVS(942)*X(147)-JVS(943)*X(148)-JVS(944)*X(149)&
             &-JVS(945)*X(150)-JVS(946)*X(151)-JVS(947)*X(152)-JVS(948)*X(153)-JVS(949)*X(154)-JVS(950)*X(155)-JVS(951)&
             &*X(156)-JVS(952)*X(157)-JVS(953)*X(158)-JVS(954)*X(159)-JVS(955)*X(160)-JVS(956)*X(161)-JVS(957)*X(162)&
             &-JVS(958)*X(163)-JVS(959)*X(164))/(JVS(938))
  X(142) = (X(142)-JVS(884)*X(155)-JVS(885)*X(157)-JVS(886)*X(158)-JVS(887)*X(159)-JVS(888)*X(161)-JVS(889)*X(163)&
             &-JVS(890)*X(164))/(JVS(883))
  X(141) = (X(141)-JVS(871)*X(155)-JVS(872)*X(157)-JVS(873)*X(158)-JVS(874)*X(159)-JVS(875)*X(161)-JVS(876)*X(163)&
             &-JVS(877)*X(164))/(JVS(870))
  X(140) = (X(140)-JVS(861)*X(155)-JVS(862)*X(157)-JVS(863)*X(158)-JVS(864)*X(159)-JVS(865)*X(161)-JVS(866)*X(163)&
             &-JVS(867)*X(164))/(JVS(860))
  X(139) = (X(139)-JVS(833)*X(140)-JVS(834)*X(141)-JVS(835)*X(142)-JVS(836)*X(144)-JVS(837)*X(145)-JVS(838)*X(146)&
             &-JVS(839)*X(147)-JVS(840)*X(148)-JVS(841)*X(150)-JVS(842)*X(151)-JVS(843)*X(152)-JVS(844)*X(153)-JVS(845)&
             &*X(154)-JVS(846)*X(155)-JVS(847)*X(156)-JVS(848)*X(157)-JVS(849)*X(158)-JVS(850)*X(159)-JVS(851)*X(161)&
             &-JVS(852)*X(162)-JVS(853)*X(163)-JVS(854)*X(164))/(JVS(832))
  X(138) = (X(138)-JVS(804)*X(142)-JVS(805)*X(151)-JVS(806)*X(152)-JVS(807)*X(156)-JVS(808)*X(157)-JVS(809)*X(161)&
             &-JVS(810)*X(162)-JVS(811)*X(163)-JVS(812)*X(164))/(JVS(803))
  X(137) = (X(137)-JVS(783)*X(140)-JVS(784)*X(151)-JVS(785)*X(155)-JVS(786)*X(157)-JVS(787)*X(158)-JVS(788)*X(159)&
             &-JVS(789)*X(161)-JVS(790)*X(162)-JVS(791)*X(163)-JVS(792)*X(164))/(JVS(782))
  X(136) = (X(136)-JVS(771)*X(140)-JVS(772)*X(155)-JVS(773)*X(157)-JVS(774)*X(158)-JVS(775)*X(161)-JVS(776)*X(163)&
             &-JVS(777)*X(164))/(JVS(770))
  X(135) = (X(135)-JVS(760)*X(145)-JVS(761)*X(146)-JVS(762)*X(150)-JVS(763)*X(151)-JVS(764)*X(155)-JVS(765)*X(158)&
             &-JVS(766)*X(159)-JVS(767)*X(161)-JVS(768)*X(163)-JVS(769)*X(164))/(JVS(759))
  X(134) = (X(134)-JVS(747)*X(148)-JVS(748)*X(150)-JVS(749)*X(153)-JVS(750)*X(154)-JVS(751)*X(155)-JVS(752)*X(158)&
             &-JVS(753)*X(159)-JVS(754)*X(161)-JVS(755)*X(163)-JVS(756)*X(164))/(JVS(746))
  X(133) = (X(133)-JVS(731)*X(136)-JVS(732)*X(152)-JVS(733)*X(153)-JVS(734)*X(155)-JVS(735)*X(158)-JVS(736)*X(159)&
             &-JVS(737)*X(160)-JVS(738)*X(161)-JVS(739)*X(162)-JVS(740)*X(163)-JVS(741)*X(164))/(JVS(730))
  X(132) = (X(132)-JVS(719)*X(152)-JVS(720)*X(155)-JVS(721)*X(157)-JVS(722)*X(158)-JVS(723)*X(159)-JVS(724)*X(161)&
             &-JVS(725)*X(163)-JVS(726)*X(164))/(JVS(718))
  X(131) = (X(131)-JVS(710)*X(142)-JVS(711)*X(155)-JVS(712)*X(158)-JVS(713)*X(159)-JVS(714)*X(161)-JVS(715)*X(163))&
             &/(JVS(709))
  X(130) = (X(130)-JVS(692)*X(131)-JVS(693)*X(132)-JVS(694)*X(138)-JVS(695)*X(140)-JVS(696)*X(141)-JVS(697)*X(142)&
             &-JVS(698)*X(145)-JVS(699)*X(146)-JVS(700)*X(155)-JVS(701)*X(156)-JVS(702)*X(157)-JVS(703)*X(158)-JVS(704)&
             &*X(159)-JVS(705)*X(161)-JVS(706)*X(162)-JVS(707)*X(163))/(JVS(691))
  X(129) = (X(129)-JVS(672)*X(133)-JVS(673)*X(148)-JVS(674)*X(150)-JVS(675)*X(153)-JVS(676)*X(154)-JVS(677)*X(155)&
             &-JVS(678)*X(158)-JVS(679)*X(159)-JVS(680)*X(160)-JVS(681)*X(161)-JVS(682)*X(163)-JVS(683)*X(164))/(JVS(671))
  X(128) = (X(128)-JVS(662)*X(153)-JVS(663)*X(155)-JVS(664)*X(158)-JVS(665)*X(159)-JVS(666)*X(161)-JVS(667)*X(163)&
             &-JVS(668)*X(164))/(JVS(661))
  X(127) = (X(127)-JVS(653)*X(152)-JVS(654)*X(155)-JVS(655)*X(157)-JVS(656)*X(158)-JVS(657)*X(159)-JVS(658)*X(161)&
             &-JVS(659)*X(163)-JVS(660)*X(164))/(JVS(652))
  X(126) = (X(126)-JVS(642)*X(148)-JVS(643)*X(150)-JVS(644)*X(154)-JVS(645)*X(155)-JVS(646)*X(158)-JVS(647)*X(159)&
             &-JVS(648)*X(161)-JVS(649)*X(163))/(JVS(641))
  X(125) = (X(125)-JVS(633)*X(151)-JVS(634)*X(155)-JVS(635)*X(158)-JVS(636)*X(159)-JVS(637)*X(163)-JVS(638)*X(164))&
             &/(JVS(632))
  X(124) = (X(124)-JVS(626)*X(136)-JVS(627)*X(159)-JVS(628)*X(161)-JVS(629)*X(162)-JVS(630)*X(163)-JVS(631)*X(164))&
             &/(JVS(625))
  X(123) = (X(123)-JVS(617)*X(152)-JVS(618)*X(155)-JVS(619)*X(157)-JVS(620)*X(158)-JVS(621)*X(159)-JVS(622)*X(161)&
             &-JVS(623)*X(163)-JVS(624)*X(164))/(JVS(616))
  X(122) = (X(122)-JVS(607)*X(138)-JVS(608)*X(156)-JVS(609)*X(157)-JVS(610)*X(161)-JVS(611)*X(162)-JVS(612)*X(163)&
             &-JVS(613)*X(164))/(JVS(606))
  X(121) = (X(121)-JVS(597)*X(141)-JVS(598)*X(144)-JVS(599)*X(145)-JVS(600)*X(146)-JVS(601)*X(157)-JVS(602)*X(159)&
             &-JVS(603)*X(161)-JVS(604)*X(163))/(JVS(596))
  X(120) = (X(120)-JVS(589)*X(140)-JVS(590)*X(141)-JVS(591)*X(157)-JVS(592)*X(158)-JVS(593)*X(161)-JVS(594)*X(163))&
             &/(JVS(588))
  X(119) = (X(119)-JVS(582)*X(137)-JVS(583)*X(155)-JVS(584)*X(158)-JVS(585)*X(159)-JVS(586)*X(161)-JVS(587)*X(163))&
             &/(JVS(581))
  X(118) = (X(118)-JVS(569)*X(130)-JVS(570)*X(131)-JVS(571)*X(136)-JVS(572)*X(141)-JVS(573)*X(155)-JVS(574)*X(157)&
             &-JVS(575)*X(158)-JVS(576)*X(159)-JVS(577)*X(161)-JVS(578)*X(163)-JVS(579)*X(164))/(JVS(568))
  X(117) = (X(117)-JVS(556)*X(148)-JVS(557)*X(150)-JVS(558)*X(154)-JVS(559)*X(155)-JVS(560)*X(158)-JVS(561)*X(159)&
             &-JVS(562)*X(161)-JVS(563)*X(163))/(JVS(555))
  X(116) = (X(116)-JVS(532)*X(118)-JVS(533)*X(119)-JVS(534)*X(131)-JVS(535)*X(136)-JVS(536)*X(139)-JVS(537)*X(141)&
             &-JVS(538)*X(143)-JVS(539)*X(145)-JVS(540)*X(146)-JVS(541)*X(149)-JVS(542)*X(151)-JVS(543)*X(153)-JVS(544)&
             &*X(154)-JVS(545)*X(155)-JVS(546)*X(157)-JVS(547)*X(159)-JVS(548)*X(160)-JVS(549)*X(161)-JVS(550)*X(162)&
             &-JVS(551)*X(163)-JVS(552)*X(164))/(JVS(531))
  X(115) = (X(115)-JVS(518)*X(136)-JVS(519)*X(158)-JVS(520)*X(159)-JVS(521)*X(163)-JVS(522)*X(164))/(JVS(517))
  X(114) = (X(114)-JVS(511)*X(138)-JVS(512)*X(152)-JVS(513)*X(156)-JVS(514)*X(157)-JVS(515)*X(162)-JVS(516)*X(163))&
             &/(JVS(510))
  X(113) = (X(113)-JVS(500)*X(120)-JVS(501)*X(125)-JVS(502)*X(127)-JVS(503)*X(146)-JVS(504)*X(155)-JVS(505)*X(157)&
             &-JVS(506)*X(158)-JVS(507)*X(159)-JVS(508)*X(161)-JVS(509)*X(163))/(JVS(499))
  X(112) = (X(112)-JVS(491)*X(154)-JVS(492)*X(157)-JVS(493)*X(159)-JVS(494)*X(161)-JVS(495)*X(162)-JVS(496)*X(163)&
             &-JVS(497)*X(164))/(JVS(490))
  X(111) = (X(111)-JVS(482)*X(138)-JVS(483)*X(142)-JVS(484)*X(151)-JVS(485)*X(156)-JVS(486)*X(157)-JVS(487)*X(162)&
             &-JVS(488)*X(163))/(JVS(481))
  X(110) = (X(110)-JVS(477)*X(152)-JVS(478)*X(157)-JVS(479)*X(161)-JVS(480)*X(164))/(JVS(476))
  X(109) = (X(109)-JVS(472)*X(157)-JVS(473)*X(161)-JVS(474)*X(164))/(JVS(471))
  X(108) = (X(108)-JVS(463)*X(109)-JVS(464)*X(110)-JVS(465)*X(138)-JVS(466)*X(151)-JVS(467)*X(152)-JVS(468)*X(157)&
             &-JVS(469)*X(162)-JVS(470)*X(163))/(JVS(462))
  X(107) = (X(107)-JVS(457)*X(136)-JVS(458)*X(158)-JVS(459)*X(159)-JVS(460)*X(161)-JVS(461)*X(163))/(JVS(456))
  X(106) = (X(106)-JVS(447)*X(126)-JVS(448)*X(148)-JVS(449)*X(150)-JVS(450)*X(154)-JVS(451)*X(155)-JVS(452)*X(158)&
             &-JVS(453)*X(159)-JVS(454)*X(161)-JVS(455)*X(163))/(JVS(446))
  X(105) = (X(105)-JVS(439)*X(138)-JVS(440)*X(142)-JVS(441)*X(156)-JVS(442)*X(157)-JVS(443)*X(162)-JVS(444)*X(163))&
             &/(JVS(438))
  X(104) = (X(104)-JVS(431)*X(132)-JVS(432)*X(141)-JVS(433)*X(145)-JVS(434)*X(157)-JVS(435)*X(159)-JVS(436)*X(161)&
             &-JVS(437)*X(163))/(JVS(430))
  X(103) = (X(103)-JVS(423)*X(110)-JVS(424)*X(138)-JVS(425)*X(156)-JVS(426)*X(157)-JVS(427)*X(162)-JVS(428)*X(163))&
             &/(JVS(422))
  X(102) = (X(102)-JVS(416)*X(109)-JVS(417)*X(138)-JVS(418)*X(156)-JVS(419)*X(157)-JVS(420)*X(162)-JVS(421)*X(163))&
             &/(JVS(415))
  X(101) = (X(101)-JVS(410)*X(124)-JVS(411)*X(137)-JVS(412)*X(157)-JVS(413)*X(161)-JVS(414)*X(162))/(JVS(409))
  X(100) = (X(100)-JVS(405)*X(141)-JVS(406)*X(155)-JVS(407)*X(159)-JVS(408)*X(163))/(JVS(404))
  X(99) = (X(99)-JVS(390)*X(117)-JVS(391)*X(119)-JVS(392)*X(123)-JVS(393)*X(126)-JVS(394)*X(127)-JVS(395)*X(128)&
            &-JVS(396)*X(131)-JVS(397)*X(132)-JVS(398)*X(135)-JVS(399)*X(141)-JVS(400)*X(144)-JVS(401)*X(148)-JVS(402)&
            &*X(158)-JVS(403)*X(161))/(JVS(389))
  X(98) = (X(98)-JVS(383)*X(131)-JVS(384)*X(155)-JVS(385)*X(157)-JVS(386)*X(159)-JVS(387)*X(161)-JVS(388)*X(163))&
            &/(JVS(382))
  X(97) = (X(97)-JVS(376)*X(121)-JVS(377)*X(159)-JVS(378)*X(161)-JVS(379)*X(162)-JVS(380)*X(163))/(JVS(375))
  X(96) = (X(96)-JVS(370)*X(141)-JVS(371)*X(159)-JVS(372)*X(161)-JVS(373)*X(163))/(JVS(369))
  X(95) = (X(95)-JVS(363)*X(141)-JVS(364)*X(157)-JVS(365)*X(161)-JVS(366)*X(163))/(JVS(362))
  X(94) = (X(94)-JVS(357)*X(125)-JVS(358)*X(145)-JVS(359)*X(146)-JVS(360)*X(159)-JVS(361)*X(161))/(JVS(356))
  X(93) = (X(93)-JVS(351)*X(107)-JVS(352)*X(115)-JVS(353)*X(140)-JVS(354)*X(159)-JVS(355)*X(161))/(JVS(350))
  X(92) = (X(92)-JVS(345)*X(131)-JVS(346)*X(155)-JVS(347)*X(157)-JVS(348)*X(161)-JVS(349)*X(163))/(JVS(344))
  X(91) = (X(91)-JVS(339)*X(131)-JVS(340)*X(155)-JVS(341)*X(159)-JVS(342)*X(161)-JVS(343)*X(163))/(JVS(338))
  X(90) = (X(90)-JVS(333)*X(122)-JVS(334)*X(144)-JVS(335)*X(159)-JVS(336)*X(161))/(JVS(332))
  X(89) = (X(89)-JVS(329)*X(131)-JVS(330)*X(159)-JVS(331)*X(161))/(JVS(328))
  X(88) = (X(88)-JVS(325)*X(141)-JVS(326)*X(159)-JVS(327)*X(161))/(JVS(324))
  X(87) = (X(87)-JVS(321)*X(132)-JVS(322)*X(159)-JVS(323)*X(161))/(JVS(320))
  X(86) = (X(86)-JVS(315)*X(98)-JVS(316)*X(100)-JVS(317)*X(148)-JVS(318)*X(159)-JVS(319)*X(161))/(JVS(314))
  X(85) = (X(85)-JVS(310)*X(128)-JVS(311)*X(134)-JVS(312)*X(159)-JVS(313)*X(161))/(JVS(309))
  X(84) = (X(84)-JVS(306)*X(127)-JVS(307)*X(159)-JVS(308)*X(161))/(JVS(305))
  X(83) = (X(83)-JVS(296)*X(102)-JVS(297)*X(103)-JVS(298)*X(105)-JVS(299)*X(108)-JVS(300)*X(111)-JVS(301)*X(114)&
            &-JVS(302)*X(122)-JVS(303)*X(159)-JVS(304)*X(161))/(JVS(295))
  X(82) = (X(82)-JVS(292)*X(154)-JVS(293)*X(161)-JVS(294)*X(164))/(JVS(291))
  X(81) = (X(81)-JVS(288)*X(123)-JVS(289)*X(159)-JVS(290)*X(161))/(JVS(287))
  X(80) = (X(80)-JVS(281)*X(120)-JVS(282)*X(146)-JVS(283)*X(157)-JVS(284)*X(159)-JVS(285)*X(161)-JVS(286)*X(163))&
            &/(JVS(280))
  X(79) = (X(79)-JVS(277)*X(112)-JVS(278)*X(154)-JVS(279)*X(162))/(JVS(276))
  X(78) = (X(78)-JVS(272)*X(124)-JVS(273)*X(133)-JVS(274)*X(159)-JVS(275)*X(161))/(JVS(271))
  X(77) = (X(77)-JVS(268)*X(137)-JVS(269)*X(159)-JVS(270)*X(161))/(JVS(267))
  X(76) = (X(76)-JVS(260)*X(82)-JVS(261)*X(106)-JVS(262)*X(143)-JVS(263)*X(149)-JVS(264)*X(154)-JVS(265)*X(159)-JVS(266)&
            &*X(161))/(JVS(259))
  X(75) = (X(75)-JVS(252)*X(76)-JVS(253)*X(79)-JVS(254)*X(112)-JVS(255)*X(154)-JVS(256)*X(159)-JVS(257)*X(161))&
            &/(JVS(251))
  X(74) = (X(74)-JVS(245)*X(75)-JVS(246)*X(79)-JVS(247)*X(112)-JVS(248)*X(159)-JVS(249)*X(161))/(JVS(244))
  X(73) = (X(73)-JVS(241)*X(158)-JVS(242)*X(159)-JVS(243)*X(161))/(JVS(240))
  X(72) = (X(72)-JVS(237)*X(159)-JVS(238)*X(161)-JVS(239)*X(162))/(JVS(236))
  X(71) = (X(71)-JVS(233)*X(129)-JVS(234)*X(159)-JVS(235)*X(161))/(JVS(232))
  X(70) = (X(70)-JVS(230)*X(154)-JVS(231)*X(161))/(JVS(229))
  X(69) = (X(69)-JVS(226)*X(117)-JVS(227)*X(159)-JVS(228)*X(161))/(JVS(225))
  X(68) = (X(68)-JVS(223)*X(161)-JVS(224)*X(164))/(JVS(222))
  X(67) = (X(67)-JVS(219)*X(126)-JVS(220)*X(159)-JVS(221)*X(161))/(JVS(218))
  X(66) = (X(66)-JVS(216)*X(158)-JVS(217)*X(162))/(JVS(215))
  X(65) = (X(65)-JVS(212)*X(155)-JVS(213)*X(159)-JVS(214)*X(161))/(JVS(211))
  X(64) = (X(64)-JVS(208)*X(161)-JVS(209)*X(162)-JVS(210)*X(163))/(JVS(207))
  X(63) = (X(63)-JVS(205)*X(161)-JVS(206)*X(164))/(JVS(204))
  X(62) = (X(62)-JVS(202)*X(155)-JVS(203)*X(162))/(JVS(201))
  X(61) = (X(61)-JVS(199)*X(162)-JVS(200)*X(164))/(JVS(198))
  X(60) = (X(60)-JVS(195)*X(159)-JVS(196)*X(161)-JVS(197)*X(163))/(JVS(194))
  X(59) = (X(59)-JVS(190)*X(159)-JVS(191)*X(161)-JVS(192)*X(163))/(JVS(189))
  X(58) = (X(58)-JVS(185)*X(159)-JVS(186)*X(161)-JVS(187)*X(163))/(JVS(184))
  X(57) = (X(57)-JVS(181)*X(97)-JVS(182)*X(162))/(JVS(180))
  X(56) = (X(56)-JVS(178)*X(88)-JVS(179)*X(161))/(JVS(177))
  X(55) = (X(55)-JVS(175)*X(154)-JVS(176)*X(162))/(JVS(174))
  X(54) = (X(54)-JVS(172)*X(133)-JVS(173)*X(162))/(JVS(171))
  X(53) = (X(53)-JVS(170)*X(161))/(JVS(169))
  X(52) = (X(52)-JVS(168)*X(161))/(JVS(167))
  X(51) = (X(51)-JVS(166)*X(161))/(JVS(165))
  X(50) = (X(50)-JVS(164)*X(161))/(JVS(163))
  X(49) = (X(49)-JVS(162)*X(161))/(JVS(161))
  X(48) = (X(48)-JVS(160)*X(161))/(JVS(159))
  X(47) = (X(47)-JVS(132)*X(54)-JVS(133)*X(56)-JVS(134)*X(57)-JVS(135)*X(61)-JVS(136)*X(62)-JVS(137)*X(74)-JVS(138)&
            &*X(75)-JVS(139)*X(76)-JVS(140)*X(79)-JVS(141)*X(83)-JVS(142)*X(88)-JVS(143)*X(95)-JVS(144)*X(101)-JVS(145)&
            &*X(104)-JVS(146)*X(106)-JVS(147)*X(113)-JVS(148)*X(116)-JVS(149)*X(120)-JVS(150)*X(121)-JVS(151)*X(143)&
            &-JVS(152)*X(147)-JVS(153)*X(149)-JVS(154)*X(150)-JVS(155)*X(151)-JVS(156)*X(152)-JVS(157)*X(157)-JVS(158)&
            &*X(162))/(JVS(131))
  X(46) = (X(46)-JVS(112)*X(80)-JVS(113)*X(90)-JVS(114)*X(92)-JVS(115)*X(96)-JVS(116)*X(97)-JVS(117)*X(98)-JVS(118)&
            &*X(119)-JVS(119)*X(125)-JVS(120)*X(130)-JVS(121)*X(137)-JVS(122)*X(141)-JVS(123)*X(147)-JVS(124)*X(155)&
            &-JVS(125)*X(156)-JVS(126)*X(157)-JVS(127)*X(158)-JVS(128)*X(159)-JVS(129)*X(161)-JVS(130)*X(163))/(JVS(111))
  X(45) = X(45)/JVS(110)
  X(44) = X(44)/JVS(109)
  X(43) = (X(43)-JVS(108)*X(113))/(JVS(107))
  X(42) = (X(42)-JVS(106)*X(104))/(JVS(105))
  X(41) = (X(41)-JVS(104)*X(121))/(JVS(103))
  X(40) = (X(40)-JVS(102)*X(56))/(JVS(101))
  X(39) = (X(39)-JVS(100)*X(88))/(JVS(99))
  X(38) = (X(38)-JVS(98)*X(95))/(JVS(97))
  X(37) = (X(37)-JVS(96)*X(120))/(JVS(95))
  X(36) = (X(36)-JVS(93)*X(104)-JVS(94)*X(161))/(JVS(92))
  X(35) = (X(35)-JVS(91)*X(75))/(JVS(90))
  X(34) = (X(34)-JVS(89)*X(79))/(JVS(88))
  X(33) = (X(33)-JVS(87)*X(76))/(JVS(86))
  X(32) = (X(32)-JVS(85)*X(74))/(JVS(84))
  X(31) = (X(31)-JVS(83)*X(161))/(JVS(82))
  X(30) = (X(30)-JVS(78)*X(31)-JVS(79)*X(159)-JVS(80)*X(161)-JVS(81)*X(163))/(JVS(77))
  X(29) = (X(29)-JVS(75)*X(30)-JVS(76)*X(163))/(JVS(74))
  X(28) = (X(28)-JVS(72)*X(30)-JVS(73)*X(159))/(JVS(71))
  X(27) = (X(27)-JVS(69)*X(60)-JVS(70)*X(163))/(JVS(68))
  X(26) = (X(26)-JVS(66)*X(60)-JVS(67)*X(159))/(JVS(65))
  X(25) = (X(25)-JVS(63)*X(59)-JVS(64)*X(163))/(JVS(62))
  X(24) = (X(24)-JVS(60)*X(59)-JVS(61)*X(159))/(JVS(59))
  X(23) = (X(23)-JVS(57)*X(58)-JVS(58)*X(163))/(JVS(56))
  X(22) = (X(22)-JVS(54)*X(58)-JVS(55)*X(159))/(JVS(53))
  X(21) = (X(21)-JVS(51)*X(109)-JVS(52)*X(164))/(JVS(50))
  X(20) = (X(20)-JVS(48)*X(109)-JVS(49)*X(161))/(JVS(47))
  X(19) = (X(19)-JVS(45)*X(68)-JVS(46)*X(161))/(JVS(44))
  X(18) = (X(18)-JVS(35)*X(102)-JVS(36)*X(103)-JVS(37)*X(105)-JVS(38)*X(108)-JVS(39)*X(111)-JVS(40)*X(114)-JVS(41)&
            &*X(122)-JVS(42)*X(138)-JVS(43)*X(161))/(JVS(34))
  X(17) = (X(17)-JVS(33)*X(149))/(JVS(32))
  X(16) = (X(16)-JVS(31)*X(147))/(JVS(30))
  X(15) = (X(15)-JVS(29)*X(151))/(JVS(28))
  X(14) = (X(14)-JVS(27)*X(152))/(JVS(26))
  X(13) = (X(13)-JVS(25)*X(150))/(JVS(24))
  X(12) = (X(12)-JVS(23)*X(54))/(JVS(22))
  X(11) = (X(11)-JVS(21)*X(57))/(JVS(20))
  X(10) = (X(10)-JVS(19)*X(101))/(JVS(18))
  X(9) = (X(9)-JVS(17)*X(62))/(JVS(16))
  X(8) = (X(8)-JVS(15)*X(157))/(JVS(14))
  X(7) = (X(7)-JVS(13)*X(162))/(JVS(12))
  X(6) = (X(6)-JVS(11)*X(61))/(JVS(10))
  X(5) = (X(5)-JVS(9)*X(116))/(JVS(8))
  X(4) = (X(4)-JVS(7)*X(83))/(JVS(6))
  X(3) = (X(3)-JVS(5)*X(143))/(JVS(4))
  X(2) = (X(2)-JVS(3)*X(106))/(JVS(2))
  X(1) = X(1)/JVS(1)
      
END SUBROUTINE KppSolve

! End of KppSolve function
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~


! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
! 
! KppSolveTR - sparse, transposed back substitution
!   Arguments :
!      JVS       - sparse Jacobian of variables
!      X         - Vector for variables
!      XX        - Vector for output variables
! 
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

SUBROUTINE KppSolveTR ( JVS, X, XX )

! JVS - sparse Jacobian of variables
  REAL(kind=dp) :: JVS(LU_NONZERO)
! X - Vector for variables
  REAL(kind=dp) :: X(NVAR)
! XX - Vector for output variables
  REAL(kind=dp) :: XX(NVAR)

  XX(1) = X(1)/JVS(1)
  XX(2) = X(2)/JVS(2)
  XX(3) = X(3)/JVS(4)
  XX(4) = X(4)/JVS(6)
  XX(5) = X(5)/JVS(8)
  XX(6) = X(6)/JVS(10)
  XX(7) = X(7)/JVS(12)
  XX(8) = X(8)/JVS(14)
  XX(9) = X(9)/JVS(16)
  XX(10) = X(10)/JVS(18)
  XX(11) = X(11)/JVS(20)
  XX(12) = X(12)/JVS(22)
  XX(13) = X(13)/JVS(24)
  XX(14) = X(14)/JVS(26)
  XX(15) = X(15)/JVS(28)
  XX(16) = X(16)/JVS(30)
  XX(17) = X(17)/JVS(32)
  XX(18) = X(18)/JVS(34)
  XX(19) = X(19)/JVS(44)
  XX(20) = X(20)/JVS(47)
  XX(21) = X(21)/JVS(50)
  XX(22) = X(22)/JVS(53)
  XX(23) = X(23)/JVS(56)
  XX(24) = X(24)/JVS(59)
  XX(25) = X(25)/JVS(62)
  XX(26) = X(26)/JVS(65)
  XX(27) = X(27)/JVS(68)
  XX(28) = X(28)/JVS(71)
  XX(29) = X(29)/JVS(74)
  XX(30) = (X(30)-JVS(72)*XX(28)-JVS(75)*XX(29))/(JVS(77))
  XX(31) = (X(31)-JVS(78)*XX(30))/(JVS(82))
  XX(32) = X(32)/JVS(84)
  XX(33) = X(33)/JVS(86)
  XX(34) = X(34)/JVS(88)
  XX(35) = X(35)/JVS(90)
  XX(36) = X(36)/JVS(92)
  XX(37) = X(37)/JVS(95)
  XX(38) = X(38)/JVS(97)
  XX(39) = X(39)/JVS(99)
  XX(40) = X(40)/JVS(101)
  XX(41) = X(41)/JVS(103)
  XX(42) = X(42)/JVS(105)
  XX(43) = X(43)/JVS(107)
  XX(44) = X(44)/JVS(109)
  XX(45) = X(45)/JVS(110)
  XX(46) = X(46)/JVS(111)
  XX(47) = X(47)/JVS(131)
  XX(48) = X(48)/JVS(159)
  XX(49) = X(49)/JVS(161)
  XX(50) = X(50)/JVS(163)
  XX(51) = X(51)/JVS(165)
  XX(52) = X(52)/JVS(167)
  XX(53) = X(53)/JVS(169)
  XX(54) = (X(54)-JVS(23)*XX(12)-JVS(132)*XX(47))/(JVS(171))
  XX(55) = X(55)/JVS(174)
  XX(56) = (X(56)-JVS(102)*XX(40)-JVS(133)*XX(47))/(JVS(177))
  XX(57) = (X(57)-JVS(21)*XX(11)-JVS(134)*XX(47))/(JVS(180))
  XX(58) = (X(58)-JVS(54)*XX(22)-JVS(57)*XX(23))/(JVS(184))
  XX(59) = (X(59)-JVS(60)*XX(24)-JVS(63)*XX(25))/(JVS(189))
  XX(60) = (X(60)-JVS(66)*XX(26)-JVS(69)*XX(27))/(JVS(194))
  XX(61) = (X(61)-JVS(11)*XX(6)-JVS(135)*XX(47))/(JVS(198))
  XX(62) = (X(62)-JVS(17)*XX(9)-JVS(136)*XX(47))/(JVS(201))
  XX(63) = X(63)/JVS(204)
  XX(64) = X(64)/JVS(207)
  XX(65) = X(65)/JVS(211)
  XX(66) = X(66)/JVS(215)
  XX(67) = X(67)/JVS(218)
  XX(68) = (X(68)-JVS(45)*XX(19))/(JVS(222))
  XX(69) = X(69)/JVS(225)
  XX(70) = X(70)/JVS(229)
  XX(71) = X(71)/JVS(232)
  XX(72) = X(72)/JVS(236)
  XX(73) = X(73)/JVS(240)
  XX(74) = (X(74)-JVS(85)*XX(32)-JVS(137)*XX(47))/(JVS(244))
  XX(75) = (X(75)-JVS(91)*XX(35)-JVS(138)*XX(47)-JVS(245)*XX(74))/(JVS(251))
  XX(76) = (X(76)-JVS(87)*XX(33)-JVS(139)*XX(47)-JVS(252)*XX(75))/(JVS(259))
  XX(77) = X(77)/JVS(267)
  XX(78) = X(78)/JVS(271)
  XX(79) = (X(79)-JVS(89)*XX(34)-JVS(140)*XX(47)-JVS(246)*XX(74)-JVS(253)*XX(75))/(JVS(276))
  XX(80) = (X(80)-JVS(112)*XX(46))/(JVS(280))
  XX(81) = X(81)/JVS(287)
  XX(82) = (X(82)-JVS(260)*XX(76))/(JVS(291))
  XX(83) = (X(83)-JVS(7)*XX(4)-JVS(141)*XX(47))/(JVS(295))
  XX(84) = X(84)/JVS(305)
  XX(85) = X(85)/JVS(309)
  XX(86) = X(86)/JVS(314)
  XX(87) = X(87)/JVS(320)
  XX(88) = (X(88)-JVS(100)*XX(39)-JVS(142)*XX(47)-JVS(178)*XX(56))/(JVS(324))
  XX(89) = X(89)/JVS(328)
  XX(90) = (X(90)-JVS(113)*XX(46))/(JVS(332))
  XX(91) = X(91)/JVS(338)
  XX(92) = (X(92)-JVS(114)*XX(46))/(JVS(344))
  XX(93) = X(93)/JVS(350)
  XX(94) = X(94)/JVS(356)
  XX(95) = (X(95)-JVS(98)*XX(38)-JVS(143)*XX(47))/(JVS(362))
  XX(96) = (X(96)-JVS(115)*XX(46))/(JVS(369))
  XX(97) = (X(97)-JVS(116)*XX(46)-JVS(181)*XX(57))/(JVS(375))
  XX(98) = (X(98)-JVS(117)*XX(46)-JVS(315)*XX(86))/(JVS(382))
  XX(99) = X(99)/JVS(389)
  XX(100) = (X(100)-JVS(316)*XX(86))/(JVS(404))
  XX(101) = (X(101)-JVS(19)*XX(10)-JVS(144)*XX(47))/(JVS(409))
  XX(102) = (X(102)-JVS(35)*XX(18)-JVS(296)*XX(83))/(JVS(415))
  XX(103) = (X(103)-JVS(36)*XX(18)-JVS(297)*XX(83))/(JVS(422))
  XX(104) = (X(104)-JVS(93)*XX(36)-JVS(106)*XX(42)-JVS(145)*XX(47))/(JVS(430))
  XX(105) = (X(105)-JVS(37)*XX(18)-JVS(298)*XX(83))/(JVS(438))
  XX(106) = (X(106)-JVS(3)*XX(2)-JVS(146)*XX(47)-JVS(261)*XX(76))/(JVS(446))
  XX(107) = (X(107)-JVS(351)*XX(93))/(JVS(456))
  XX(108) = (X(108)-JVS(38)*XX(18)-JVS(299)*XX(83))/(JVS(462))
  XX(109) = (X(109)-JVS(48)*XX(20)-JVS(51)*XX(21)-JVS(416)*XX(102)-JVS(463)*XX(108))/(JVS(471))
  XX(110) = (X(110)-JVS(423)*XX(103)-JVS(464)*XX(108))/(JVS(476))
  XX(111) = (X(111)-JVS(39)*XX(18)-JVS(300)*XX(83))/(JVS(481))
  XX(112) = (X(112)-JVS(247)*XX(74)-JVS(254)*XX(75)-JVS(277)*XX(79))/(JVS(490))
  XX(113) = (X(113)-JVS(108)*XX(43)-JVS(147)*XX(47))/(JVS(499))
  XX(114) = (X(114)-JVS(40)*XX(18)-JVS(301)*XX(83))/(JVS(510))
  XX(115) = (X(115)-JVS(352)*XX(93))/(JVS(517))
  XX(116) = (X(116)-JVS(9)*XX(5)-JVS(148)*XX(47))/(JVS(531))
  XX(117) = (X(117)-JVS(226)*XX(69)-JVS(390)*XX(99))/(JVS(555))
  XX(118) = (X(118)-JVS(532)*XX(116))/(JVS(568))
  XX(119) = (X(119)-JVS(118)*XX(46)-JVS(391)*XX(99)-JVS(533)*XX(116))/(JVS(581))
  XX(120) = (X(120)-JVS(96)*XX(37)-JVS(149)*XX(47)-JVS(281)*XX(80)-JVS(500)*XX(113))/(JVS(588))
  XX(121) = (X(121)-JVS(104)*XX(41)-JVS(150)*XX(47)-JVS(376)*XX(97))/(JVS(596))
  XX(122) = (X(122)-JVS(41)*XX(18)-JVS(302)*XX(83)-JVS(333)*XX(90))/(JVS(606))
  XX(123) = (X(123)-JVS(288)*XX(81)-JVS(392)*XX(99))/(JVS(616))
  XX(124) = (X(124)-JVS(272)*XX(78)-JVS(410)*XX(101))/(JVS(625))
  XX(125) = (X(125)-JVS(119)*XX(46)-JVS(357)*XX(94)-JVS(501)*XX(113))/(JVS(632))
  XX(126) = (X(126)-JVS(219)*XX(67)-JVS(393)*XX(99)-JVS(447)*XX(106))/(JVS(641))
  XX(127) = (X(127)-JVS(306)*XX(84)-JVS(394)*XX(99)-JVS(502)*XX(113))/(JVS(652))
  XX(128) = (X(128)-JVS(310)*XX(85)-JVS(395)*XX(99))/(JVS(661))
  XX(129) = (X(129)-JVS(233)*XX(71))/(JVS(671))
  XX(130) = (X(130)-JVS(120)*XX(46)-JVS(569)*XX(118))/(JVS(691))
  XX(131) = (X(131)-JVS(329)*XX(89)-JVS(339)*XX(91)-JVS(345)*XX(92)-JVS(383)*XX(98)-JVS(396)*XX(99)-JVS(534)*XX(116)&
              &-JVS(570)*XX(118)-JVS(692)*XX(130))/(JVS(709))
  XX(132) = (X(132)-JVS(321)*XX(87)-JVS(397)*XX(99)-JVS(431)*XX(104)-JVS(693)*XX(130))/(JVS(718))
  XX(133) = (X(133)-JVS(172)*XX(54)-JVS(273)*XX(78)-JVS(672)*XX(129))/(JVS(730))
  XX(134) = (X(134)-JVS(311)*XX(85))/(JVS(746))
  XX(135) = (X(135)-JVS(398)*XX(99))/(JVS(759))
  XX(136) = (X(136)-JVS(457)*XX(107)-JVS(518)*XX(115)-JVS(535)*XX(116)-JVS(571)*XX(118)-JVS(626)*XX(124)-JVS(731)&
              &*XX(133))/(JVS(770))
  XX(137) = (X(137)-JVS(121)*XX(46)-JVS(268)*XX(77)-JVS(411)*XX(101)-JVS(582)*XX(119))/(JVS(782))
  XX(138) = (X(138)-JVS(42)*XX(18)-JVS(417)*XX(102)-JVS(424)*XX(103)-JVS(439)*XX(105)-JVS(465)*XX(108)-JVS(482)*XX(111)&
              &-JVS(511)*XX(114)-JVS(607)*XX(122)-JVS(694)*XX(130))/(JVS(803))
  XX(139) = (X(139)-JVS(536)*XX(116))/(JVS(832))
  XX(140) = (X(140)-JVS(353)*XX(93)-JVS(589)*XX(120)-JVS(695)*XX(130)-JVS(771)*XX(136)-JVS(783)*XX(137)-JVS(833)&
              &*XX(139))/(JVS(860))
  XX(141) = (X(141)-JVS(122)*XX(46)-JVS(325)*XX(88)-JVS(363)*XX(95)-JVS(370)*XX(96)-JVS(399)*XX(99)-JVS(405)*XX(100)&
              &-JVS(432)*XX(104)-JVS(537)*XX(116)-JVS(572)*XX(118)-JVS(590)*XX(120)-JVS(597)*XX(121)-JVS(696)*XX(130)&
              &-JVS(834)*XX(139))/(JVS(870))
  XX(142) = (X(142)-JVS(440)*XX(105)-JVS(483)*XX(111)-JVS(697)*XX(130)-JVS(710)*XX(131)-JVS(804)*XX(138)-JVS(835)&
              &*XX(139))/(JVS(883))
  XX(143) = (X(143)-JVS(5)*XX(3)-JVS(151)*XX(47)-JVS(262)*XX(76)-JVS(538)*XX(116))/(JVS(938))
  XX(144) = (X(144)-JVS(334)*XX(90)-JVS(400)*XX(99)-JVS(598)*XX(121)-JVS(836)*XX(139)-JVS(939)*XX(143))/(JVS(964))
  XX(145) = (X(145)-JVS(358)*XX(94)-JVS(433)*XX(104)-JVS(539)*XX(116)-JVS(599)*XX(121)-JVS(698)*XX(130)-JVS(760)*XX(135)&
              &-JVS(837)*XX(139)-JVS(940)*XX(143))/(JVS(980))
  XX(146) = (X(146)-JVS(282)*XX(80)-JVS(359)*XX(94)-JVS(503)*XX(113)-JVS(540)*XX(116)-JVS(600)*XX(121)-JVS(699)*XX(130)&
              &-JVS(761)*XX(135)-JVS(838)*XX(139)-JVS(941)*XX(143)-JVS(981)*XX(145))/(JVS(996))
  XX(147) = (X(147)-JVS(31)*XX(16)-JVS(123)*XX(46)-JVS(152)*XX(47)-JVS(839)*XX(139)-JVS(942)*XX(143))/(JVS(1031))
  XX(148) = (X(148)-JVS(317)*XX(86)-JVS(401)*XX(99)-JVS(448)*XX(106)-JVS(556)*XX(117)-JVS(642)*XX(126)-JVS(673)*XX(129)&
              &-JVS(747)*XX(134)-JVS(840)*XX(139)-JVS(943)*XX(143)-JVS(1032)*XX(147))/(JVS(1057))
  XX(149) = (X(149)-JVS(33)*XX(17)-JVS(153)*XX(47)-JVS(263)*XX(76)-JVS(541)*XX(116)-JVS(944)*XX(143))/(JVS(1086))
  XX(150) = (X(150)-JVS(25)*XX(13)-JVS(154)*XX(47)-JVS(449)*XX(106)-JVS(557)*XX(117)-JVS(643)*XX(126)-JVS(674)*XX(129)&
              &-JVS(748)*XX(134)-JVS(762)*XX(135)-JVS(841)*XX(139)-JVS(945)*XX(143)-JVS(1033)*XX(147)-JVS(1058)*XX(148)&
              &-JVS(1087)*XX(149))/(JVS(1115))
  XX(151) = (X(151)-JVS(29)*XX(15)-JVS(155)*XX(47)-JVS(466)*XX(108)-JVS(484)*XX(111)-JVS(542)*XX(116)-JVS(633)*XX(125)&
              &-JVS(763)*XX(135)-JVS(784)*XX(137)-JVS(805)*XX(138)-JVS(842)*XX(139)-JVS(946)*XX(143)-JVS(965)*XX(144)&
              &-JVS(982)*XX(145)-JVS(997)*XX(146)-JVS(1034)*XX(147)-JVS(1059)*XX(148)-JVS(1088)*XX(149)-JVS(1116)*XX(150))&
              &/(JVS(1134))
  XX(152) = (X(152)-JVS(27)*XX(14)-JVS(156)*XX(47)-JVS(467)*XX(108)-JVS(477)*XX(110)-JVS(512)*XX(114)-JVS(617)*XX(123)&
              &-JVS(653)*XX(127)-JVS(719)*XX(132)-JVS(732)*XX(133)-JVS(806)*XX(138)-JVS(843)*XX(139)-JVS(947)*XX(143)&
              &-JVS(966)*XX(144)-JVS(1035)*XX(147)-JVS(1089)*XX(149)-JVS(1135)*XX(151))/(JVS(1153))
  XX(153) = (X(153)-JVS(543)*XX(116)-JVS(662)*XX(128)-JVS(675)*XX(129)-JVS(733)*XX(133)-JVS(749)*XX(134)-JVS(844)&
              &*XX(139)-JVS(948)*XX(143)-JVS(1036)*XX(147)-JVS(1090)*XX(149)-JVS(1117)*XX(150))/(JVS(1185))
  XX(154) = (X(154)-JVS(175)*XX(55)-JVS(230)*XX(70)-JVS(255)*XX(75)-JVS(264)*XX(76)-JVS(278)*XX(79)-JVS(292)*XX(82)&
              &-JVS(450)*XX(106)-JVS(491)*XX(112)-JVS(544)*XX(116)-JVS(558)*XX(117)-JVS(644)*XX(126)-JVS(676)*XX(129)&
              &-JVS(750)*XX(134)-JVS(845)*XX(139)-JVS(949)*XX(143)-JVS(1037)*XX(147)-JVS(1091)*XX(149)-JVS(1118)*XX(150)&
              &-JVS(1186)*XX(153))/(JVS(1220))
  XX(155) = (X(155)-JVS(124)*XX(46)-JVS(202)*XX(62)-JVS(212)*XX(65)-JVS(340)*XX(91)-JVS(346)*XX(92)-JVS(384)*XX(98)&
              &-JVS(406)*XX(100)-JVS(451)*XX(106)-JVS(504)*XX(113)-JVS(545)*XX(116)-JVS(559)*XX(117)-JVS(573)*XX(118)&
              &-JVS(583)*XX(119)-JVS(618)*XX(123)-JVS(634)*XX(125)-JVS(645)*XX(126)-JVS(654)*XX(127)-JVS(663)*XX(128)&
              &-JVS(677)*XX(129)-JVS(700)*XX(130)-JVS(711)*XX(131)-JVS(720)*XX(132)-JVS(734)*XX(133)-JVS(751)*XX(134)&
              &-JVS(764)*XX(135)-JVS(772)*XX(136)-JVS(785)*XX(137)-JVS(846)*XX(139)-JVS(861)*XX(140)-JVS(871)*XX(141)&
              &-JVS(884)*XX(142)-JVS(950)*XX(143)-JVS(967)*XX(144)-JVS(983)*XX(145)-JVS(998)*XX(146)-JVS(1038)*XX(147)&
              &-JVS(1060)*XX(148)-JVS(1092)*XX(149)-JVS(1119)*XX(150)-JVS(1136)*XX(151)-JVS(1154)*XX(152)-JVS(1187)*XX(153)&
              &-JVS(1221)*XX(154))/(JVS(1273))
  XX(156) = (X(156)-JVS(125)*XX(46)-JVS(418)*XX(102)-JVS(425)*XX(103)-JVS(441)*XX(105)-JVS(485)*XX(111)-JVS(513)*XX(114)&
              &-JVS(608)*XX(122)-JVS(701)*XX(130)-JVS(807)*XX(138)-JVS(847)*XX(139)-JVS(951)*XX(143)-JVS(968)*XX(144)&
              &-JVS(1039)*XX(147)-JVS(1093)*XX(149)-JVS(1137)*XX(151)-JVS(1155)*XX(152)-JVS(1188)*XX(153)-JVS(1222)*XX(154)&
              &-JVS(1274)*XX(155))/(JVS(1330))
  XX(157) = (X(157)-JVS(15)*XX(8)-JVS(126)*XX(46)-JVS(157)*XX(47)-JVS(283)*XX(80)-JVS(347)*XX(92)-JVS(364)*XX(95)&
              &-JVS(385)*XX(98)-JVS(412)*XX(101)-JVS(419)*XX(102)-JVS(426)*XX(103)-JVS(434)*XX(104)-JVS(442)*XX(105)&
              &-JVS(468)*XX(108)-JVS(472)*XX(109)-JVS(478)*XX(110)-JVS(486)*XX(111)-JVS(492)*XX(112)-JVS(505)*XX(113)&
              &-JVS(514)*XX(114)-JVS(546)*XX(116)-JVS(574)*XX(118)-JVS(591)*XX(120)-JVS(601)*XX(121)-JVS(609)*XX(122)&
              &-JVS(619)*XX(123)-JVS(655)*XX(127)-JVS(702)*XX(130)-JVS(721)*XX(132)-JVS(773)*XX(136)-JVS(786)*XX(137)&
              &-JVS(808)*XX(138)-JVS(848)*XX(139)-JVS(862)*XX(140)-JVS(872)*XX(141)-JVS(885)*XX(142)-JVS(952)*XX(143)&
              &-JVS(969)*XX(144)-JVS(984)*XX(145)-JVS(999)*XX(146)-JVS(1040)*XX(147)-JVS(1061)*XX(148)-JVS(1094)*XX(149)&
              &-JVS(1120)*XX(150)-JVS(1138)*XX(151)-JVS(1156)*XX(152)-JVS(1189)*XX(153)-JVS(1223)*XX(154)-JVS(1275)*XX(155)&
              &-JVS(1331)*XX(156))/(JVS(1360))
  XX(158) = (X(158)-JVS(127)*XX(46)-JVS(216)*XX(66)-JVS(241)*XX(73)-JVS(402)*XX(99)-JVS(452)*XX(106)-JVS(458)*XX(107)&
              &-JVS(506)*XX(113)-JVS(519)*XX(115)-JVS(560)*XX(117)-JVS(575)*XX(118)-JVS(584)*XX(119)-JVS(592)*XX(120)&
              &-JVS(620)*XX(123)-JVS(635)*XX(125)-JVS(646)*XX(126)-JVS(656)*XX(127)-JVS(664)*XX(128)-JVS(678)*XX(129)&
              &-JVS(703)*XX(130)-JVS(712)*XX(131)-JVS(722)*XX(132)-JVS(735)*XX(133)-JVS(752)*XX(134)-JVS(765)*XX(135)&
              &-JVS(774)*XX(136)-JVS(787)*XX(137)-JVS(849)*XX(139)-JVS(863)*XX(140)-JVS(873)*XX(141)-JVS(886)*XX(142)&
              &-JVS(953)*XX(143)-JVS(970)*XX(144)-JVS(985)*XX(145)-JVS(1000)*XX(146)-JVS(1041)*XX(147)-JVS(1062)*XX(148)&
              &-JVS(1095)*XX(149)-JVS(1121)*XX(150)-JVS(1139)*XX(151)-JVS(1157)*XX(152)-JVS(1190)*XX(153)-JVS(1224)*XX(154)&
              &-JVS(1276)*XX(155)-JVS(1332)*XX(156)-JVS(1361)*XX(157))/(JVS(1414))
  XX(159) = (X(159)-JVS(55)*XX(22)-JVS(61)*XX(24)-JVS(67)*XX(26)-JVS(73)*XX(28)-JVS(79)*XX(30)-JVS(128)*XX(46)-JVS(185)&
              &*XX(58)-JVS(190)*XX(59)-JVS(195)*XX(60)-JVS(213)*XX(65)-JVS(220)*XX(67)-JVS(227)*XX(69)-JVS(234)*XX(71)&
              &-JVS(237)*XX(72)-JVS(242)*XX(73)-JVS(248)*XX(74)-JVS(256)*XX(75)-JVS(265)*XX(76)-JVS(269)*XX(77)-JVS(274)&
              &*XX(78)-JVS(284)*XX(80)-JVS(289)*XX(81)-JVS(303)*XX(83)-JVS(307)*XX(84)-JVS(312)*XX(85)-JVS(318)*XX(86)&
              &-JVS(322)*XX(87)-JVS(326)*XX(88)-JVS(330)*XX(89)-JVS(335)*XX(90)-JVS(341)*XX(91)-JVS(354)*XX(93)-JVS(360)&
              &*XX(94)-JVS(371)*XX(96)-JVS(377)*XX(97)-JVS(386)*XX(98)-JVS(407)*XX(100)-JVS(435)*XX(104)-JVS(453)*XX(106)&
              &-JVS(459)*XX(107)-JVS(493)*XX(112)-JVS(507)*XX(113)-JVS(520)*XX(115)-JVS(547)*XX(116)-JVS(561)*XX(117)&
              &-JVS(576)*XX(118)-JVS(585)*XX(119)-JVS(602)*XX(121)-JVS(621)*XX(123)-JVS(627)*XX(124)-JVS(636)*XX(125)&
              &-JVS(647)*XX(126)-JVS(657)*XX(127)-JVS(665)*XX(128)-JVS(679)*XX(129)-JVS(704)*XX(130)-JVS(713)*XX(131)&
              &-JVS(723)*XX(132)-JVS(736)*XX(133)-JVS(753)*XX(134)-JVS(766)*XX(135)-JVS(788)*XX(137)-JVS(850)*XX(139)&
              &-JVS(864)*XX(140)-JVS(874)*XX(141)-JVS(887)*XX(142)-JVS(954)*XX(143)-JVS(971)*XX(144)-JVS(986)*XX(145)&
              &-JVS(1001)*XX(146)-JVS(1042)*XX(147)-JVS(1063)*XX(148)-JVS(1096)*XX(149)-JVS(1122)*XX(150)-JVS(1140)*XX(151)&
              &-JVS(1158)*XX(152)-JVS(1191)*XX(153)-JVS(1225)*XX(154)-JVS(1277)*XX(155)-JVS(1333)*XX(156)-JVS(1362)*XX(157)&
              &-JVS(1415)*XX(158))/(JVS(1502))
  XX(160) = (X(160)-JVS(548)*XX(116)-JVS(680)*XX(129)-JVS(737)*XX(133)-JVS(955)*XX(143)-JVS(1097)*XX(149)-JVS(1226)&
              &*XX(154)-JVS(1278)*XX(155)-JVS(1334)*XX(156)-JVS(1363)*XX(157)-JVS(1416)*XX(158)-JVS(1503)*XX(159))&
              &/(JVS(1547))
  XX(161) = (X(161)-JVS(43)*XX(18)-JVS(46)*XX(19)-JVS(49)*XX(20)-JVS(80)*XX(30)-JVS(83)*XX(31)-JVS(94)*XX(36)-JVS(129)&
              &*XX(46)-JVS(160)*XX(48)-JVS(162)*XX(49)-JVS(164)*XX(50)-JVS(166)*XX(51)-JVS(168)*XX(52)-JVS(170)*XX(53)&
              &-JVS(179)*XX(56)-JVS(186)*XX(58)-JVS(191)*XX(59)-JVS(196)*XX(60)-JVS(205)*XX(63)-JVS(208)*XX(64)-JVS(214)&
              &*XX(65)-JVS(221)*XX(67)-JVS(223)*XX(68)-JVS(228)*XX(69)-JVS(231)*XX(70)-JVS(235)*XX(71)-JVS(238)*XX(72)&
              &-JVS(243)*XX(73)-JVS(249)*XX(74)-JVS(257)*XX(75)-JVS(266)*XX(76)-JVS(270)*XX(77)-JVS(275)*XX(78)-JVS(285)&
              &*XX(80)-JVS(290)*XX(81)-JVS(293)*XX(82)-JVS(304)*XX(83)-JVS(308)*XX(84)-JVS(313)*XX(85)-JVS(319)*XX(86)&
              &-JVS(323)*XX(87)-JVS(327)*XX(88)-JVS(331)*XX(89)-JVS(336)*XX(90)-JVS(342)*XX(91)-JVS(348)*XX(92)-JVS(355)&
              &*XX(93)-JVS(361)*XX(94)-JVS(365)*XX(95)-JVS(372)*XX(96)-JVS(378)*XX(97)-JVS(387)*XX(98)-JVS(403)*XX(99)&
              &-JVS(413)*XX(101)-JVS(436)*XX(104)-JVS(454)*XX(106)-JVS(460)*XX(107)-JVS(473)*XX(109)-JVS(479)*XX(110)&
              &-JVS(494)*XX(112)-JVS(508)*XX(113)-JVS(549)*XX(116)-JVS(562)*XX(117)-JVS(577)*XX(118)-JVS(586)*XX(119)&
              &-JVS(593)*XX(120)-JVS(603)*XX(121)-JVS(610)*XX(122)-JVS(622)*XX(123)-JVS(628)*XX(124)-JVS(648)*XX(126)&
              &-JVS(658)*XX(127)-JVS(666)*XX(128)-JVS(681)*XX(129)-JVS(705)*XX(130)-JVS(714)*XX(131)-JVS(724)*XX(132)&
              &-JVS(738)*XX(133)-JVS(754)*XX(134)-JVS(767)*XX(135)-JVS(775)*XX(136)-JVS(789)*XX(137)-JVS(809)*XX(138)&
              &-JVS(851)*XX(139)-JVS(865)*XX(140)-JVS(875)*XX(141)-JVS(888)*XX(142)-JVS(956)*XX(143)-JVS(972)*XX(144)&
              &-JVS(987)*XX(145)-JVS(1002)*XX(146)-JVS(1043)*XX(147)-JVS(1064)*XX(148)-JVS(1098)*XX(149)-JVS(1123)*XX(150)&
              &-JVS(1141)*XX(151)-JVS(1159)*XX(152)-JVS(1192)*XX(153)-JVS(1227)*XX(154)-JVS(1279)*XX(155)-JVS(1335)*XX(156)&
              &-JVS(1364)*XX(157)-JVS(1417)*XX(158)-JVS(1504)*XX(159)-JVS(1548)*XX(160))/(JVS(1656))
  XX(162) = (X(162)-JVS(13)*XX(7)-JVS(158)*XX(47)-JVS(173)*XX(54)-JVS(176)*XX(55)-JVS(182)*XX(57)-JVS(199)*XX(61)&
              &-JVS(203)*XX(62)-JVS(209)*XX(64)-JVS(217)*XX(66)-JVS(239)*XX(72)-JVS(279)*XX(79)-JVS(379)*XX(97)-JVS(414)&
              &*XX(101)-JVS(420)*XX(102)-JVS(427)*XX(103)-JVS(443)*XX(105)-JVS(469)*XX(108)-JVS(487)*XX(111)-JVS(495)&
              &*XX(112)-JVS(515)*XX(114)-JVS(550)*XX(116)-JVS(611)*XX(122)-JVS(629)*XX(124)-JVS(706)*XX(130)-JVS(739)&
              &*XX(133)-JVS(790)*XX(137)-JVS(810)*XX(138)-JVS(852)*XX(139)-JVS(957)*XX(143)-JVS(973)*XX(144)-JVS(1044)&
              &*XX(147)-JVS(1099)*XX(149)-JVS(1124)*XX(150)-JVS(1142)*XX(151)-JVS(1160)*XX(152)-JVS(1193)*XX(153)-JVS(1228)&
              &*XX(154)-JVS(1280)*XX(155)-JVS(1336)*XX(156)-JVS(1365)*XX(157)-JVS(1418)*XX(158)-JVS(1505)*XX(159)-JVS(1549)&
              &*XX(160)-JVS(1657)*XX(161))/(JVS(1738))
  XX(163) = (X(163)-JVS(58)*XX(23)-JVS(64)*XX(25)-JVS(70)*XX(27)-JVS(76)*XX(29)-JVS(81)*XX(30)-JVS(130)*XX(46)-JVS(187)&
              &*XX(58)-JVS(192)*XX(59)-JVS(197)*XX(60)-JVS(210)*XX(64)-JVS(286)*XX(80)-JVS(343)*XX(91)-JVS(349)*XX(92)&
              &-JVS(366)*XX(95)-JVS(373)*XX(96)-JVS(380)*XX(97)-JVS(388)*XX(98)-JVS(408)*XX(100)-JVS(421)*XX(102)-JVS(428)&
              &*XX(103)-JVS(437)*XX(104)-JVS(444)*XX(105)-JVS(455)*XX(106)-JVS(461)*XX(107)-JVS(470)*XX(108)-JVS(488)&
              &*XX(111)-JVS(496)*XX(112)-JVS(509)*XX(113)-JVS(516)*XX(114)-JVS(521)*XX(115)-JVS(551)*XX(116)-JVS(563)&
              &*XX(117)-JVS(578)*XX(118)-JVS(587)*XX(119)-JVS(594)*XX(120)-JVS(604)*XX(121)-JVS(612)*XX(122)-JVS(623)&
              &*XX(123)-JVS(630)*XX(124)-JVS(637)*XX(125)-JVS(649)*XX(126)-JVS(659)*XX(127)-JVS(667)*XX(128)-JVS(682)&
              &*XX(129)-JVS(707)*XX(130)-JVS(715)*XX(131)-JVS(725)*XX(132)-JVS(740)*XX(133)-JVS(755)*XX(134)-JVS(768)&
              &*XX(135)-JVS(776)*XX(136)-JVS(791)*XX(137)-JVS(811)*XX(138)-JVS(853)*XX(139)-JVS(866)*XX(140)-JVS(876)&
              &*XX(141)-JVS(889)*XX(142)-JVS(958)*XX(143)-JVS(974)*XX(144)-JVS(988)*XX(145)-JVS(1003)*XX(146)-JVS(1045)&
              &*XX(147)-JVS(1065)*XX(148)-JVS(1100)*XX(149)-JVS(1125)*XX(150)-JVS(1143)*XX(151)-JVS(1161)*XX(152)-JVS(1194)&
              &*XX(153)-JVS(1229)*XX(154)-JVS(1281)*XX(155)-JVS(1337)*XX(156)-JVS(1366)*XX(157)-JVS(1419)*XX(158)-JVS(1506)&
              &*XX(159)-JVS(1550)*XX(160)-JVS(1658)*XX(161)-JVS(1739)*XX(162))/(JVS(1799))
  XX(164) = (X(164)-JVS(52)*XX(21)-JVS(200)*XX(61)-JVS(206)*XX(63)-JVS(224)*XX(68)-JVS(294)*XX(82)-JVS(474)*XX(109)&
              &-JVS(480)*XX(110)-JVS(497)*XX(112)-JVS(522)*XX(115)-JVS(552)*XX(116)-JVS(579)*XX(118)-JVS(613)*XX(122)&
              &-JVS(624)*XX(123)-JVS(631)*XX(124)-JVS(638)*XX(125)-JVS(660)*XX(127)-JVS(668)*XX(128)-JVS(683)*XX(129)&
              &-JVS(726)*XX(132)-JVS(741)*XX(133)-JVS(756)*XX(134)-JVS(769)*XX(135)-JVS(777)*XX(136)-JVS(792)*XX(137)&
              &-JVS(812)*XX(138)-JVS(854)*XX(139)-JVS(867)*XX(140)-JVS(877)*XX(141)-JVS(890)*XX(142)-JVS(959)*XX(143)&
              &-JVS(975)*XX(144)-JVS(989)*XX(145)-JVS(1004)*XX(146)-JVS(1046)*XX(147)-JVS(1066)*XX(148)-JVS(1101)*XX(149)&
              &-JVS(1126)*XX(150)-JVS(1144)*XX(151)-JVS(1162)*XX(152)-JVS(1195)*XX(153)-JVS(1230)*XX(154)-JVS(1282)*XX(155)&
              &-JVS(1338)*XX(156)-JVS(1367)*XX(157)-JVS(1420)*XX(158)-JVS(1507)*XX(159)-JVS(1551)*XX(160)-JVS(1659)*XX(161)&
              &-JVS(1740)*XX(162)-JVS(1800)*XX(163))/(JVS(1859))
  XX(164) = XX(164)
  XX(163) = XX(163)-JVS(1858)*XX(164)
  XX(162) = XX(162)-JVS(1798)*XX(163)-JVS(1857)*XX(164)
  XX(161) = XX(161)-JVS(1737)*XX(162)-JVS(1797)*XX(163)-JVS(1856)*XX(164)
  XX(160) = XX(160)-JVS(1655)*XX(161)-JVS(1736)*XX(162)-JVS(1796)*XX(163)-JVS(1855)*XX(164)
  XX(159) = XX(159)-JVS(1546)*XX(160)-JVS(1654)*XX(161)-JVS(1735)*XX(162)-JVS(1795)*XX(163)-JVS(1854)*XX(164)
  XX(158) = XX(158)-JVS(1501)*XX(159)-JVS(1545)*XX(160)-JVS(1653)*XX(161)-JVS(1734)*XX(162)-JVS(1794)*XX(163)-JVS(1853)&
              &*XX(164)
  XX(157) = XX(157)-JVS(1413)*XX(158)-JVS(1500)*XX(159)-JVS(1544)*XX(160)-JVS(1652)*XX(161)-JVS(1733)*XX(162)-JVS(1793)&
              &*XX(163)-JVS(1852)*XX(164)
  XX(156) = XX(156)-JVS(1359)*XX(157)-JVS(1412)*XX(158)-JVS(1499)*XX(159)-JVS(1543)*XX(160)-JVS(1651)*XX(161)-JVS(1732)&
              &*XX(162)-JVS(1792)*XX(163)-JVS(1851)*XX(164)
  XX(155) = XX(155)-JVS(1329)*XX(156)-JVS(1358)*XX(157)-JVS(1411)*XX(158)-JVS(1498)*XX(159)-JVS(1542)*XX(160)-JVS(1650)&
              &*XX(161)-JVS(1731)*XX(162)-JVS(1791)*XX(163)-JVS(1850)*XX(164)
  XX(154) = XX(154)-JVS(1272)*XX(155)-JVS(1328)*XX(156)-JVS(1357)*XX(157)-JVS(1410)*XX(158)-JVS(1497)*XX(159)-JVS(1541)&
              &*XX(160)-JVS(1649)*XX(161)-JVS(1730)*XX(162)-JVS(1790)*XX(163)-JVS(1849)*XX(164)
  XX(153) = XX(153)-JVS(1219)*XX(154)-JVS(1271)*XX(155)-JVS(1327)*XX(156)-JVS(1356)*XX(157)-JVS(1409)*XX(158)-JVS(1496)&
              &*XX(159)-JVS(1540)*XX(160)-JVS(1648)*XX(161)-JVS(1729)*XX(162)-JVS(1789)*XX(163)-JVS(1848)*XX(164)
  XX(152) = XX(152)-JVS(1184)*XX(153)-JVS(1218)*XX(154)-JVS(1270)*XX(155)-JVS(1326)*XX(156)-JVS(1355)*XX(157)-JVS(1408)&
              &*XX(158)-JVS(1495)*XX(159)-JVS(1539)*XX(160)-JVS(1647)*XX(161)-JVS(1728)*XX(162)-JVS(1788)*XX(163)-JVS(1847)&
              &*XX(164)
  XX(151) = XX(151)-JVS(1152)*XX(152)-JVS(1183)*XX(153)-JVS(1217)*XX(154)-JVS(1269)*XX(155)-JVS(1325)*XX(156)-JVS(1354)&
              &*XX(157)-JVS(1407)*XX(158)-JVS(1494)*XX(159)-JVS(1538)*XX(160)-JVS(1646)*XX(161)-JVS(1727)*XX(162)-JVS(1787)&
              &*XX(163)-JVS(1846)*XX(164)
  XX(150) = XX(150)-JVS(1182)*XX(153)-JVS(1216)*XX(154)-JVS(1268)*XX(155)-JVS(1324)*XX(156)-JVS(1406)*XX(158)-JVS(1493)&
              &*XX(159)-JVS(1537)*XX(160)-JVS(1645)*XX(161)-JVS(1726)*XX(162)-JVS(1786)*XX(163)-JVS(1845)*XX(164)
  XX(149) = XX(149)-JVS(1215)*XX(154)-JVS(1267)*XX(155)-JVS(1323)*XX(156)-JVS(1405)*XX(158)-JVS(1492)*XX(159)-JVS(1644)&
              &*XX(161)-JVS(1725)*XX(162)-JVS(1844)*XX(164)
  XX(148) = XX(148)-JVS(1085)*XX(149)-JVS(1114)*XX(150)-JVS(1181)*XX(153)-JVS(1214)*XX(154)-JVS(1266)*XX(155)-JVS(1322)&
              &*XX(156)-JVS(1404)*XX(158)-JVS(1491)*XX(159)-JVS(1536)*XX(160)-JVS(1643)*XX(161)-JVS(1724)*XX(162)-JVS(1785)&
              &*XX(163)-JVS(1843)*XX(164)
  XX(147) = XX(147)-JVS(1213)*XX(154)-JVS(1265)*XX(155)-JVS(1321)*XX(156)-JVS(1403)*XX(158)-JVS(1490)*XX(159)-JVS(1642)&
              &*XX(161)-JVS(1723)*XX(162)-JVS(1842)*XX(164)
  XX(146) = XX(146)-JVS(1030)*XX(147)-JVS(1056)*XX(148)-JVS(1084)*XX(149)-JVS(1113)*XX(150)-JVS(1180)*XX(153)-JVS(1212)&
              &*XX(154)-JVS(1264)*XX(155)-JVS(1320)*XX(156)-JVS(1402)*XX(158)-JVS(1489)*XX(159)-JVS(1535)*XX(160)-JVS(1641)&
              &*XX(161)-JVS(1722)*XX(162)-JVS(1784)*XX(163)-JVS(1841)*XX(164)
  XX(145) = XX(145)-JVS(995)*XX(146)-JVS(1029)*XX(147)-JVS(1055)*XX(148)-JVS(1083)*XX(149)-JVS(1112)*XX(150)-JVS(1211)&
              &*XX(154)-JVS(1263)*XX(155)-JVS(1319)*XX(156)-JVS(1401)*XX(158)-JVS(1488)*XX(159)-JVS(1534)*XX(160)-JVS(1640)&
              &*XX(161)-JVS(1721)*XX(162)-JVS(1783)*XX(163)-JVS(1840)*XX(164)
  XX(144) = XX(144)-JVS(1028)*XX(147)-JVS(1179)*XX(153)-JVS(1210)*XX(154)-JVS(1262)*XX(155)-JVS(1318)*XX(156)-JVS(1400)&
              &*XX(158)-JVS(1487)*XX(159)-JVS(1533)*XX(160)-JVS(1639)*XX(161)-JVS(1720)*XX(162)-JVS(1782)*XX(163)-JVS(1839)&
              &*XX(164)
  XX(143) = XX(143)-JVS(1209)*XX(154)-JVS(1317)*XX(156)-JVS(1486)*XX(159)-JVS(1638)*XX(161)-JVS(1719)*XX(162)-JVS(1838)&
              &*XX(164)
  XX(142) = XX(142)-JVS(937)*XX(143)-JVS(963)*XX(144)-JVS(1027)*XX(147)-JVS(1054)*XX(148)-JVS(1082)*XX(149)-JVS(1133)&
              &*XX(151)-JVS(1151)*XX(152)-JVS(1178)*XX(153)-JVS(1261)*XX(155)-JVS(1316)*XX(156)-JVS(1353)*XX(157)-JVS(1399)&
              &*XX(158)-JVS(1485)*XX(159)-JVS(1532)*XX(160)-JVS(1637)*XX(161)-JVS(1718)*XX(162)-JVS(1781)*XX(163)-JVS(1837)&
              &*XX(164)
  XX(141) = XX(141)-JVS(882)*XX(142)-JVS(936)*XX(143)-JVS(979)*XX(145)-JVS(994)*XX(146)-JVS(1026)*XX(147)-JVS(1053)&
              &*XX(148)-JVS(1132)*XX(151)-JVS(1150)*XX(152)-JVS(1177)*XX(153)-JVS(1260)*XX(155)-JVS(1315)*XX(156)-JVS(1352)&
              &*XX(157)-JVS(1398)*XX(158)-JVS(1484)*XX(159)-JVS(1531)*XX(160)-JVS(1636)*XX(161)-JVS(1717)*XX(162)-JVS(1780)&
              &*XX(163)-JVS(1836)*XX(164)
  XX(140) = XX(140)-JVS(881)*XX(142)-JVS(935)*XX(143)-JVS(993)*XX(146)-JVS(1025)*XX(147)-JVS(1081)*XX(149)-JVS(1111)&
              &*XX(150)-JVS(1131)*XX(151)-JVS(1149)*XX(152)-JVS(1176)*XX(153)-JVS(1259)*XX(155)-JVS(1314)*XX(156)-JVS(1351)&
              &*XX(157)-JVS(1397)*XX(158)-JVS(1483)*XX(159)-JVS(1530)*XX(160)-JVS(1635)*XX(161)-JVS(1716)*XX(162)-JVS(1779)&
              &*XX(163)-JVS(1835)*XX(164)
  XX(139) = XX(139)-JVS(1258)*XX(155)-JVS(1313)*XX(156)-JVS(1482)*XX(159)-JVS(1634)*XX(161)-JVS(1715)*XX(162)-JVS(1834)&
              &*XX(164)
  XX(138) = XX(138)-JVS(831)*XX(139)-JVS(934)*XX(143)-JVS(962)*XX(144)-JVS(1024)*XX(147)-JVS(1080)*XX(149)-JVS(1130)&
              &*XX(151)-JVS(1148)*XX(152)-JVS(1257)*XX(155)-JVS(1312)*XX(156)-JVS(1396)*XX(158)-JVS(1481)*XX(159)-JVS(1633)&
              &*XX(161)-JVS(1714)*XX(162)-JVS(1778)*XX(163)-JVS(1833)*XX(164)
  XX(137) = XX(137)-JVS(933)*XX(143)-JVS(1023)*XX(147)-JVS(1175)*XX(153)-JVS(1256)*XX(155)-JVS(1311)*XX(156)-JVS(1350)&
              &*XX(157)-JVS(1395)*XX(158)-JVS(1480)*XX(159)-JVS(1529)*XX(160)-JVS(1632)*XX(161)-JVS(1713)*XX(162)-JVS(1777)&
              &*XX(163)-JVS(1832)*XX(164)
  XX(136) = XX(136)-JVS(781)*XX(137)-JVS(859)*XX(140)-JVS(932)*XX(143)-JVS(1022)*XX(147)-JVS(1079)*XX(149)-JVS(1110)&
              &*XX(150)-JVS(1174)*XX(153)-JVS(1255)*XX(155)-JVS(1310)*XX(156)-JVS(1349)*XX(157)-JVS(1394)*XX(158)-JVS(1479)&
              &*XX(159)-JVS(1528)*XX(160)-JVS(1631)*XX(161)-JVS(1712)*XX(162)-JVS(1776)*XX(163)-JVS(1831)*XX(164)
  XX(135) = XX(135)-JVS(931)*XX(143)-JVS(1052)*XX(148)-JVS(1078)*XX(149)-JVS(1109)*XX(150)-JVS(1254)*XX(155)-JVS(1393)&
              &*XX(158)-JVS(1478)*XX(159)-JVS(1527)*XX(160)-JVS(1630)*XX(161)-JVS(1711)*XX(162)-JVS(1775)*XX(163)
  XX(134) = XX(134)-JVS(830)*XX(139)-JVS(930)*XX(143)-JVS(1021)*XX(147)-JVS(1108)*XX(150)-JVS(1173)*XX(153)-JVS(1253)&
              &*XX(155)-JVS(1392)*XX(158)-JVS(1477)*XX(159)-JVS(1629)*XX(161)-JVS(1710)*XX(162)-JVS(1774)*XX(163)
  XX(133) = XX(133)-JVS(929)*XX(143)-JVS(1077)*XX(149)-JVS(1252)*XX(155)-JVS(1348)*XX(157)-JVS(1391)*XX(158)-JVS(1476)&
              &*XX(159)-JVS(1628)*XX(161)-JVS(1709)*XX(162)-JVS(1773)*XX(163)
  XX(132) = XX(132)-JVS(829)*XX(139)-JVS(928)*XX(143)-JVS(1172)*XX(153)-JVS(1251)*XX(155)-JVS(1309)*XX(156)-JVS(1390)&
              &*XX(158)-JVS(1475)*XX(159)-JVS(1526)*XX(160)-JVS(1627)*XX(161)-JVS(1708)*XX(162)-JVS(1772)*XX(163)-JVS(1830)&
              &*XX(164)
  XX(131) = XX(131)-JVS(828)*XX(139)-JVS(927)*XX(143)-JVS(1020)*XX(147)-JVS(1051)*XX(148)-JVS(1171)*XX(153)-JVS(1250)&
              &*XX(155)-JVS(1308)*XX(156)-JVS(1347)*XX(157)-JVS(1389)*XX(158)-JVS(1474)*XX(159)-JVS(1525)*XX(160)-JVS(1626)&
              &*XX(161)-JVS(1707)*XX(162)-JVS(1771)*XX(163)-JVS(1829)*XX(164)
  XX(130) = XX(130)-JVS(926)*XX(143)-JVS(1307)*XX(156)-JVS(1473)*XX(159)-JVS(1625)*XX(161)-JVS(1706)*XX(162)-JVS(1828)&
              &*XX(164)
  XX(129) = XX(129)-JVS(925)*XX(143)-JVS(1076)*XX(149)-JVS(1249)*XX(155)-JVS(1388)*XX(158)-JVS(1472)*XX(159)-JVS(1624)&
              &*XX(161)-JVS(1705)*XX(162)-JVS(1770)*XX(163)
  XX(128) = XX(128)-JVS(745)*XX(134)-JVS(827)*XX(139)-JVS(924)*XX(143)-JVS(1075)*XX(149)-JVS(1107)*XX(150)-JVS(1170)&
              &*XX(153)-JVS(1248)*XX(155)-JVS(1387)*XX(158)-JVS(1471)*XX(159)-JVS(1524)*XX(160)-JVS(1623)*XX(161)-JVS(1704)&
              &*XX(162)-JVS(1769)*XX(163)
  XX(127) = XX(127)-JVS(826)*XX(139)-JVS(923)*XX(143)-JVS(1074)*XX(149)-JVS(1247)*XX(155)-JVS(1306)*XX(156)-JVS(1386)&
              &*XX(158)-JVS(1470)*XX(159)-JVS(1523)*XX(160)-JVS(1622)*XX(161)-JVS(1703)*XX(162)-JVS(1768)*XX(163)
  XX(126) = XX(126)-JVS(744)*XX(134)-JVS(922)*XX(143)-JVS(1208)*XX(154)-JVS(1246)*XX(155)-JVS(1305)*XX(156)-JVS(1385)&
              &*XX(158)-JVS(1469)*XX(159)-JVS(1522)*XX(160)-JVS(1621)*XX(161)-JVS(1702)*XX(162)-JVS(1767)*XX(163)
  XX(125) = XX(125)-JVS(758)*XX(135)-JVS(825)*XX(139)-JVS(921)*XX(143)-JVS(978)*XX(145)-JVS(992)*XX(146)-JVS(1106)&
              &*XX(150)-JVS(1245)*XX(155)-JVS(1304)*XX(156)-JVS(1384)*XX(158)-JVS(1468)*XX(159)-JVS(1521)*XX(160)-JVS(1620)&
              &*XX(161)-JVS(1701)*XX(162)-JVS(1766)*XX(163)
  XX(124) = XX(124)-JVS(729)*XX(133)-JVS(780)*XX(137)-JVS(920)*XX(143)-JVS(1019)*XX(147)-JVS(1073)*XX(149)-JVS(1105)&
              &*XX(150)-JVS(1244)*XX(155)-JVS(1303)*XX(156)-JVS(1346)*XX(157)-JVS(1467)*XX(159)-JVS(1619)*XX(161)-JVS(1700)&
              &*XX(162)-JVS(1765)*XX(163)-JVS(1827)*XX(164)
  XX(123) = XX(123)-JVS(919)*XX(143)-JVS(1018)*XX(147)-JVS(1072)*XX(149)-JVS(1243)*XX(155)-JVS(1383)*XX(158)-JVS(1466)&
              &*XX(159)-JVS(1520)*XX(160)-JVS(1618)*XX(161)-JVS(1699)*XX(162)-JVS(1764)*XX(163)
  XX(122) = XX(122)-JVS(802)*XX(138)-JVS(918)*XX(143)-JVS(961)*XX(144)-JVS(1017)*XX(147)-JVS(1129)*XX(151)-JVS(1302)&
              &*XX(156)-JVS(1465)*XX(159)-JVS(1617)*XX(161)-JVS(1698)*XX(162)-JVS(1763)*XX(163)-JVS(1826)*XX(164)
  XX(121) = XX(121)-JVS(824)*XX(139)-JVS(917)*XX(143)-JVS(1016)*XX(147)-JVS(1301)*XX(156)-JVS(1464)*XX(159)-JVS(1616)&
              &*XX(161)-JVS(1697)*XX(162)-JVS(1762)*XX(163)-JVS(1825)*XX(164)
  XX(120) = XX(120)-JVS(690)*XX(130)-JVS(823)*XX(139)-JVS(880)*XX(142)-JVS(916)*XX(143)-JVS(991)*XX(146)-JVS(1015)&
              &*XX(147)-JVS(1300)*XX(156)-JVS(1345)*XX(157)-JVS(1382)*XX(158)-JVS(1463)*XX(159)-JVS(1615)*XX(161)-JVS(1696)&
              &*XX(162)
  XX(119) = XX(119)-JVS(915)*XX(143)-JVS(1014)*XX(147)-JVS(1169)*XX(153)-JVS(1242)*XX(155)-JVS(1381)*XX(158)-JVS(1462)&
              &*XX(159)-JVS(1519)*XX(160)-JVS(1614)*XX(161)-JVS(1695)*XX(162)-JVS(1761)*XX(163)-JVS(1824)*XX(164)
  XX(118) = XX(118)-JVS(914)*XX(143)-JVS(1299)*XX(156)-JVS(1461)*XX(159)-JVS(1613)*XX(161)-JVS(1694)*XX(162)-JVS(1823)&
              &*XX(164)
  XX(117) = XX(117)-JVS(913)*XX(143)-JVS(1241)*XX(155)-JVS(1380)*XX(158)-JVS(1460)*XX(159)-JVS(1518)*XX(160)-JVS(1612)&
              &*XX(161)-JVS(1693)*XX(162)-JVS(1760)*XX(163)
  XX(116) = XX(116)-JVS(1611)*XX(161)-JVS(1692)*XX(162)-JVS(1822)*XX(164)
  XX(115) = XX(115)-JVS(567)*XX(118)-JVS(858)*XX(140)-JVS(912)*XX(143)-JVS(1104)*XX(150)-JVS(1168)*XX(153)-JVS(1379)&
              &*XX(158)-JVS(1459)*XX(159)-JVS(1517)*XX(160)-JVS(1610)*XX(161)-JVS(1691)*XX(162)-JVS(1759)*XX(163)-JVS(1821)&
              &*XX(164)
  XX(114) = XX(114)-JVS(801)*XX(138)-JVS(822)*XX(139)-JVS(1240)*XX(155)-JVS(1298)*XX(156)-JVS(1378)*XX(158)-JVS(1458)&
              &*XX(159)-JVS(1609)*XX(161)-JVS(1690)*XX(162)-JVS(1758)*XX(163)-JVS(1820)*XX(164)
  XX(113) = XX(113)-JVS(821)*XX(139)-JVS(911)*XX(143)-JVS(1297)*XX(156)-JVS(1377)*XX(158)-JVS(1608)*XX(161)-JVS(1689)&
              &*XX(162)
  XX(112) = XX(112)-JVS(530)*XX(116)-JVS(1207)*XX(154)-JVS(1344)*XX(157)-JVS(1457)*XX(159)-JVS(1607)*XX(161)-JVS(1688)&
              &*XX(162)-JVS(1757)*XX(163)-JVS(1819)*XX(164)
  XX(111) = XX(111)-JVS(800)*XX(138)-JVS(820)*XX(139)-JVS(1296)*XX(156)-JVS(1456)*XX(159)-JVS(1606)*XX(161)-JVS(1687)&
              &*XX(162)-JVS(1756)*XX(163)-JVS(1818)*XX(164)
  XX(110) = XX(110)-JVS(615)*XX(123)-JVS(651)*XX(127)-JVS(799)*XX(138)-JVS(910)*XX(143)-JVS(1071)*XX(149)-JVS(1295)&
              &*XX(156)-JVS(1343)*XX(157)-JVS(1376)*XX(158)-JVS(1455)*XX(159)-JVS(1605)*XX(161)-JVS(1686)*XX(162)-JVS(1755)&
              &*XX(163)-JVS(1817)*XX(164)
  XX(109) = XX(109)-JVS(475)*XX(110)-JVS(605)*XX(122)-JVS(717)*XX(132)-JVS(798)*XX(138)-JVS(857)*XX(140)-JVS(869)&
              &*XX(141)-JVS(909)*XX(143)-JVS(1128)*XX(151)-JVS(1147)*XX(152)-JVS(1239)*XX(155)-JVS(1294)*XX(156)-JVS(1342)&
              &*XX(157)-JVS(1375)*XX(158)-JVS(1454)*XX(159)-JVS(1604)*XX(161)-JVS(1685)*XX(162)-JVS(1754)*XX(163)-JVS(1816)&
              &*XX(164)
  XX(108) = XX(108)-JVS(797)*XX(138)-JVS(908)*XX(143)-JVS(1453)*XX(159)-JVS(1603)*XX(161)-JVS(1684)*XX(162)-JVS(1753)&
              &*XX(163)-JVS(1815)*XX(164)
  XX(107) = XX(107)-JVS(566)*XX(118)-JVS(856)*XX(140)-JVS(907)*XX(143)-JVS(1103)*XX(150)-JVS(1167)*XX(153)-JVS(1374)&
              &*XX(158)-JVS(1452)*XX(159)-JVS(1516)*XX(160)-JVS(1602)*XX(161)-JVS(1683)*XX(162)-JVS(1752)*XX(163)
  XX(106) = XX(106)-JVS(743)*XX(134)-JVS(1206)*XX(154)-JVS(1238)*XX(155)-JVS(1293)*XX(156)-JVS(1373)*XX(158)-JVS(1601)&
              &*XX(161)
  XX(105) = XX(105)-JVS(689)*XX(130)-JVS(796)*XX(138)-JVS(1013)*XX(147)-JVS(1292)*XX(156)-JVS(1451)*XX(159)-JVS(1600)&
              &*XX(161)-JVS(1682)*XX(162)-JVS(1751)*XX(163)-JVS(1814)*XX(164)
  XX(104) = XX(104)-JVS(688)*XX(130)-JVS(819)*XX(139)-JVS(906)*XX(143)-JVS(1237)*XX(155)-JVS(1599)*XX(161)-JVS(1681)&
              &*XX(162)-JVS(1813)*XX(164)
  XX(103) = XX(103)-JVS(795)*XX(138)-JVS(1070)*XX(149)-JVS(1291)*XX(156)-JVS(1450)*XX(159)-JVS(1598)*XX(161)-JVS(1680)&
              &*XX(162)-JVS(1750)*XX(163)-JVS(1812)*XX(164)
  XX(102) = XX(102)-JVS(794)*XX(138)-JVS(1146)*XX(152)-JVS(1290)*XX(156)-JVS(1449)*XX(159)-JVS(1597)*XX(161)-JVS(1679)&
              &*XX(162)-JVS(1749)*XX(163)-JVS(1811)*XX(164)
  XX(101) = XX(101)-JVS(779)*XX(137)-JVS(905)*XX(143)-JVS(1012)*XX(147)-JVS(1236)*XX(155)-JVS(1289)*XX(156)-JVS(1341)&
              &*XX(157)-JVS(1596)*XX(161)-JVS(1678)*XX(162)-JVS(1810)*XX(164)
  XX(100) = XX(100)-JVS(529)*XX(116)-JVS(565)*XX(118)-JVS(687)*XX(130)-JVS(818)*XX(139)-JVS(1011)*XX(147)-JVS(1050)&
              &*XX(148)-JVS(1448)*XX(159)-JVS(1515)*XX(160)-JVS(1595)*XX(161)-JVS(1677)*XX(162)-JVS(1748)*XX(163)
  XX(99) = XX(99)-JVS(1447)*XX(159)-JVS(1514)*XX(160)-JVS(1594)*XX(161)
  XX(98) = XX(98)-JVS(528)*XX(116)-JVS(1049)*XX(148)-JVS(1446)*XX(159)-JVS(1513)*XX(160)-JVS(1593)*XX(161)-JVS(1676)&
             &*XX(162)-JVS(1747)*XX(163)
  XX(97) = XX(97)-JVS(817)*XX(139)-JVS(904)*XX(143)-JVS(1010)*XX(147)-JVS(1445)*XX(159)-JVS(1592)*XX(161)-JVS(1675)&
             &*XX(162)-JVS(1746)*XX(163)-JVS(1809)*XX(164)
  XX(96) = XX(96)-JVS(564)*XX(118)-JVS(686)*XX(130)-JVS(816)*XX(139)-JVS(903)*XX(143)-JVS(1009)*XX(147)-JVS(1288)&
             &*XX(156)-JVS(1444)*XX(159)-JVS(1591)*XX(161)-JVS(1674)*XX(162)-JVS(1745)*XX(163)
  XX(95) = XX(95)-JVS(429)*XX(104)-JVS(595)*XX(121)-JVS(879)*XX(142)-JVS(902)*XX(143)-JVS(977)*XX(145)-JVS(1340)*XX(157)&
             &-JVS(1443)*XX(159)-JVS(1590)*XX(161)-JVS(1673)*XX(162)
  XX(94) = XX(94)-JVS(757)*XX(135)-JVS(976)*XX(145)-JVS(990)*XX(146)-JVS(1442)*XX(159)-JVS(1512)*XX(160)-JVS(1589)&
             &*XX(161)-JVS(1672)*XX(162)
  XX(93) = XX(93)-JVS(855)*XX(140)-JVS(1102)*XX(150)-JVS(1166)*XX(153)-JVS(1441)*XX(159)-JVS(1511)*XX(160)-JVS(1588)&
             &*XX(161)-JVS(1671)*XX(162)
  XX(92) = XX(92)-JVS(381)*XX(98)-JVS(1165)*XX(153)-JVS(1339)*XX(157)-JVS(1440)*XX(159)-JVS(1587)*XX(161)
  XX(91) = XX(91)-JVS(1008)*XX(147)-JVS(1164)*XX(153)-JVS(1287)*XX(156)-JVS(1439)*XX(159)-JVS(1586)*XX(161)
  XX(90) = XX(90)-JVS(901)*XX(143)-JVS(960)*XX(144)-JVS(1007)*XX(147)-JVS(1286)*XX(156)-JVS(1438)*XX(159)-JVS(1585)&
             &*XX(161)
  XX(89) = XX(89)-JVS(337)*XX(91)-JVS(685)*XX(130)-JVS(708)*XX(131)-JVS(815)*XX(139)-JVS(1006)*XX(147)-JVS(1285)*XX(156)&
             &-JVS(1437)*XX(159)-JVS(1584)*XX(161)
  XX(88) = XX(88)-JVS(368)*XX(96)-JVS(868)*XX(141)-JVS(878)*XX(142)-JVS(900)*XX(143)-JVS(1127)*XX(151)-JVS(1145)*XX(152)&
             &-JVS(1436)*XX(159)-JVS(1583)*XX(161)
  XX(87) = XX(87)-JVS(684)*XX(130)-JVS(716)*XX(132)-JVS(814)*XX(139)-JVS(899)*XX(143)-JVS(1163)*XX(153)-JVS(1235)&
             &*XX(155)-JVS(1435)*XX(159)-JVS(1582)*XX(161)
  XX(86) = XX(86)-JVS(1048)*XX(148)-JVS(1434)*XX(159)-JVS(1510)*XX(160)-JVS(1581)*XX(161)
  XX(85) = XX(85)-JVS(742)*XX(134)-JVS(813)*XX(139)-JVS(898)*XX(143)-JVS(1234)*XX(155)-JVS(1580)*XX(161)
  XX(84) = XX(84)-JVS(498)*XX(113)-JVS(650)*XX(127)-JVS(1433)*XX(159)-JVS(1509)*XX(160)-JVS(1579)*XX(161)-JVS(1670)&
             &*XX(162)
  XX(83) = XX(83)-JVS(1432)*XX(159)-JVS(1578)*XX(161)
  XX(82) = XX(82)-JVS(527)*XX(116)-JVS(670)*XX(129)-JVS(1205)*XX(154)-JVS(1284)*XX(156)-JVS(1577)*XX(161)-JVS(1808)&
             &*XX(164)
  XX(81) = XX(81)-JVS(614)*XX(123)-JVS(897)*XX(143)-JVS(1005)*XX(147)-JVS(1069)*XX(149)-JVS(1431)*XX(159)-JVS(1576)&
             &*XX(161)
  XX(80) = XX(80)-JVS(896)*XX(143)-JVS(1575)*XX(161)-JVS(1669)*XX(162)
  XX(79) = XX(79)-JVS(489)*XX(112)-JVS(526)*XX(116)-JVS(1204)*XX(154)-JVS(1574)*XX(161)-JVS(1668)*XX(162)-JVS(1807)&
             &*XX(164)
  XX(78) = XX(78)-JVS(728)*XX(133)-JVS(1068)*XX(149)-JVS(1430)*XX(159)-JVS(1573)*XX(161)
  XX(77) = XX(77)-JVS(580)*XX(119)-JVS(778)*XX(137)-JVS(895)*XX(143)-JVS(1233)*XX(155)-JVS(1572)*XX(161)
  XX(76) = XX(76)-JVS(1203)*XX(154)-JVS(1571)*XX(161)
  XX(75) = XX(75)-JVS(1202)*XX(154)-JVS(1570)*XX(161)
  XX(74) = XX(74)-JVS(250)*XX(75)-JVS(1201)*XX(154)-JVS(1569)*XX(161)
  XX(73) = XX(73)-JVS(894)*XX(143)-JVS(1372)*XX(158)-JVS(1429)*XX(159)-JVS(1568)*XX(161)
  XX(72) = XX(72)-JVS(1428)*XX(159)-JVS(1567)*XX(161)-JVS(1667)*XX(162)-JVS(1806)*XX(164)
  XX(71) = XX(71)-JVS(669)*XX(129)-JVS(1067)*XX(149)-JVS(1427)*XX(159)-JVS(1566)*XX(161)
  XX(70) = XX(70)-JVS(258)*XX(76)-JVS(554)*XX(117)-JVS(640)*XX(126)-JVS(1200)*XX(154)-JVS(1283)*XX(156)-JVS(1565)&
             &*XX(161)
  XX(69) = XX(69)-JVS(553)*XX(117)-JVS(1426)*XX(159)-JVS(1508)*XX(160)-JVS(1564)*XX(161)
  XX(68) = XX(68)-JVS(525)*XX(116)-JVS(793)*XX(138)-JVS(893)*XX(143)-JVS(1371)*XX(158)-JVS(1563)*XX(161)-JVS(1805)&
             &*XX(164)
  XX(67) = XX(67)-JVS(445)*XX(106)-JVS(639)*XX(126)-JVS(1425)*XX(159)-JVS(1562)*XX(161)
  XX(66) = XX(66)-JVS(892)*XX(143)-JVS(1370)*XX(158)-JVS(1424)*XX(159)-JVS(1666)*XX(162)-JVS(1804)*XX(164)
  XX(65) = XX(65)-JVS(1232)*XX(155)-JVS(1369)*XX(158)-JVS(1561)*XX(161)
  XX(64) = XX(64)-JVS(1560)*XX(161)-JVS(1665)*XX(162)-JVS(1744)*XX(163)
  XX(63) = XX(63)-JVS(524)*XX(116)-JVS(1047)*XX(148)-JVS(1559)*XX(161)-JVS(1803)*XX(164)
  XX(62) = XX(62)-JVS(1231)*XX(155)-JVS(1368)*XX(158)-JVS(1664)*XX(162)-JVS(1802)*XX(164)
  XX(61) = XX(61)-JVS(523)*XX(116)-JVS(1663)*XX(162)-JVS(1801)*XX(164)
  XX(60) = XX(60)-JVS(1423)*XX(159)-JVS(1743)*XX(163)
  XX(59) = XX(59)-JVS(1422)*XX(159)-JVS(1742)*XX(163)
  XX(58) = XX(58)-JVS(1421)*XX(159)-JVS(1741)*XX(163)
  XX(57) = XX(57)-JVS(374)*XX(97)-JVS(1662)*XX(162)
  XX(56) = XX(56)-JVS(367)*XX(96)-JVS(1558)*XX(161)
  XX(55) = XX(55)-JVS(1199)*XX(154)-JVS(1661)*XX(162)
  XX(54) = XX(54)-JVS(727)*XX(133)-JVS(1660)*XX(162)
  XX(53) = XX(53)-JVS(183)*XX(58)-JVS(1557)*XX(161)
  XX(52) = XX(52)-JVS(1198)*XX(154)-JVS(1556)*XX(161)
  XX(51) = XX(51)-JVS(1197)*XX(154)-JVS(1555)*XX(161)
  XX(50) = XX(50)-JVS(1196)*XX(154)-JVS(1554)*XX(161)
  XX(49) = XX(49)-JVS(193)*XX(60)-JVS(1553)*XX(161)
  XX(48) = XX(48)-JVS(188)*XX(59)-JVS(1552)*XX(161)
  XX(47) = XX(47)
  XX(46) = XX(46)
  XX(45) = XX(45)
  XX(44) = XX(44)
  XX(43) = XX(43)
  XX(42) = XX(42)
  XX(41) = XX(41)
  XX(40) = XX(40)
  XX(39) = XX(39)
  XX(38) = XX(38)
  XX(37) = XX(37)
  XX(36) = XX(36)
  XX(35) = XX(35)
  XX(34) = XX(34)
  XX(33) = XX(33)
  XX(32) = XX(32)
  XX(31) = XX(31)
  XX(30) = XX(30)
  XX(29) = XX(29)
  XX(28) = XX(28)
  XX(27) = XX(27)
  XX(26) = XX(26)
  XX(25) = XX(25)
  XX(24) = XX(24)
  XX(23) = XX(23)
  XX(22) = XX(22)
  XX(21) = XX(21)
  XX(20) = XX(20)
  XX(19) = XX(19)
  XX(18) = XX(18)
  XX(17) = XX(17)
  XX(16) = XX(16)
  XX(15) = XX(15)
  XX(14) = XX(14)
  XX(13) = XX(13)
  XX(12) = XX(12)
  XX(11) = XX(11)
  XX(10) = XX(10)
  XX(9) = XX(9)
  XX(8) = XX(8)
  XX(7) = XX(7)
  XX(6) = XX(6)
  XX(5) = XX(5)
  XX(4) = XX(4)
  XX(3) = XX(3)
  XX(2) = XX(2)
  XX(1) = XX(1)-JVS(891)*XX(143)
      
END SUBROUTINE KppSolveTR

! End of KppSolveTR function
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~


! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
! 
! BLAS_UTIL - BLAS-LIKE utility functions
!   Arguments :
! 
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

!--------------------------------------------------------------
!
! BLAS/LAPACK-like subroutines used by the integration algorithms
! It is recommended to replace them by calls to the optimized
!      BLAS/LAPACK library for your machine
!
!  (C) Adrian Sandu, Aug. 2004
!      Virginia Polytechnic Institute and State University
!--------------------------------------------------------------


!--------------------------------------------------------------
      SUBROUTINE WCOPY(N,X,incX,Y,incY)
!--------------------------------------------------------------
!     copies a vector, x, to a vector, y:  y <- x
!     only for incX=incY=1
!     after BLAS
!     replace this by the function from the optimized BLAS implementation:
!         CALL  SCOPY(N,X,1,Y,1)   or   CALL  DCOPY(N,X,1,Y,1)
!--------------------------------------------------------------
!     USE gckpp_Precision
      
      INTEGER  :: i,incX,incY,M,MP1,N
      REAL(kind=dp) :: X(N),Y(N)

      IF (N.LE.0) RETURN

      M = MOD(N,8)
      IF( M .NE. 0 ) THEN
        DO i = 1,M
          Y(i) = X(i)
        END DO
        IF( N .LT. 8 ) RETURN
      END IF    
      MP1 = M+1
      DO i = MP1,N,8
        Y(i) = X(i)
        Y(i + 1) = X(i + 1)
        Y(i + 2) = X(i + 2)
        Y(i + 3) = X(i + 3)
        Y(i + 4) = X(i + 4)
        Y(i + 5) = X(i + 5)
        Y(i + 6) = X(i + 6)
        Y(i + 7) = X(i + 7)
      END DO

      END SUBROUTINE WCOPY


!--------------------------------------------------------------
      SUBROUTINE WAXPY(N,Alpha,X,incX,Y,incY)
!--------------------------------------------------------------
!     constant times a vector plus a vector: y <- y + Alpha*x
!     only for incX=incY=1
!     after BLAS
!     replace this by the function from the optimized BLAS implementation:
!         CALL SAXPY(N,Alpha,X,1,Y,1) or  CALL DAXPY(N,Alpha,X,1,Y,1)
!--------------------------------------------------------------

      INTEGER  :: i,incX,incY,M,MP1,N
      REAL(kind=dp) :: X(N),Y(N),Alpha
      REAL(kind=dp), PARAMETER :: ZERO = 0.0_dp

      IF (Alpha .EQ. ZERO) RETURN
      IF (N .LE. 0) RETURN

      M = MOD(N,4)
      IF( M .NE. 0 ) THEN
        DO i = 1,M
          Y(i) = Y(i) + Alpha*X(i)
        END DO
        IF( N .LT. 4 ) RETURN
      END IF
      MP1 = M + 1
      DO i = MP1,N,4
        Y(i) = Y(i) + Alpha*X(i)
        Y(i + 1) = Y(i + 1) + Alpha*X(i + 1)
        Y(i + 2) = Y(i + 2) + Alpha*X(i + 2)
        Y(i + 3) = Y(i + 3) + Alpha*X(i + 3)
      END DO
      
      END SUBROUTINE WAXPY



!--------------------------------------------------------------
      SUBROUTINE WSCAL(N,Alpha,X,incX)
!--------------------------------------------------------------
!     constant times a vector: x(1:N) <- Alpha*x(1:N) 
!     only for incX=incY=1
!     after BLAS
!     replace this by the function from the optimized BLAS implementation:
!         CALL SSCAL(N,Alpha,X,1) or  CALL DSCAL(N,Alpha,X,1)
!--------------------------------------------------------------

      INTEGER  :: i,incX,M,MP1,N
      REAL(kind=dp)  :: X(N),Alpha
      REAL(kind=dp), PARAMETER  :: ZERO=0.0_dp, ONE=1.0_dp

      IF (Alpha .EQ. ONE) RETURN
      IF (N .LE. 0) RETURN

      M = MOD(N,5)
      IF( M .NE. 0 ) THEN
        IF (Alpha .EQ. (-ONE)) THEN
          DO i = 1,M
            X(i) = -X(i)
          END DO
        ELSEIF (Alpha .EQ. ZERO) THEN
          DO i = 1,M
            X(i) = ZERO
          END DO
        ELSE
          DO i = 1,M
            X(i) = Alpha*X(i)
          END DO
        END IF
        IF( N .LT. 5 ) RETURN
      END IF
      MP1 = M + 1
      IF (Alpha .EQ. (-ONE)) THEN
        DO i = MP1,N,5
          X(i)     = -X(i)
          X(i + 1) = -X(i + 1)
          X(i + 2) = -X(i + 2)
          X(i + 3) = -X(i + 3)
          X(i + 4) = -X(i + 4)
        END DO
      ELSEIF (Alpha .EQ. ZERO) THEN
        DO i = MP1,N,5
          X(i)     = ZERO
          X(i + 1) = ZERO
          X(i + 2) = ZERO
          X(i + 3) = ZERO
          X(i + 4) = ZERO
        END DO
      ELSE
        DO i = MP1,N,5
          X(i)     = Alpha*X(i)
          X(i + 1) = Alpha*X(i + 1)
          X(i + 2) = Alpha*X(i + 2)
          X(i + 3) = Alpha*X(i + 3)
          X(i + 4) = Alpha*X(i + 4)
        END DO
      END IF

      END SUBROUTINE WSCAL

!--------------------------------------------------------------
      REAL(kind=dp) FUNCTION WLAMCH( C )
!--------------------------------------------------------------
!     returns epsilon machine
!     after LAPACK
!     replace this by the function from the optimized LAPACK implementation:
!          CALL SLAMCH('E') or CALL DLAMCH('E')
!--------------------------------------------------------------
!      USE gckpp_Precision

      CHARACTER ::  C
      INTEGER    :: i
      REAL(kind=dp), SAVE  ::  Eps
      REAL(kind=dp)  ::  Suma
      REAL(kind=dp), PARAMETER  ::  ONE=1.0_dp, HALF=0.5_dp
      LOGICAL, SAVE   ::  First=.TRUE.
      
!$OMP THREADPRIVATE( Eps, First )

      IF (First) THEN
        First = .FALSE.
        Eps = HALF**(16)
        DO i = 17, 80
          Eps = Eps*HALF
          CALL WLAMCH_ADD(ONE,Eps,Suma)
          IF (Suma.LE.ONE) GOTO 10
        END DO
        PRINT*,'ERROR IN WLAMCH. EPS < ',Eps
        RETURN
10      Eps = Eps*2
        i = i-1      
      END IF

      WLAMCH = Eps

      END FUNCTION WLAMCH
     
      SUBROUTINE WLAMCH_ADD( A, B, Suma )
!      USE gckpp_Precision
      
      REAL(kind=dp) A, B, Suma
      Suma = A + B

      END SUBROUTINE WLAMCH_ADD
!--------------------------------------------------------------


!--------------------------------------------------------------
      SUBROUTINE SET2ZERO(N,Y)
!--------------------------------------------------------------
!     copies zeros into the vector y:  y <- 0
!     after BLAS
!--------------------------------------------------------------
      
      INTEGER ::  i,M,MP1,N
      REAL(kind=dp) ::  Y(N)
      REAL(kind=dp), PARAMETER :: ZERO = 0.0d0

      IF (N.LE.0) RETURN

      M = MOD(N,8)
      IF( M .NE. 0 ) THEN
        DO i = 1,M
          Y(i) = ZERO
        END DO
        IF( N .LT. 8 ) RETURN
      END IF    
      MP1 = M+1
      DO i = MP1,N,8
        Y(i)     = ZERO
        Y(i + 1) = ZERO
        Y(i + 2) = ZERO
        Y(i + 3) = ZERO
        Y(i + 4) = ZERO
        Y(i + 5) = ZERO
        Y(i + 6) = ZERO
        Y(i + 7) = ZERO
      END DO

      END SUBROUTINE SET2ZERO


!--------------------------------------------------------------
      REAL(kind=dp) FUNCTION WDOT (N, DX, incX, DY, incY) 
!--------------------------------------------------------------
!     dot produce: wdot = x(1:N)*y(1:N) 
!     only for incX=incY=1
!     after BLAS
!     replace this by the function from the optimized BLAS implementation:
!         CALL SDOT(N,X,1,Y,1) or  CALL DDOT(N,X,1,Y,1)
!--------------------------------------------------------------
!      USE messy_mecca_kpp_Precision
!--------------------------------------------------------------
      IMPLICIT NONE
      INTEGER :: N, incX, incY
      REAL(kind=dp) :: DX(N), DY(N) 

      INTEGER :: i, IX, IY, M, MP1, NS
                                 
      WDOT = 0.0D0 
      IF (N .LE. 0) RETURN 
      IF (incX .EQ. incY) IF (incX-1) 5,20,60 
!                                                                       
!     Code for unequal or nonpositive increments.                       
!                                                                       
    5 IX = 1 
      IY = 1 
      IF (incX .LT. 0) IX = (-N+1)*incX + 1 
      IF (incY .LT. 0) IY = (-N+1)*incY + 1 
      DO i = 1,N 
        WDOT = WDOT + DX(IX)*DY(IY) 
        IX = IX + incX 
        IY = IY + incY 
      END DO 
      RETURN 
!                                                                       
!     Code for both increments equal to 1.                              
!                                                                       
!     Clean-up loop so remaining vector length is a multiple of 5.      
!                                                                       
   20 M = MOD(N,5) 
      IF (M .EQ. 0) GO TO 40 
      DO i = 1,M 
         WDOT = WDOT + DX(i)*DY(i) 
      END DO 
      IF (N .LT. 5) RETURN 
   40 MP1 = M + 1 
      DO i = MP1,N,5 
          WDOT = WDOT + DX(i)*DY(i) + DX(i+1)*DY(i+1) + DX(i+2)*DY(i+2) +  &
                   DX(i+3)*DY(i+3) + DX(i+4)*DY(i+4)                   
      END DO 
      RETURN 
!                                                                       
!     Code for equal, positive, non-unit increments.                    
!                                                                       
   60 NS = N*incX 
      DO i = 1,NS,incX 
        WDOT = WDOT + DX(i)*DY(i) 
      END DO 

      END FUNCTION WDOT                                          


!--------------------------------------------------------------
      SUBROUTINE WADD(N,X,Y,Z)
!--------------------------------------------------------------
!     adds two vectors: z <- x + y
!     BLAS - like
!--------------------------------------------------------------
!     USE gckpp_Precision
      
      INTEGER :: i, M, MP1, N
      REAL(kind=dp) :: X(N),Y(N),Z(N)

      IF (N.LE.0) RETURN

      M = MOD(N,5)
      IF( M /= 0 ) THEN
         DO i = 1,M
            Z(i) = X(i) + Y(i)
         END DO
         IF( N < 5 ) RETURN
      END IF    
      MP1 = M+1
      DO i = MP1,N,5
         Z(i)     = X(i)     + Y(i)
         Z(i + 1) = X(i + 1) + Y(i + 1)
         Z(i + 2) = X(i + 2) + Y(i + 2)
         Z(i + 3) = X(i + 3) + Y(i + 3)
         Z(i + 4) = X(i + 4) + Y(i + 4)
      END DO

      END SUBROUTINE WADD
      
      
      
!--------------------------------------------------------------
      SUBROUTINE WGEFA(N,A,Ipvt,info)
!--------------------------------------------------------------
!     WGEFA FACTORS THE MATRIX A (N,N) BY
!           GAUSS ELIMINATION WITH PARTIAL PIVOTING
!     LINPACK - LIKE 
!--------------------------------------------------------------
!
      INTEGER       :: N,Ipvt(N),info
      REAL(kind=dp) :: A(N,N)
      REAL(kind=dp) :: t, dmax, da
      INTEGER       :: j,k,l
      REAL(kind=dp), PARAMETER :: ZERO = 0.0, ONE = 1.0

      info = 0

size: IF (n > 1) THEN
      
col:  DO k = 1, n-1

!        find l = pivot index
!        l = idamax(n-k+1,A(k,k),1) + k - 1
         l = k; dmax = abs(A(k,k))
         DO j = k+1,n
            da = ABS(A(j,k))
            IF (da > dmax) THEN
              l = j; dmax = da
            END IF
         END DO
         Ipvt(k) = l

!        zero pivot implies this column already triangularized
         IF (ABS(A(l,k)) < TINY(ZERO)) THEN
            info = k
            return
         ELSE   
            IF (l /= k) THEN
               t = A(l,k); A(l,k) = A(k,k); A(k,k) = t
            END IF
            t = -ONE/A(k,k)
            CALL WSCAL(n-k,t,A(k+1,k),1)
            DO j = k+1, n
               t = A(l,j)
               IF (l /= k) THEN
                  A(l,j) = A(k,j); A(k,j) = t
               END IF
               CALL WAXPY(n-k,t,A(k+1,k),1,A(k+1,j),1)
            END DO         
         END IF
         
       END DO col
       
      END IF size
      
      Ipvt(N) = N
      IF (ABS(A(N,N)) == ZERO) info = N
      
      END SUBROUTINE WGEFA


!--------------------------------------------------------------
      SUBROUTINE WGESL(Trans,N,A,Ipvt,b)
!--------------------------------------------------------------
!     WGESL solves the system
!     a * x = b  or  trans(a) * x = b
!     using the factors computed by WGEFA.
!
!     Trans      = 'N'   to solve  A*x = b ,
!                = 'T'   to solve  transpose(A)*x = b
!     LINPACK - LIKE 
!--------------------------------------------------------------

      INTEGER       :: N,Ipvt(N)
      CHARACTER     :: trans
      REAL(kind=dp) :: A(N,N),b(N)
      REAL(kind=dp) :: t
      INTEGER       :: k,kb,l

      
      SELECT CASE (Trans)

      CASE ('n','N')  !  Solve  A * x = b

!        first solve  L*y = b
         IF (n >= 2) THEN
          DO k = 1, n-1
            l = Ipvt(k)
            t = b(l)
            IF (l /= k) THEN
               b(l) = b(k)
               b(k) = t
            END IF
            CALL WAXPY(n-k,t,a(k+1,k),1,b(k+1),1)
          END DO
         END IF
!        now solve  U*x = y
         DO kb = 1, n
            k = n + 1 - kb
            b(k) = b(k)/a(k,k)
            t = -b(k)
            CALL WAXPY(k-1,t,a(1,k),1,b(1),1)
         END DO
      
      CASE ('t','T')  !  Solve transpose(A) * x = b

!        first solve  trans(U)*y = b
         DO k = 1, n
            t = WDOT(k-1,a(1,k),1,b(1),1)
            b(k) = (b(k) - t)/a(k,k)
         END DO
!        now solve trans(L)*x = y
         IF (n >= 2) THEN
         DO kb = 1, n-1
            k = n - kb
            b(k) = b(k) + WDOT(n-k,a(k+1,k),1,b(k+1),1)
            l = Ipvt(k)
            IF (l /= k) THEN
               t = b(l); b(l) = b(k); b(k) = t
            END IF
         END DO
         END IF
   
      END SELECT

      END SUBROUTINE WGESL
! End of BLAS_UTIL function
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~



END MODULE gckpp_LinearAlgebra

