! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
! 
! Linear Algebra Data and Routines File
! 
! Generated by KPP-2.2.1_rs5 symbolic chemistry Kinetics PreProcessor
!       (http://www.cs.vt.edu/~asandu/Software/KPP)
! KPP is distributed under GPL, the general public licence
!       (http://www.gnu.org/copyleft/gpl.html)
! (C) 1995-1997, V. Damian & A. Sandu, CGRER, Univ. Iowa
! (C) 1997-2005, A. Sandu, Michigan Tech, Virginia Tech
!     With important contributions from:
!        M. Damian, Villanova University, USA
!        R. Sander, Max-Planck Institute for Chemistry, Mainz, Germany
! 
! File                 : gckpp_LinearAlgebra.f90
! Time                 : Mon Oct 19 12:59:47 2015
! Working directory    : /home/mpayer/KPP
! Equation file        : gckpp.kpp
! Output root filename : gckpp
! 
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~



MODULE gckpp_LinearAlgebra

  USE gckpp_Parameters
  USE gckpp_JacobianSP

  IMPLICIT NONE

CONTAINS


! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
! 
! SPARSE_UTIL - SPARSE utility functions
!   Arguments :
! 
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~


! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
SUBROUTINE KppDecomp( JVS, IER )
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!        Sparse LU factorization
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

  USE gckpp_Parameters
  USE gckpp_JacobianSP

      INTEGER  :: IER
      REAL(kind=dp) :: JVS(LU_NONZERO), W(NVAR), a
      INTEGER  :: k, kk, j, jj

      a = 0. ! mz_rs_20050606
      IER = 0
      DO k=1,NVAR
        ! mz_rs_20050606: don't check if real value == 0
        ! IF ( JVS( LU_DIAG(k) ) .EQ. 0. ) THEN
        IF ( ABS(JVS(LU_DIAG(k))) < TINY(a) ) THEN
            IER = k
            RETURN
        END IF
        DO kk = LU_CROW(k), LU_CROW(k+1)-1
              W( LU_ICOL(kk) ) = JVS(kk)
        END DO
        DO kk = LU_CROW(k), LU_DIAG(k)-1
            j = LU_ICOL(kk)
            a = -W(j) / JVS( LU_DIAG(j) )
            W(j) = -a
            DO jj = LU_DIAG(j)+1, LU_CROW(j+1)-1
               W( LU_ICOL(jj) ) = W( LU_ICOL(jj) ) + a*JVS(jj)
            END DO
         END DO
         DO kk = LU_CROW(k), LU_CROW(k+1)-1
            JVS(kk) = W( LU_ICOL(kk) )
         END DO
      END DO
      
END SUBROUTINE KppDecomp


! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
SUBROUTINE KppDecompCmplx( JVS, IER )
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!        Sparse LU factorization, complex
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

  USE gckpp_Parameters
  USE gckpp_JacobianSP

      INTEGER        :: IER
      DOUBLE COMPLEX :: JVS(LU_NONZERO), W(NVAR), a
      REAL(kind=dp)  :: b = 0.0
      INTEGER        :: k, kk, j, jj

      IER = 0
      DO k=1,NVAR
        IF ( ABS(JVS(LU_DIAG(k))) < TINY(b) ) THEN
            IER = k
            RETURN
        END IF
        DO kk = LU_CROW(k), LU_CROW(k+1)-1
              W( LU_ICOL(kk) ) = JVS(kk)
        END DO
        DO kk = LU_CROW(k), LU_DIAG(k)-1
            j = LU_ICOL(kk)
            a = -W(j) / JVS( LU_DIAG(j) )
            W(j) = -a
            DO jj = LU_DIAG(j)+1, LU_CROW(j+1)-1
               W( LU_ICOL(jj) ) = W( LU_ICOL(jj) ) + a*JVS(jj)
            END DO
         END DO
         DO kk = LU_CROW(k), LU_CROW(k+1)-1
            JVS(kk) = W( LU_ICOL(kk) )
         END DO
      END DO
      
END SUBROUTINE KppDecompCmplx


! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
SUBROUTINE KppDecompCmplxR( JVSR, JVSI, IER )
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!    Sparse LU factorization, complex
!   (Real and Imaginary parts are used instead of complex data type)     
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

  USE gckpp_Parameters
  USE gckpp_JacobianSP

      INTEGER       :: IER
      REAL(kind=dp) :: JVSR(LU_NONZERO), JVSI(LU_NONZERO) 
      REAL(kind=dp) :: WR(NVAR), WI(NVAR), ar, ai, den
      INTEGER       :: k, kk, j, jj

      IER = 0
      ar  = 0.0
      DO k=1,NVAR
        IF (  ( ABS(JVSR(LU_DIAG(k))) < TINY(ar) ) .AND. &
              ( ABS(JVSI(LU_DIAG(k))) < TINY(ar) ) )  THEN
            IER = k
            RETURN
        END IF
        DO kk = LU_CROW(k), LU_CROW(k+1)-1
              WR( LU_ICOL(kk) ) = JVSR(kk)
              WI( LU_ICOL(kk) ) = JVSI(kk)
        END DO
        DO kk = LU_CROW(k), LU_DIAG(k)-1
            j = LU_ICOL(kk)
            den = JVSR(LU_DIAG(j))**2 + JVSI(LU_DIAG(j))**2
            ar = -(WR(j)*JVSR(LU_DIAG(j)) + WI(j)*JVSI(LU_DIAG(j)))/den
            ai = -(WI(j)*JVSR(LU_DIAG(j)) - WR(j)*JVSI(LU_DIAG(j)))/den
            WR(j) = -ar
            WI(j) = -ai
            DO jj = LU_DIAG(j)+1, LU_CROW(j+1)-1
               WR( LU_ICOL(jj) ) = WR( LU_ICOL(jj) ) + ar*JVSR(jj) - ai*JVSI(jj)
               WI( LU_ICOL(jj) ) = WI( LU_ICOL(jj) ) + ar*JVSI(jj) + ai*JVSR(jj)
            END DO
         END DO
         DO kk = LU_CROW(k), LU_CROW(k+1)-1
            JVSR(kk) = WR( LU_ICOL(kk) )
            JVSI(kk) = WI( LU_ICOL(kk) )
         END DO
      END DO

END SUBROUTINE KppDecompCmplxR


! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
SUBROUTINE KppSolveIndirect( JVS, X )
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!        Sparse solve subroutine using indirect addressing
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

  USE gckpp_Parameters
  USE gckpp_JacobianSP

      INTEGER  :: i, j
      REAL(kind=dp) :: JVS(LU_NONZERO), X(NVAR), sum

      DO i=1,NVAR
         DO j = LU_CROW(i), LU_DIAG(i)-1 
             X(i) = X(i) - JVS(j)*X(LU_ICOL(j));
         END DO  
      END DO

      DO i=NVAR,1,-1
        sum = X(i);
        DO j = LU_DIAG(i)+1, LU_CROW(i+1)-1
          sum = sum - JVS(j)*X(LU_ICOL(j));
        END DO
        X(i) = sum/JVS(LU_DIAG(i));
      END DO
      
END SUBROUTINE KppSolveIndirect


! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
SUBROUTINE KppSolveTRIndirect( JVS, X )
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!        Complex sparse solve transpose subroutine using indirect addressing
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

  USE gckpp_Parameters
  USE gckpp_JacobianSP

      INTEGER       :: i, j
      REAL(kind=dp) :: JVS(LU_NONZERO), X(NVAR)

      DO i=1,NVAR
        X(i) = X(i)/JVS(LU_DIAG(i))
        ! subtract all nonzero elements in row i of JVS from X
        DO j=LU_DIAG(i)+1,LU_CROW(i+1)-1
          X(LU_ICOL(j)) = X(LU_ICOL(j))-JVS(j)*X(i)
        END DO
      END DO

      DO i=NVAR, 1, -1
        ! subtract all nonzero elements in row i of JVS from X
        DO j=LU_CROW(i),LU_DIAG(i)-1
          X(LU_ICOL(j)) = X(LU_ICOL(j))-JVS(j)*X(i)
        END DO
      END DO
      
END SUBROUTINE KppSolveTRIndirect


! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
SUBROUTINE KppSolveCmplx( JVS, X )
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!        Complex sparse solve subroutine using indirect addressing
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

  USE gckpp_Parameters
  USE gckpp_JacobianSP

      INTEGER        :: i, j
      DOUBLE COMPLEX :: JVS(LU_NONZERO), X(NVAR), sum

      DO i=1,NVAR
         DO j = LU_CROW(i), LU_DIAG(i)-1 
             X(i) = X(i) - JVS(j)*X(LU_ICOL(j));
         END DO  
      END DO

      DO i=NVAR,1,-1
        sum = X(i);
        DO j = LU_DIAG(i)+1, LU_CROW(i+1)-1
          sum = sum - JVS(j)*X(LU_ICOL(j));
        END DO
        X(i) = sum/JVS(LU_DIAG(i));
      END DO
      
END SUBROUTINE KppSolveCmplx

! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
SUBROUTINE KppSolveCmplxR( JVSR, JVSI, XR, XI )
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!   Complex sparse solve subroutine using indirect addressing
!   (Real and Imaginary parts are used instead of complex data type)     
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

  USE gckpp_Parameters
  USE gckpp_JacobianSP

      INTEGER       ::  i, j
      REAL(kind=dp) ::  JVSR(LU_NONZERO), JVSI(LU_NONZERO), XR(NVAR), XI(NVAR), sumr, sumi, den

      DO i=1,NVAR
         DO j = LU_CROW(i), LU_DIAG(i)-1 
             XR(i) = XR(i) - (JVSR(j)*XR(LU_ICOL(j)) - JVSI(j)*XI(LU_ICOL(j)))
             XI(i) = XI(i) - (JVSR(j)*XI(LU_ICOL(j)) + JVSI(j)*XR(LU_ICOL(j)))
         END DO  
      END DO

      DO i=NVAR,1,-1
        sumr = XR(i); sumi = XI(i)
        DO j = LU_DIAG(i)+1, LU_CROW(i+1)-1
            sumr = sumr - (JVSR(j)*XR(LU_ICOL(j)) - JVSI(j)*XI(LU_ICOL(j)))
            sumi = sumi - (JVSR(j)*XI(LU_ICOL(j)) + JVSI(j)*XR(LU_ICOL(j)))
        END DO
        den   = JVSR(LU_DIAG(i))**2 + JVSI(LU_DIAG(i))**2
        XR(i) = (sumr*JVSR(LU_DIAG(i)) + sumi*JVSI(LU_DIAG(i)))/den
        XI(i) = (sumi*JVSR(LU_DIAG(i)) - sumr*JVSI(LU_DIAG(i)))/den
      END DO
      
END SUBROUTINE KppSolveCmplxR


! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
SUBROUTINE KppSolveTRCmplx( JVS, X )
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!        Complex sparse solve transpose subroutine using indirect addressing
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

  USE gckpp_Parameters
  USE gckpp_JacobianSP

      INTEGER        :: i, j
      DOUBLE COMPLEX :: JVS(LU_NONZERO), X(NVAR)

      DO i=1,NVAR
        X(i) = X(i)/JVS(LU_DIAG(i))
        ! subtract all nonzero elements in row i of JVS from X
        DO j=LU_DIAG(i)+1,LU_CROW(i+1)-1
          X(LU_ICOL(j)) = X(LU_ICOL(j))-JVS(j)*X(i)
        END DO
      END DO

      DO i=NVAR, 1, -1
        ! subtract all nonzero elements in row i of JVS from X
        DO j=LU_CROW(i),LU_DIAG(i)-1
          X(LU_ICOL(j)) = X(LU_ICOL(j))-JVS(j)*X(i)
        END DO
      END DO
      
END SUBROUTINE KppSolveTRCmplx


! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
SUBROUTINE KppSolveTRCmplxR( JVSR, JVSI, XR, XI )
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!   Complex sparse solve transpose subroutine using indirect addressing
!   (Real and Imaginary parts are used instead of complex data type)     
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

  USE gckpp_Parameters
  USE gckpp_JacobianSP

      INTEGER       ::  i, j
      REAL(kind=dp) ::  JVSR(LU_NONZERO), JVSI(LU_NONZERO), XR(NVAR), XI(NVAR), den

      DO i=1,NVAR
        den   = JVSR(LU_DIAG(i))**2 + JVSI(LU_DIAG(i))**2
        XR(i) = (XR(i)*JVSR(LU_DIAG(i)) + XI(i)*JVSI(LU_DIAG(i)))/den
        XI(i) = (XI(i)*JVSR(LU_DIAG(i)) - XR(i)*JVSI(LU_DIAG(i)))/den
        ! subtract all nonzero elements in row i of JVS from X
        DO j=LU_DIAG(i)+1,LU_CROW(i+1)-1
          XR(LU_ICOL(j)) = XR(LU_ICOL(j))-(JVSR(j)*XR(i) - JVSI(j)*XI(i))
          XI(LU_ICOL(j)) = XI(LU_ICOL(j))-(JVSI(j)*XR(i) + JVSR(j)*XI(i))
        END DO
      END DO

      DO i=NVAR, 1, -1
        ! subtract all nonzero elements in row i of JVS from X
        DO j=LU_CROW(i),LU_DIAG(i)-1
          XR(LU_ICOL(j)) = XR(LU_ICOL(j))-(JVSR(j)*XR(i) - JVSI(j)*XI(i))
          XI(LU_ICOL(j)) = XI(LU_ICOL(j))-(JVSI(j)*XR(i) + JVSR(j)*XI(i))
        END DO
      END DO
      
END SUBROUTINE KppSolveTRCmplxR


!
! Next few commented subroutines perform sparse big linear algebra
!
!! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!SUBROUTINE KppDecompBig( JVS, IP, IER )
!! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!!        Sparse LU factorization
!!        for the Runge Kutta (3n)x(3n) linear system
!! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!
!  USE gckpp_Parameters
!  USE gckpp_JacobianSP
!
!      INTEGER  :: IP3(3), IER, IP(3,NVAR)
!      REAL(kind=dp) :: JVS(3,3,LU_NONZERO), W(3,3,NVAR), a(3,3), E(3,3)
!      INTEGER  :: k, kk, j, jj
!
!      a = 0.0d0
!      IER = 0
!      DO k=1,NVAR
!        DO kk = LU_CROW(k), LU_CROW(k+1)-1
!              W( 1:3,1:3,LU_ICOL(kk) ) = JVS(1:3,1:3,kk)
!        END DO
!        DO kk = LU_CROW(k), LU_DIAG(k)-1
!            j = LU_ICOL(kk)
!            E(1:3,1:3) = JVS( 1:3,1:3,LU_DIAG(j) )
!            ! CALL DGETRF(3,3,E,3,IP3,IER) 
!            CALL FAC3(E,IP3,IER)
!            IF ( IER /= 0 )  RETURN
!            ! a = W(j) / JVS( LU_DIAG(j) )
!            a(1:3,1:3) = W( 1:3,1:3,j )
!            ! CALL DGETRS ('N',3,3,E,3,IP3,a,3,IER) 
!            CALL SOL3('N',E,IP3,a(1,1))
!            CALL SOL3('N',E,IP3,a(1,2))
!            CALL SOL3('N',E,IP3,a(1,3))
!            W(1:3,1:3,j) = a(1:3,1:3)
!            DO jj = LU_DIAG(j)+1, LU_CROW(j+1)-1
!               W( 1:3,1:3,LU_ICOL(jj) ) = W( 1:3,1:3,LU_ICOL(jj) ) &
!                        - MATMUL( a(1:3,1:3) , JVS(1:3,1:3,jj) )
!            END DO
!         END DO
!         DO kk = LU_CROW(k), LU_CROW(k+1)-1
!            JVS(1:3,1:3,kk) = W( 1:3,1:3,LU_ICOL(kk) )
!         END DO
!      END DO
!
!      DO k=1,NVAR
!         ! CALL WGEFA(JVS(1,1,LU_DIAG(k)),3,3,IP(1,k),IER)
!         ! CALL DGETRF(3,3,JVS(1,1,LU_DIAG(k)),3,IP(1,k),IER)
!         CALL FAC3(JVS(1,1,LU_DIAG(k)),IP(1,k),IER)
!         IF ( IER /= 0 )  RETURN
!      END DO 
!      
!END SUBROUTINE KppDecompBig
!
!
!! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!SUBROUTINE KppSolveBig( JVS, IP, X )
!! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!!        Sparse solve subroutine using indirect addressing
!!        for the Runge Kutta (3n)x(3n) linear system
!! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!
!  USE gckpp_Parameters
!  USE gckpp_JacobianSP
!
!      INTEGER  :: i, j, k, m, IP3(3), IP(3,NVAR), IER
!      REAL(kind=dp) :: JVS(3,3,LU_NONZERO), X(3,NVAR), sum(3)
!
!      DO i=1,NVAR
!        DO j = LU_CROW(i), LU_DIAG(i)-1 
!          !X(1:3,i) = X(1:3,i) - MATMUL(JVS(1:3,1:3,j),X(1:3,LU_ICOL(j)));
!          DO k=1,3
!            DO m=1,3
!              X(k,i) = X(k,i) - JVS(k,m,j)*X(m,LU_ICOL(j))
!            END DO
!          END DO
!        END DO  
!      END DO
!
!      DO i=NVAR,1,-1
!        sum(1:3) = X(1:3,i);
!        DO j = LU_DIAG(i)+1, LU_CROW(i+1)-1
!          !sum(1:3) = sum(1:3) - MATMUL(JVS(1:3,1:3,j),X(1:3,LU_ICOL(j)));
!          DO k=1,3
!            DO m=1,3
!              sum(k) = sum(k) - JVS(k,m,j)*X(m,LU_ICOL(j))
!            END DO
!          END DO
!        END DO
!        ! X(i) = sum/JVS(LU_DIAG(i));
!        ! CALL DGETRS ('N',3,1,JVS(1:3,1:3,LU_DIAG(i)),3,IP(1,i),sum,3,0) 
!        ! CALL WGESL('N',JVS(1,1,LU_DIAG(i)),3,3,IP(1,i),sum)
!        CALL SOL3('N',JVS(1,1,LU_DIAG(i)),IP(1,i),sum)
!        X(1:3,i) = sum(1:3)
!      END DO
!      
!END SUBROUTINE KppSolveBig
!
!
!! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!SUBROUTINE KppSolveBigTR( JVS, IP, X )
!! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!!        Big sparse transpose solve using indirect addressing
!! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!
!  USE gckpp_Parameters
!  USE gckpp_JacobianSP
!
!      INTEGER       :: i, j, k, m, IP(3,NVAR)
!      REAL(kind=dp) :: JVS(3,3,LU_NONZERO), X(3,NVAR)
!
!      DO i=1,NVAR
!        ! X(i) = X(i)/JVS(LU_DIAG(i))
!        CALL SOL3('T',JVS(1,1,LU_DIAG(i)),IP(1,i),X(1,i))
!        DO j=LU_DIAG(i)+1,LU_CROW(i+1)-1
!         !X(1:3,LU_ICOL(j)) = X(1:3,LU_ICOL(j)) &
!          !    - MATMUL( TRANSPOSE(JVS(1:3,1:3,j)), X(1:3,i) )
!          DO k=1,3
!            DO m=1,3
!              X(k,LU_ICOL(j)) = X(k,LU_ICOL(j)) - JVS(m,k,j)*X(m,i)
!            END DO
!          END DO
!       END DO
!      END DO
!
!      DO i=NVAR, 1, -1
!        DO j=LU_CROW(i),LU_DIAG(i)-1
!         !X(1:3,LU_ICOL(j)) = X(1:3,LU_ICOL(j)) &
!          !   - MATMUL( TRANSPOSE(JVS(1:3,1:3,j)), X(1:3,i) )
!          DO k=1,3
!            DO m=1,3
!              X(k,LU_ICOL(j)) = X(k,LU_ICOL(j)) - JVS(m,k,j)*X(m,i)
!            END DO
!          END DO
!       END DO
!      END DO
!      
!END SUBROUTINE KppSolveBigTR
!
!
!
!! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!SUBROUTINE FAC3(A,IPVT,INFO)
!! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!!     FAC3 FACTORS THE MATRIX A (3,3) BY
!!           GAUSS ELIMINATION WITH PARTIAL PIVOTING
!!     LINPACK - LIKE 
!!
!!     Remove comments to perform pivoting
!! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!!
!      REAL(kind=dp) :: A(3,3)
!      INTEGER       :: IPVT(3),INFO
!!      INTEGER       :: L
!!      REAL(kind=dp) :: t, dmax, da, TMP(3)
!      REAL(kind=dp), PARAMETER :: ZERO = 0.0, ONE = 1.0
!
!      info = 0
!!      t = TINY(da)
!!      
!!      da = ABS(A(1,1)); L = 1
!!      IF ( ABS(A(2,1))>da ) THEN
!!        da = ABS(A(2,1)); L = 2
!!        IF ( ABS(A(3,1))>da ) THEN
!!          L = 3
!!        END IF  
!!      END IF  
!!      IPVT(1)  = L
!!      IF (L /=1 ) THEN
!!         TMP(1:3) = A(L,1:3)
!!         A(L,1:3) = A(1,1:3)
!!         A(1,1:3) = TMP(1:3)
!!      END IF
!!      IF (ABS(A(1,1)) < t) THEN
!!         info = 1
!!         return
!!      END IF   
!!
!      A(2,1) = A(2,1)/A(1,1)
!      A(2,2) = A(2,2) - A(2,1)*A(1,2)
!      A(2,3) = A(2,3) - A(2,1)*A(1,3)
!      A(3,1) = A(3,1)/A(1,1)
!      A(3,2) = A(3,2) - A(3,1)*A(1,2)
!      A(3,3) = A(3,3) - A(3,1)*A(1,3)
!      
!!      IPVT(2)  = 2
!!      IF (ABS(A(3,2))>ABS(A(2,2))) THEN
!!         IPVT(2)  = 3
!!         TMP(2:3) = A(3,2:3)
!!         A(3,2:3) = A(2,2:3)
!!         A(2,2:3) = TMP(2:3)
!!      END IF
!!      IF (ABS(A(2,2)) < t) THEN
!!         info = 1
!!         return
!!      END IF   
!!      
!      A(3,2)   = A(3,2)/A(2,2)
!      A(3,3)   = A(3,3) - A(3,2)*A(2,3)
!      IPVT(3)  = 3
!      
!END SUBROUTINE FAC3
!
!
!! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!SUBROUTINE SOL3(Trans,A,IPVT,b)
!! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!!     SOL3 solves the system 3x3
!!     A * x = b  or  trans(a) * x = b
!!     using the factors computed by WGEFA.
!!
!!     Trans      = 'N'   to solve  A*x = b ,
!!                = 'T'   to solve  transpose(A)*x = b
!!     LINPACK - LIKE 
!!
!!     Remove comments to use pivoting
!! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!
!      CHARACTER     :: Trans
!      REAL(kind=dp) :: a(3,3),b(3)
!      INTEGER       :: IPVT(3)
!!      INTEGER       :: L
!!      REAL(kind=dp) :: TMP
!      
!      SELECT CASE (Trans)
!
!      CASE ('n','N')  !  Solve  A * x = b
!
!!     Solve  L*y = b
!!         L = IPVT(1)
!!         IF (L /= 1) THEN
!!            TMP = B(1); B(1) = B(L); B(L) = TMP
!!         END IF
!         b(2) = b(2)-A(2,1)*b(1)
!         b(3) = b(3)-A(3,1)*b(1)
!         
!!         L = IPVT(2)
!!         IF (L /= 2) THEN
!!            TMP = B(2); B(2) = B(L); B(L) = TMP
!!         END IF
!         b(3) = b(3)-A(3,2)*b(2)
!
!!     Solve  U*x = y
!         b(3) = b(3)/A(3,3)
!         b(2) = (b(2)-A(2,3)*b(3))/A(2,2)
!         b(1) = (b(1)-A(1,3)*b(3)-A(1,2)*b(2))/A(1,1)
!      
!      
!      CASE ('t','T')  !  Solve transpose(A) * x = b
!
!!      Solve transpose(U)*y = b
!         b(1) = b(1)/A(1,1)
!         b(2) = (b(2)-A(1,2)*b(1))/A(2,2)
!         b(3) = (b(3)-A(1,3)*b(1)-A(2,3)*b(2))/A(3,3)
!
!!      Solve transpose(L)*x = y
!         b(2) = b(2)-A(3,2)*b(3)
!!         L = ipvt(2)
!!         IF (L /= 2) THEN
!!            TMP = B(2); B(2) = B(L); B(L) = TMP
!!         END IF
!         b(1) = b(1)-A(3,1)*b(3)-A(2,1)*b(2)
!!         L = ipvt(1)
!!         IF (L /= 1) THEN
!!            TMP = B(1); B(1) = B(L); B(L) = TMP
!!         END IF
!   
!      END SELECT
!
!END SUBROUTINE SOL3

! End of SPARSE_UTIL function
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~


! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
! 
! KppSolve - sparse back substitution
!   Arguments :
!      JVS       - sparse Jacobian of variables
!      X         - Vector for variables
! 
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

SUBROUTINE KppSolve ( JVS, X )

! JVS - sparse Jacobian of variables
  REAL(kind=dp) :: JVS(LU_NONZERO)
! X - Vector for variables
  REAL(kind=dp) :: X(NVAR)

  X(60) = X(60)-JVS(176)*X(46)
  X(61) = X(61)-JVS(181)*X(44)
  X(62) = X(62)-JVS(186)*X(45)
  X(88) = X(88)-JVS(273)*X(73)
  X(106) = X(106)-JVS(370)*X(105)
  X(112) = X(112)-JVS(405)*X(50)-JVS(406)*X(101)
  X(113) = X(113)-JVS(412)*X(51)
  X(115) = X(115)-JVS(434)*X(107)
  X(120) = X(120)-JVS(467)*X(111)
  X(124) = X(124)-JVS(500)*X(88)
  X(130) = X(130)-JVS(545)*X(128)
  X(133) = X(133)-JVS(564)*X(98)-JVS(565)*X(121)
  X(134) = X(134)-JVS(576)*X(75)-JVS(577)*X(83)
  X(135) = X(135)-JVS(587)*X(97)-JVS(588)*X(129)
  X(136) = X(136)-JVS(601)*X(112)-JVS(602)*X(116)-JVS(603)*X(125)-JVS(604)*X(131)
  X(137) = X(137)-JVS(617)*X(91)
  X(138) = X(138)-JVS(625)*X(130)
  X(139) = X(139)-JVS(634)*X(111)
  X(140) = X(140)-JVS(644)*X(77)
  X(141) = X(141)-JVS(655)*X(96)-JVS(656)*X(130)
  X(142) = X(142)-JVS(666)*X(128)
  X(143) = X(143)-JVS(675)*X(118)-JVS(676)*X(119)-JVS(677)*X(123)-JVS(678)*X(126)-JVS(679)*X(127)-JVS(680)*X(128)&
             &-JVS(681)*X(130)-JVS(682)*X(132)-JVS(683)*X(142)
  X(146) = X(146)-JVS(712)*X(97)-JVS(713)*X(130)
  X(147) = X(147)-JVS(723)*X(102)-JVS(724)*X(128)
  X(148) = X(148)-JVS(734)*X(84)-JVS(735)*X(104)
  X(150) = X(150)-JVS(757)*X(48)-JVS(758)*X(93)-JVS(759)*X(144)
  X(151) = X(151)-JVS(772)*X(102)-JVS(773)*X(105)-JVS(774)*X(112)-JVS(775)*X(116)-JVS(776)*X(120)-JVS(777)*X(123)&
             &-JVS(778)*X(129)-JVS(779)*X(147)
  X(152) = X(152)-JVS(798)*X(105)
  X(153) = X(153)-JVS(806)*X(77)-JVS(807)*X(83)
  X(155) = X(155)-JVS(826)*X(110)-JVS(827)*X(145)
  X(156) = X(156)-JVS(839)*X(73)-JVS(840)*X(76)-JVS(841)*X(78)-JVS(842)*X(108)-JVS(843)*X(124)-JVS(844)*X(154)
  X(157) = X(157)-JVS(859)*X(100)-JVS(860)*X(140)-JVS(861)*X(149)-JVS(862)*X(153)
  X(159) = X(159)-JVS(882)*X(68)-JVS(883)*X(74)-JVS(884)*X(104)-JVS(885)*X(108)-JVS(886)*X(115)-JVS(887)*X(116)-JVS(888)&
             &*X(121)-JVS(889)*X(136)-JVS(890)*X(137)-JVS(891)*X(151)-JVS(892)*X(152)-JVS(893)*X(154)-JVS(894)*X(158)
  X(160) = X(160)-JVS(927)*X(91)-JVS(928)*X(117)-JVS(929)*X(144)-JVS(930)*X(158)
  X(161) = X(161)-JVS(942)*X(103)-JVS(943)*X(142)
  X(162) = X(162)-JVS(956)*X(96)-JVS(957)*X(103)-JVS(958)*X(105)-JVS(959)*X(106)-JVS(960)*X(112)-JVS(961)*X(113)&
             &-JVS(962)*X(116)-JVS(963)*X(117)-JVS(964)*X(123)-JVS(965)*X(129)-JVS(966)*X(137)-JVS(967)*X(139)-JVS(968)&
             &*X(141)-JVS(969)*X(142)-JVS(970)*X(144)-JVS(971)*X(152)-JVS(972)*X(157)-JVS(973)*X(158)-JVS(974)*X(160)&
             &-JVS(975)*X(161)
  X(163) = X(163)-JVS(998)*X(109)-JVS(999)*X(125)-JVS(1000)*X(128)-JVS(1001)*X(131)-JVS(1002)*X(158)
  X(164) = X(164)-JVS(1011)*X(101)-JVS(1012)*X(128)
  X(165) = X(165)-JVS(1021)*X(101)-JVS(1022)*X(111)-JVS(1023)*X(129)-JVS(1024)*X(163)-JVS(1025)*X(164)
  X(166) = X(166)-JVS(1034)*X(100)-JVS(1035)*X(102)-JVS(1036)*X(105)-JVS(1037)*X(112)-JVS(1038)*X(113)-JVS(1039)*X(116)&
             &-JVS(1040)*X(120)-JVS(1041)*X(126)-JVS(1042)*X(132)-JVS(1043)*X(135)-JVS(1044)*X(139)-JVS(1045)*X(145)&
             &-JVS(1046)*X(146)-JVS(1047)*X(147)-JVS(1048)*X(149)-JVS(1049)*X(152)-JVS(1050)*X(157)-JVS(1051)*X(161)&
             &-JVS(1052)*X(162)-JVS(1053)*X(163)-JVS(1054)*X(164)-JVS(1055)*X(165)
  X(167) = X(167)-JVS(1075)*X(110)-JVS(1076)*X(111)-JVS(1077)*X(145)-JVS(1078)*X(164)
  X(168) = X(168)-JVS(1089)*X(110)-JVS(1090)*X(129)-JVS(1091)*X(145)-JVS(1092)*X(163)-JVS(1093)*X(164)-JVS(1094)*X(167)
  X(169) = X(169)-JVS(1104)*X(68)-JVS(1105)*X(99)-JVS(1106)*X(115)-JVS(1107)*X(116)-JVS(1108)*X(152)-JVS(1109)*X(155)&
             &-JVS(1110)*X(164)-JVS(1111)*X(165)-JVS(1112)*X(167)-JVS(1113)*X(168)
  X(170) = X(170)-JVS(1124)*X(109)-JVS(1125)*X(125)-JVS(1126)*X(131)-JVS(1127)*X(144)-JVS(1128)*X(145)-JVS(1129)*X(149)&
             &-JVS(1130)*X(155)-JVS(1131)*X(157)-JVS(1132)*X(158)-JVS(1133)*X(163)-JVS(1134)*X(167)-JVS(1135)*X(168)&
             &-JVS(1136)*X(169)
  X(171) = X(171)-JVS(1149)*X(75)-JVS(1150)*X(97)-JVS(1151)*X(99)-JVS(1152)*X(109)-JVS(1153)*X(110)-JVS(1154)*X(114)&
             &-JVS(1155)*X(115)-JVS(1156)*X(116)-JVS(1157)*X(125)-JVS(1158)*X(131)-JVS(1159)*X(134)-JVS(1160)*X(137)&
             &-JVS(1161)*X(141)-JVS(1162)*X(145)-JVS(1163)*X(146)-JVS(1164)*X(147)-JVS(1165)*X(149)-JVS(1166)*X(152)&
             &-JVS(1167)*X(153)-JVS(1168)*X(155)-JVS(1169)*X(158)-JVS(1170)*X(160)-JVS(1171)*X(161)-JVS(1172)*X(163)&
             &-JVS(1173)*X(164)-JVS(1174)*X(165)-JVS(1175)*X(167)-JVS(1176)*X(168)-JVS(1177)*X(169)-JVS(1178)*X(170)
  X(172) = X(172)-JVS(1194)*X(58)-JVS(1195)*X(59)-JVS(1196)*X(64)-JVS(1197)*X(70)-JVS(1198)*X(71)-JVS(1199)*X(86)&
             &-JVS(1200)*X(121)-JVS(1201)*X(133)
  X(173) = X(173)-JVS(1216)*X(101)-JVS(1217)*X(118)-JVS(1218)*X(128)-JVS(1219)*X(163)-JVS(1220)*X(164)
  X(174) = X(174)-JVS(1232)*X(102)-JVS(1233)*X(106)-JVS(1234)*X(107)-JVS(1235)*X(109)-JVS(1236)*X(125)-JVS(1237)*X(131)&
             &-JVS(1238)*X(137)-JVS(1239)*X(147)-JVS(1240)*X(149)-JVS(1241)*X(152)-JVS(1242)*X(157)-JVS(1243)*X(158)&
             &-JVS(1244)*X(160)-JVS(1245)*X(161)-JVS(1246)*X(163)-JVS(1247)*X(164)-JVS(1248)*X(165)-JVS(1249)*X(168)&
             &-JVS(1250)*X(169)-JVS(1251)*X(170)-JVS(1252)*X(173)
  X(175) = X(175)-JVS(1266)*X(101)-JVS(1267)*X(128)-JVS(1268)*X(142)-JVS(1269)*X(163)-JVS(1270)*X(164)
  X(176) = X(176)-JVS(1282)*X(84)-JVS(1283)*X(93)-JVS(1284)*X(96)-JVS(1285)*X(119)-JVS(1286)*X(130)-JVS(1287)*X(141)&
             &-JVS(1288)*X(144)-JVS(1289)*X(146)-JVS(1290)*X(148)-JVS(1291)*X(149)-JVS(1292)*X(150)-JVS(1293)*X(155)&
             &-JVS(1294)*X(158)-JVS(1295)*X(163)-JVS(1296)*X(167)-JVS(1297)*X(168)-JVS(1298)*X(169)-JVS(1299)*X(170)&
             &-JVS(1300)*X(171)-JVS(1301)*X(173)-JVS(1302)*X(174)-JVS(1303)*X(175)
  X(177) = X(177)-JVS(1316)*X(67)-JVS(1317)*X(74)-JVS(1318)*X(94)-JVS(1319)*X(118)-JVS(1320)*X(119)-JVS(1321)*X(123)&
             &-JVS(1322)*X(126)-JVS(1323)*X(127)-JVS(1324)*X(128)-JVS(1325)*X(130)-JVS(1326)*X(132)-JVS(1327)*X(142)&
             &-JVS(1328)*X(165)-JVS(1329)*X(173)-JVS(1330)*X(175)
  X(178) = X(178)-JVS(1342)*X(65)-JVS(1343)*X(69)-JVS(1344)*X(91)-JVS(1345)*X(100)-JVS(1346)*X(102)-JVS(1347)*X(117)&
             &-JVS(1348)*X(120)-JVS(1349)*X(128)-JVS(1350)*X(132)-JVS(1351)*X(134)-JVS(1352)*X(137)-JVS(1353)*X(140)&
             &-JVS(1354)*X(141)-JVS(1355)*X(144)-JVS(1356)*X(145)-JVS(1357)*X(146)-JVS(1358)*X(147)-JVS(1359)*X(148)&
             &-JVS(1360)*X(149)-JVS(1361)*X(150)-JVS(1362)*X(152)-JVS(1363)*X(153)-JVS(1364)*X(155)-JVS(1365)*X(157)&
             &-JVS(1366)*X(158)-JVS(1367)*X(160)-JVS(1368)*X(161)-JVS(1369)*X(162)-JVS(1370)*X(163)-JVS(1371)*X(164)&
             &-JVS(1372)*X(165)-JVS(1373)*X(166)-JVS(1374)*X(167)-JVS(1375)*X(168)-JVS(1376)*X(169)-JVS(1377)*X(170)&
             &-JVS(1378)*X(171)-JVS(1379)*X(173)-JVS(1380)*X(174)-JVS(1381)*X(175)-JVS(1382)*X(176)-JVS(1383)*X(177)
  X(179) = X(179)-JVS(1395)*X(1)-JVS(1396)*X(72)-JVS(1397)*X(74)-JVS(1398)*X(89)-JVS(1399)*X(91)-JVS(1400)*X(95)&
             &-JVS(1401)*X(96)-JVS(1402)*X(100)-JVS(1403)*X(101)-JVS(1404)*X(102)-JVS(1405)*X(103)-JVS(1406)*X(111)&
             &-JVS(1407)*X(112)-JVS(1408)*X(113)-JVS(1409)*X(117)-JVS(1410)*X(120)-JVS(1411)*X(125)-JVS(1412)*X(127)&
             &-JVS(1413)*X(128)-JVS(1414)*X(129)-JVS(1415)*X(130)-JVS(1416)*X(131)-JVS(1417)*X(134)-JVS(1418)*X(135)&
             &-JVS(1419)*X(136)-JVS(1420)*X(137)-JVS(1421)*X(138)-JVS(1422)*X(139)-JVS(1423)*X(141)-JVS(1424)*X(142)&
             &-JVS(1425)*X(144)-JVS(1426)*X(145)-JVS(1427)*X(146)-JVS(1428)*X(147)-JVS(1429)*X(148)-JVS(1430)*X(149)&
             &-JVS(1431)*X(150)-JVS(1432)*X(151)-JVS(1433)*X(152)-JVS(1434)*X(153)-JVS(1435)*X(155)-JVS(1436)*X(157)&
             &-JVS(1437)*X(158)-JVS(1438)*X(160)-JVS(1439)*X(161)-JVS(1440)*X(162)-JVS(1441)*X(163)-JVS(1442)*X(164)&
             &-JVS(1443)*X(165)-JVS(1444)*X(167)-JVS(1445)*X(168)-JVS(1446)*X(169)-JVS(1447)*X(170)-JVS(1448)*X(171)&
             &-JVS(1449)*X(173)-JVS(1450)*X(174)-JVS(1451)*X(175)-JVS(1452)*X(176)-JVS(1453)*X(177)-JVS(1454)*X(178)
  X(180) = X(180)-JVS(1469)*X(67)-JVS(1470)*X(83)-JVS(1471)*X(87)-JVS(1472)*X(103)-JVS(1473)*X(104)-JVS(1474)*X(105)&
             &-JVS(1475)*X(106)-JVS(1476)*X(112)-JVS(1477)*X(117)-JVS(1478)*X(118)-JVS(1479)*X(119)-JVS(1480)*X(123)&
             &-JVS(1481)*X(126)-JVS(1482)*X(128)-JVS(1483)*X(130)-JVS(1484)*X(132)-JVS(1485)*X(135)-JVS(1486)*X(136)&
             &-JVS(1487)*X(139)-JVS(1488)*X(140)-JVS(1489)*X(142)-JVS(1490)*X(144)-JVS(1491)*X(145)-JVS(1492)*X(146)&
             &-JVS(1493)*X(151)-JVS(1494)*X(152)-JVS(1495)*X(153)-JVS(1496)*X(158)-JVS(1497)*X(160)-JVS(1498)*X(161)&
             &-JVS(1499)*X(162)-JVS(1500)*X(163)-JVS(1501)*X(164)-JVS(1502)*X(165)-JVS(1503)*X(166)-JVS(1504)*X(167)&
             &-JVS(1505)*X(168)-JVS(1506)*X(169)-JVS(1507)*X(170)-JVS(1508)*X(171)-JVS(1509)*X(173)-JVS(1510)*X(174)&
             &-JVS(1511)*X(175)-JVS(1512)*X(176)-JVS(1513)*X(177)-JVS(1514)*X(178)-JVS(1515)*X(179)
  X(181) = X(181)-JVS(1529)*X(65)-JVS(1530)*X(69)-JVS(1531)*X(72)-JVS(1532)*X(74)-JVS(1533)*X(86)-JVS(1534)*X(87)&
             &-JVS(1535)*X(89)-JVS(1536)*X(125)-JVS(1537)*X(128)-JVS(1538)*X(130)-JVS(1539)*X(131)-JVS(1540)*X(132)&
             &-JVS(1541)*X(134)-JVS(1542)*X(135)-JVS(1543)*X(137)-JVS(1544)*X(138)-JVS(1545)*X(140)-JVS(1546)*X(141)&
             &-JVS(1547)*X(145)-JVS(1548)*X(146)-JVS(1549)*X(147)-JVS(1550)*X(148)-JVS(1551)*X(149)-JVS(1552)*X(150)&
             &-JVS(1553)*X(152)-JVS(1554)*X(153)-JVS(1555)*X(155)-JVS(1556)*X(157)-JVS(1557)*X(158)-JVS(1558)*X(160)&
             &-JVS(1559)*X(161)-JVS(1560)*X(162)-JVS(1561)*X(163)-JVS(1562)*X(164)-JVS(1563)*X(165)-JVS(1564)*X(167)&
             &-JVS(1565)*X(168)-JVS(1566)*X(169)-JVS(1567)*X(170)-JVS(1568)*X(171)-JVS(1569)*X(173)-JVS(1570)*X(174)&
             &-JVS(1571)*X(175)-JVS(1572)*X(176)-JVS(1573)*X(177)-JVS(1574)*X(178)-JVS(1575)*X(180)
  X(182) = X(182)-JVS(1588)*X(107)-JVS(1589)*X(111)-JVS(1590)*X(117)-JVS(1591)*X(122)-JVS(1592)*X(128)-JVS(1593)*X(129)&
             &-JVS(1594)*X(130)-JVS(1595)*X(138)-JVS(1596)*X(144)-JVS(1597)*X(150)-JVS(1598)*X(152)-JVS(1599)*X(158)&
             &-JVS(1600)*X(160)-JVS(1601)*X(163)-JVS(1602)*X(164)-JVS(1603)*X(165)-JVS(1604)*X(171)-JVS(1605)*X(173)&
             &-JVS(1606)*X(174)-JVS(1607)*X(175)-JVS(1608)*X(176)-JVS(1609)*X(177)-JVS(1610)*X(178)-JVS(1611)*X(179)&
             &-JVS(1612)*X(180)-JVS(1613)*X(181)
  X(183) = X(183)-JVS(1627)*X(60)-JVS(1628)*X(61)-JVS(1629)*X(62)-JVS(1630)*X(63)-JVS(1631)*X(66)-JVS(1632)*X(85)&
             &-JVS(1633)*X(112)-JVS(1634)*X(113)-JVS(1635)*X(115)-JVS(1636)*X(116)-JVS(1637)*X(118)-JVS(1638)*X(119)&
             &-JVS(1639)*X(123)-JVS(1640)*X(125)-JVS(1641)*X(126)-JVS(1642)*X(127)-JVS(1643)*X(128)-JVS(1644)*X(130)&
             &-JVS(1645)*X(131)-JVS(1646)*X(132)-JVS(1647)*X(134)-JVS(1648)*X(137)-JVS(1649)*X(139)-JVS(1650)*X(141)&
             &-JVS(1651)*X(142)-JVS(1652)*X(144)-JVS(1653)*X(145)-JVS(1654)*X(146)-JVS(1655)*X(147)-JVS(1656)*X(148)&
             &-JVS(1657)*X(149)-JVS(1658)*X(150)-JVS(1659)*X(152)-JVS(1660)*X(153)-JVS(1661)*X(155)-JVS(1662)*X(157)&
             &-JVS(1663)*X(158)-JVS(1664)*X(160)-JVS(1665)*X(161)-JVS(1666)*X(163)-JVS(1667)*X(164)-JVS(1668)*X(165)&
             &-JVS(1669)*X(167)-JVS(1670)*X(168)-JVS(1671)*X(169)-JVS(1672)*X(170)-JVS(1673)*X(171)-JVS(1674)*X(172)&
             &-JVS(1675)*X(173)-JVS(1676)*X(174)-JVS(1677)*X(175)-JVS(1678)*X(177)-JVS(1679)*X(178)-JVS(1680)*X(180)&
             &-JVS(1681)*X(181)-JVS(1682)*X(182)
  X(184) = X(184)-JVS(1696)*X(47)-JVS(1697)*X(52)-JVS(1698)*X(53)-JVS(1699)*X(54)-JVS(1700)*X(55)-JVS(1701)*X(56)&
             &-JVS(1702)*X(57)-JVS(1703)*X(64)-JVS(1704)*X(73)-JVS(1705)*X(78)-JVS(1706)*X(79)-JVS(1707)*X(80)-JVS(1708)&
             &*X(81)-JVS(1709)*X(82)-JVS(1710)*X(87)-JVS(1711)*X(88)-JVS(1712)*X(92)-JVS(1713)*X(108)-JVS(1714)*X(121)&
             &-JVS(1715)*X(124)-JVS(1716)*X(133)-JVS(1717)*X(138)-JVS(1718)*X(143)-JVS(1719)*X(154)-JVS(1720)*X(156)&
             &-JVS(1721)*X(165)-JVS(1722)*X(172)-JVS(1723)*X(173)-JVS(1724)*X(175)-JVS(1725)*X(176)-JVS(1726)*X(177)&
             &-JVS(1727)*X(178)-JVS(1728)*X(179)-JVS(1729)*X(180)-JVS(1730)*X(181)-JVS(1731)*X(182)-JVS(1732)*X(183)
  X(185) = X(185)-JVS(1745)*X(52)-JVS(1746)*X(53)-JVS(1747)*X(54)-JVS(1748)*X(55)-JVS(1749)*X(56)-JVS(1750)*X(57)&
             &-JVS(1751)*X(64)-JVS(1752)*X(73)-JVS(1753)*X(76)-JVS(1754)*X(79)-JVS(1755)*X(80)-JVS(1756)*X(81)-JVS(1757)&
             &*X(82)-JVS(1758)*X(88)-JVS(1759)*X(124)-JVS(1760)*X(154)-JVS(1761)*X(156)-JVS(1762)*X(172)-JVS(1763)*X(181)&
             &-JVS(1764)*X(182)-JVS(1765)*X(183)-JVS(1766)*X(184)
  X(186) = X(186)-JVS(1778)*X(65)-JVS(1779)*X(68)-JVS(1780)*X(72)-JVS(1781)*X(74)-JVS(1782)*X(90)-JVS(1783)*X(104)&
             &-JVS(1784)*X(108)-JVS(1785)*X(113)-JVS(1786)*X(117)-JVS(1787)*X(118)-JVS(1788)*X(119)-JVS(1789)*X(120)&
             &-JVS(1790)*X(121)-JVS(1791)*X(123)-JVS(1792)*X(126)-JVS(1793)*X(127)-JVS(1794)*X(128)-JVS(1795)*X(130)&
             &-JVS(1796)*X(131)-JVS(1797)*X(132)-JVS(1798)*X(136)-JVS(1799)*X(139)-JVS(1800)*X(142)-JVS(1801)*X(144)&
             &-JVS(1802)*X(147)-JVS(1803)*X(151)-JVS(1804)*X(152)-JVS(1805)*X(154)-JVS(1806)*X(158)-JVS(1807)*X(159)&
             &-JVS(1808)*X(160)-JVS(1809)*X(161)-JVS(1810)*X(163)-JVS(1811)*X(164)-JVS(1812)*X(165)-JVS(1813)*X(166)&
             &-JVS(1814)*X(167)-JVS(1815)*X(168)-JVS(1816)*X(169)-JVS(1817)*X(170)-JVS(1818)*X(171)-JVS(1819)*X(172)&
             &-JVS(1820)*X(173)-JVS(1821)*X(174)-JVS(1822)*X(175)-JVS(1823)*X(176)-JVS(1824)*X(177)-JVS(1825)*X(178)&
             &-JVS(1826)*X(179)-JVS(1827)*X(180)-JVS(1828)*X(181)-JVS(1829)*X(182)-JVS(1830)*X(183)-JVS(1831)*X(184)&
             &-JVS(1832)*X(185)
  X(187) = X(187)-JVS(1843)*X(83)-JVS(1844)*X(104)-JVS(1845)*X(140)-JVS(1846)*X(153)-JVS(1847)*X(154)-JVS(1848)*X(156)&
             &-JVS(1849)*X(169)-JVS(1850)*X(170)-JVS(1851)*X(172)-JVS(1852)*X(174)-JVS(1853)*X(175)-JVS(1854)*X(176)&
             &-JVS(1855)*X(177)-JVS(1856)*X(178)-JVS(1857)*X(179)-JVS(1858)*X(180)-JVS(1859)*X(181)-JVS(1860)*X(182)&
             &-JVS(1861)*X(183)-JVS(1862)*X(184)-JVS(1863)*X(185)-JVS(1864)*X(186)
  X(188) = X(188)-JVS(1874)*X(52)-JVS(1875)*X(53)-JVS(1876)*X(54)-JVS(1877)*X(55)-JVS(1878)*X(56)-JVS(1879)*X(57)&
             &-JVS(1880)*X(58)-JVS(1881)*X(59)-JVS(1882)*X(64)-JVS(1883)*X(66)-JVS(1884)*X(67)-JVS(1885)*X(70)-JVS(1886)&
             &*X(71)-JVS(1887)*X(76)-JVS(1888)*X(79)-JVS(1889)*X(80)-JVS(1890)*X(81)-JVS(1891)*X(82)-JVS(1892)*X(122)&
             &-JVS(1893)*X(124)-JVS(1894)*X(133)-JVS(1895)*X(138)-JVS(1896)*X(143)-JVS(1897)*X(154)-JVS(1898)*X(156)&
             &-JVS(1899)*X(165)-JVS(1900)*X(172)-JVS(1901)*X(173)-JVS(1902)*X(175)-JVS(1903)*X(176)-JVS(1904)*X(177)&
             &-JVS(1905)*X(178)-JVS(1906)*X(179)-JVS(1907)*X(180)-JVS(1908)*X(181)-JVS(1909)*X(182)-JVS(1910)*X(183)&
             &-JVS(1911)*X(184)-JVS(1912)*X(185)-JVS(1913)*X(186)-JVS(1914)*X(187)
  X(189) = X(189)-JVS(1923)*X(44)-JVS(1924)*X(45)-JVS(1925)*X(46)-JVS(1926)*X(47)-JVS(1927)*X(50)-JVS(1928)*X(67)&
             &-JVS(1929)*X(68)-JVS(1930)*X(69)-JVS(1931)*X(70)-JVS(1932)*X(71)-JVS(1933)*X(73)-JVS(1934)*X(74)-JVS(1935)&
             &*X(75)-JVS(1936)*X(76)-JVS(1937)*X(77)-JVS(1938)*X(78)-JVS(1939)*X(79)-JVS(1940)*X(80)-JVS(1941)*X(81)&
             &-JVS(1942)*X(82)-JVS(1943)*X(83)-JVS(1944)*X(84)-JVS(1945)*X(85)-JVS(1946)*X(86)-JVS(1947)*X(87)-JVS(1948)&
             &*X(89)-JVS(1949)*X(90)-JVS(1950)*X(91)-JVS(1951)*X(93)-JVS(1952)*X(94)-JVS(1953)*X(95)-JVS(1954)*X(96)&
             &-JVS(1955)*X(97)-JVS(1956)*X(98)-JVS(1957)*X(99)-JVS(1958)*X(100)-JVS(1959)*X(101)-JVS(1960)*X(102)-JVS(1961)&
             &*X(103)-JVS(1962)*X(104)-JVS(1963)*X(105)-JVS(1964)*X(106)-JVS(1965)*X(107)-JVS(1966)*X(108)-JVS(1967)*X(109)&
             &-JVS(1968)*X(110)-JVS(1969)*X(111)-JVS(1970)*X(112)-JVS(1971)*X(113)-JVS(1972)*X(114)-JVS(1973)*X(115)&
             &-JVS(1974)*X(116)-JVS(1975)*X(117)-JVS(1976)*X(118)-JVS(1977)*X(119)-JVS(1978)*X(120)-JVS(1979)*X(121)&
             &-JVS(1980)*X(122)-JVS(1981)*X(123)-JVS(1982)*X(124)-JVS(1983)*X(125)-JVS(1984)*X(126)-JVS(1985)*X(127)&
             &-JVS(1986)*X(128)-JVS(1987)*X(129)-JVS(1988)*X(130)-JVS(1989)*X(131)-JVS(1990)*X(132)-JVS(1991)*X(133)&
             &-JVS(1992)*X(134)-JVS(1993)*X(135)-JVS(1994)*X(136)-JVS(1995)*X(137)-JVS(1996)*X(138)-JVS(1997)*X(139)&
             &-JVS(1998)*X(140)-JVS(1999)*X(141)-JVS(2000)*X(142)-JVS(2001)*X(143)-JVS(2002)*X(144)-JVS(2003)*X(145)&
             &-JVS(2004)*X(146)-JVS(2005)*X(147)-JVS(2006)*X(148)-JVS(2007)*X(149)-JVS(2008)*X(150)-JVS(2009)*X(151)&
             &-JVS(2010)*X(152)-JVS(2011)*X(153)-JVS(2012)*X(154)-JVS(2013)*X(155)-JVS(2014)*X(156)-JVS(2015)*X(157)&
             &-JVS(2016)*X(158)-JVS(2017)*X(159)-JVS(2018)*X(160)-JVS(2019)*X(161)-JVS(2020)*X(162)-JVS(2021)*X(163)&
             &-JVS(2022)*X(164)-JVS(2023)*X(165)-JVS(2024)*X(166)-JVS(2025)*X(167)-JVS(2026)*X(168)-JVS(2027)*X(169)&
             &-JVS(2028)*X(170)-JVS(2029)*X(171)-JVS(2030)*X(172)-JVS(2031)*X(173)-JVS(2032)*X(174)-JVS(2033)*X(175)&
             &-JVS(2034)*X(176)-JVS(2035)*X(177)-JVS(2036)*X(178)-JVS(2037)*X(179)-JVS(2038)*X(180)-JVS(2039)*X(181)&
             &-JVS(2040)*X(182)-JVS(2041)*X(183)-JVS(2042)*X(184)-JVS(2043)*X(185)-JVS(2044)*X(186)-JVS(2045)*X(187)&
             &-JVS(2046)*X(188)
  X(190) = X(190)-JVS(2054)*X(49)-JVS(2055)*X(58)-JVS(2056)*X(59)-JVS(2057)*X(64)-JVS(2058)*X(70)-JVS(2059)*X(71)&
             &-JVS(2060)*X(83)-JVS(2061)*X(86)-JVS(2062)*X(92)-JVS(2063)*X(98)-JVS(2064)*X(104)-JVS(2065)*X(121)-JVS(2066)&
             &*X(133)-JVS(2067)*X(140)-JVS(2068)*X(153)-JVS(2069)*X(154)-JVS(2070)*X(156)-JVS(2071)*X(169)-JVS(2072)*X(170)&
             &-JVS(2073)*X(172)-JVS(2074)*X(174)-JVS(2075)*X(175)-JVS(2076)*X(176)-JVS(2077)*X(177)-JVS(2078)*X(178)&
             &-JVS(2079)*X(179)-JVS(2080)*X(180)-JVS(2081)*X(181)-JVS(2082)*X(182)-JVS(2083)*X(183)-JVS(2084)*X(184)&
             &-JVS(2085)*X(185)-JVS(2086)*X(186)-JVS(2087)*X(187)-JVS(2088)*X(188)-JVS(2089)*X(189)
  X(191) = X(191)-JVS(2096)*X(87)-JVS(2097)*X(108)-JVS(2098)*X(121)-JVS(2099)*X(133)-JVS(2100)*X(138)-JVS(2101)*X(143)&
             &-JVS(2102)*X(154)-JVS(2103)*X(156)-JVS(2104)*X(165)-JVS(2105)*X(172)-JVS(2106)*X(173)-JVS(2107)*X(175)&
             &-JVS(2108)*X(176)-JVS(2109)*X(177)-JVS(2110)*X(178)-JVS(2111)*X(179)-JVS(2112)*X(180)-JVS(2113)*X(181)&
             &-JVS(2114)*X(182)-JVS(2115)*X(183)-JVS(2116)*X(184)-JVS(2117)*X(185)-JVS(2118)*X(186)-JVS(2119)*X(187)&
             &-JVS(2120)*X(188)-JVS(2121)*X(189)-JVS(2122)*X(190)
  X(192) = X(192)-JVS(2128)*X(47)-JVS(2129)*X(79)-JVS(2130)*X(80)-JVS(2131)*X(81)-JVS(2132)*X(82)-JVS(2133)*X(85)&
             &-JVS(2134)*X(86)-JVS(2135)*X(87)-JVS(2136)*X(89)-JVS(2137)*X(90)-JVS(2138)*X(100)-JVS(2139)*X(104)-JVS(2140)&
             &*X(108)-JVS(2141)*X(121)-JVS(2142)*X(122)-JVS(2143)*X(133)-JVS(2144)*X(138)-JVS(2145)*X(140)-JVS(2146)*X(143)&
             &-JVS(2147)*X(149)-JVS(2148)*X(153)-JVS(2149)*X(154)-JVS(2150)*X(156)-JVS(2151)*X(157)-JVS(2152)*X(159)&
             &-JVS(2153)*X(160)-JVS(2154)*X(163)-JVS(2155)*X(164)-JVS(2156)*X(165)-JVS(2157)*X(166)-JVS(2158)*X(167)&
             &-JVS(2159)*X(168)-JVS(2160)*X(169)-JVS(2161)*X(170)-JVS(2162)*X(171)-JVS(2163)*X(172)-JVS(2164)*X(173)&
             &-JVS(2165)*X(174)-JVS(2166)*X(175)-JVS(2167)*X(176)-JVS(2168)*X(177)-JVS(2169)*X(178)-JVS(2170)*X(179)&
             &-JVS(2171)*X(180)-JVS(2172)*X(181)-JVS(2173)*X(182)-JVS(2174)*X(183)-JVS(2175)*X(184)-JVS(2176)*X(185)&
             &-JVS(2177)*X(186)-JVS(2178)*X(187)-JVS(2179)*X(188)-JVS(2180)*X(189)-JVS(2181)*X(190)-JVS(2182)*X(191)
  X(193) = X(193)-JVS(2187)*X(60)-JVS(2188)*X(61)-JVS(2189)*X(62)-JVS(2190)*X(72)-JVS(2191)*X(75)-JVS(2192)*X(77)&
             &-JVS(2193)*X(84)-JVS(2194)*X(86)-JVS(2195)*X(87)-JVS(2196)*X(89)-JVS(2197)*X(90)-JVS(2198)*X(93)-JVS(2199)&
             &*X(96)-JVS(2200)*X(97)-JVS(2201)*X(99)-JVS(2202)*X(101)-JVS(2203)*X(102)-JVS(2204)*X(103)-JVS(2205)*X(105)&
             &-JVS(2206)*X(106)-JVS(2207)*X(107)-JVS(2208)*X(109)-JVS(2209)*X(110)-JVS(2210)*X(111)-JVS(2211)*X(112)&
             &-JVS(2212)*X(113)-JVS(2213)*X(114)-JVS(2214)*X(115)-JVS(2215)*X(116)-JVS(2216)*X(122)-JVS(2217)*X(125)&
             &-JVS(2218)*X(128)-JVS(2219)*X(129)-JVS(2220)*X(130)-JVS(2221)*X(131)-JVS(2222)*X(134)-JVS(2223)*X(136)&
             &-JVS(2224)*X(137)-JVS(2225)*X(138)-JVS(2226)*X(139)-JVS(2227)*X(141)-JVS(2228)*X(142)-JVS(2229)*X(143)&
             &-JVS(2230)*X(144)-JVS(2231)*X(145)-JVS(2232)*X(146)-JVS(2233)*X(147)-JVS(2234)*X(148)-JVS(2235)*X(149)&
             &-JVS(2236)*X(150)-JVS(2237)*X(151)-JVS(2238)*X(152)-JVS(2239)*X(153)-JVS(2240)*X(155)-JVS(2241)*X(157)&
             &-JVS(2242)*X(158)-JVS(2243)*X(160)-JVS(2244)*X(161)-JVS(2245)*X(162)-JVS(2246)*X(163)-JVS(2247)*X(164)&
             &-JVS(2248)*X(165)-JVS(2249)*X(166)-JVS(2250)*X(167)-JVS(2251)*X(168)-JVS(2252)*X(169)-JVS(2253)*X(170)&
             &-JVS(2254)*X(171)-JVS(2255)*X(172)-JVS(2256)*X(173)-JVS(2257)*X(174)-JVS(2258)*X(175)-JVS(2259)*X(176)&
             &-JVS(2260)*X(177)-JVS(2261)*X(178)-JVS(2262)*X(179)-JVS(2263)*X(180)-JVS(2264)*X(181)-JVS(2265)*X(182)&
             &-JVS(2266)*X(183)-JVS(2267)*X(184)-JVS(2268)*X(185)-JVS(2269)*X(186)-JVS(2270)*X(187)-JVS(2271)*X(188)&
             &-JVS(2272)*X(189)-JVS(2273)*X(190)-JVS(2274)*X(191)-JVS(2275)*X(192)
  X(194) = X(194)-JVS(2279)*X(52)-JVS(2280)*X(53)-JVS(2281)*X(54)-JVS(2282)*X(55)-JVS(2283)*X(56)-JVS(2284)*X(57)&
             &-JVS(2285)*X(58)-JVS(2286)*X(59)-JVS(2287)*X(63)-JVS(2288)*X(64)-JVS(2289)*X(66)-JVS(2290)*X(70)-JVS(2291)&
             &*X(71)-JVS(2292)*X(79)-JVS(2293)*X(80)-JVS(2294)*X(81)-JVS(2295)*X(82)-JVS(2296)*X(86)-JVS(2297)*X(124)&
             &-JVS(2298)*X(138)-JVS(2299)*X(154)-JVS(2300)*X(156)-JVS(2301)*X(172)-JVS(2302)*X(173)-JVS(2303)*X(176)&
             &-JVS(2304)*X(177)-JVS(2305)*X(178)-JVS(2306)*X(180)-JVS(2307)*X(181)-JVS(2308)*X(182)-JVS(2309)*X(183)&
             &-JVS(2310)*X(184)-JVS(2311)*X(185)-JVS(2312)*X(186)-JVS(2313)*X(187)-JVS(2314)*X(188)-JVS(2315)*X(189)&
             &-JVS(2316)*X(190)-JVS(2317)*X(191)-JVS(2318)*X(192)-JVS(2319)*X(193)
  X(195) = X(195)-JVS(2322)*X(48)-JVS(2323)*X(49)-JVS(2324)*X(51)-JVS(2325)*X(65)-JVS(2326)*X(66)-JVS(2327)*X(72)&
             &-JVS(2328)*X(78)-JVS(2329)*X(85)-JVS(2330)*X(90)-JVS(2331)*X(95)-JVS(2332)*X(97)-JVS(2333)*X(108)-JVS(2334)&
             &*X(109)-JVS(2335)*X(110)-JVS(2336)*X(111)-JVS(2337)*X(112)-JVS(2338)*X(113)-JVS(2339)*X(115)-JVS(2340)*X(116)&
             &-JVS(2341)*X(117)-JVS(2342)*X(118)-JVS(2343)*X(119)-JVS(2344)*X(120)-JVS(2345)*X(121)-JVS(2346)*X(123)&
             &-JVS(2347)*X(125)-JVS(2348)*X(126)-JVS(2349)*X(127)-JVS(2350)*X(128)-JVS(2351)*X(129)-JVS(2352)*X(130)&
             &-JVS(2353)*X(131)-JVS(2354)*X(132)-JVS(2355)*X(134)-JVS(2356)*X(135)-JVS(2357)*X(137)-JVS(2358)*X(139)&
             &-JVS(2359)*X(141)-JVS(2360)*X(142)-JVS(2361)*X(144)-JVS(2362)*X(145)-JVS(2363)*X(146)-JVS(2364)*X(147)&
             &-JVS(2365)*X(148)-JVS(2366)*X(149)-JVS(2367)*X(150)-JVS(2368)*X(152)-JVS(2369)*X(153)-JVS(2370)*X(154)&
             &-JVS(2371)*X(155)-JVS(2372)*X(157)-JVS(2373)*X(158)-JVS(2374)*X(159)-JVS(2375)*X(160)-JVS(2376)*X(161)&
             &-JVS(2377)*X(163)-JVS(2378)*X(164)-JVS(2379)*X(165)-JVS(2380)*X(166)-JVS(2381)*X(167)-JVS(2382)*X(168)&
             &-JVS(2383)*X(169)-JVS(2384)*X(170)-JVS(2385)*X(171)-JVS(2386)*X(172)-JVS(2387)*X(173)-JVS(2388)*X(174)&
             &-JVS(2389)*X(175)-JVS(2390)*X(176)-JVS(2391)*X(177)-JVS(2392)*X(178)-JVS(2393)*X(179)-JVS(2394)*X(180)&
             &-JVS(2395)*X(181)-JVS(2396)*X(182)-JVS(2397)*X(183)-JVS(2398)*X(184)-JVS(2399)*X(185)-JVS(2400)*X(186)&
             &-JVS(2401)*X(187)-JVS(2402)*X(188)-JVS(2403)*X(189)-JVS(2404)*X(190)-JVS(2405)*X(191)-JVS(2406)*X(192)&
             &-JVS(2407)*X(193)-JVS(2408)*X(194)
  X(195) = X(195)/JVS(2409)
  X(194) = (X(194)-JVS(2321)*X(195))/(JVS(2320))
  X(193) = (X(193)-JVS(2277)*X(194)-JVS(2278)*X(195))/(JVS(2276))
  X(192) = (X(192)-JVS(2184)*X(193)-JVS(2185)*X(194)-JVS(2186)*X(195))/(JVS(2183))
  X(191) = (X(191)-JVS(2124)*X(192)-JVS(2125)*X(193)-JVS(2126)*X(194)-JVS(2127)*X(195))/(JVS(2123))
  X(190) = (X(190)-JVS(2091)*X(191)-JVS(2092)*X(192)-JVS(2093)*X(193)-JVS(2094)*X(194)-JVS(2095)*X(195))/(JVS(2090))
  X(189) = (X(189)-JVS(2048)*X(190)-JVS(2049)*X(191)-JVS(2050)*X(192)-JVS(2051)*X(193)-JVS(2052)*X(194)-JVS(2053)&
             &*X(195))/(JVS(2047))
  X(188) = (X(188)-JVS(1916)*X(189)-JVS(1917)*X(190)-JVS(1918)*X(191)-JVS(1919)*X(192)-JVS(1920)*X(193)-JVS(1921)*X(194)&
             &-JVS(1922)*X(195))/(JVS(1915))
  X(187) = (X(187)-JVS(1866)*X(188)-JVS(1867)*X(189)-JVS(1868)*X(190)-JVS(1869)*X(191)-JVS(1870)*X(192)-JVS(1871)*X(193)&
             &-JVS(1872)*X(194)-JVS(1873)*X(195))/(JVS(1865))
  X(186) = (X(186)-JVS(1834)*X(187)-JVS(1835)*X(188)-JVS(1836)*X(189)-JVS(1837)*X(190)-JVS(1838)*X(191)-JVS(1839)*X(192)&
             &-JVS(1840)*X(193)-JVS(1841)*X(194)-JVS(1842)*X(195))/(JVS(1833))
  X(185) = (X(185)-JVS(1768)*X(186)-JVS(1769)*X(187)-JVS(1770)*X(188)-JVS(1771)*X(189)-JVS(1772)*X(190)-JVS(1773)*X(191)&
             &-JVS(1774)*X(192)-JVS(1775)*X(193)-JVS(1776)*X(194)-JVS(1777)*X(195))/(JVS(1767))
  X(184) = (X(184)-JVS(1734)*X(185)-JVS(1735)*X(186)-JVS(1736)*X(187)-JVS(1737)*X(188)-JVS(1738)*X(189)-JVS(1739)*X(190)&
             &-JVS(1740)*X(191)-JVS(1741)*X(192)-JVS(1742)*X(193)-JVS(1743)*X(194)-JVS(1744)*X(195))/(JVS(1733))
  X(183) = (X(183)-JVS(1684)*X(184)-JVS(1685)*X(185)-JVS(1686)*X(186)-JVS(1687)*X(187)-JVS(1688)*X(188)-JVS(1689)*X(189)&
             &-JVS(1690)*X(190)-JVS(1691)*X(191)-JVS(1692)*X(192)-JVS(1693)*X(193)-JVS(1694)*X(194)-JVS(1695)*X(195))&
             &/(JVS(1683))
  X(182) = (X(182)-JVS(1615)*X(183)-JVS(1616)*X(184)-JVS(1617)*X(185)-JVS(1618)*X(186)-JVS(1619)*X(187)-JVS(1620)*X(188)&
             &-JVS(1621)*X(189)-JVS(1622)*X(190)-JVS(1623)*X(191)-JVS(1624)*X(193)-JVS(1625)*X(194)-JVS(1626)*X(195))&
             &/(JVS(1614))
  X(181) = (X(181)-JVS(1577)*X(182)-JVS(1578)*X(183)-JVS(1579)*X(184)-JVS(1580)*X(185)-JVS(1581)*X(186)-JVS(1582)*X(188)&
             &-JVS(1583)*X(189)-JVS(1584)*X(190)-JVS(1585)*X(193)-JVS(1586)*X(194)-JVS(1587)*X(195))/(JVS(1576))
  X(180) = (X(180)-JVS(1517)*X(181)-JVS(1518)*X(182)-JVS(1519)*X(183)-JVS(1520)*X(184)-JVS(1521)*X(185)-JVS(1522)*X(186)&
             &-JVS(1523)*X(188)-JVS(1524)*X(189)-JVS(1525)*X(190)-JVS(1526)*X(193)-JVS(1527)*X(194)-JVS(1528)*X(195))&
             &/(JVS(1516))
  X(179) = (X(179)-JVS(1456)*X(180)-JVS(1457)*X(181)-JVS(1458)*X(182)-JVS(1459)*X(183)-JVS(1460)*X(184)-JVS(1461)*X(185)&
             &-JVS(1462)*X(186)-JVS(1463)*X(188)-JVS(1464)*X(189)-JVS(1465)*X(190)-JVS(1466)*X(193)-JVS(1467)*X(194)&
             &-JVS(1468)*X(195))/(JVS(1455))
  X(178) = (X(178)-JVS(1385)*X(180)-JVS(1386)*X(181)-JVS(1387)*X(182)-JVS(1388)*X(183)-JVS(1389)*X(186)-JVS(1390)*X(188)&
             &-JVS(1391)*X(189)-JVS(1392)*X(190)-JVS(1393)*X(193)-JVS(1394)*X(195))/(JVS(1384))
  X(177) = (X(177)-JVS(1332)*X(178)-JVS(1333)*X(180)-JVS(1334)*X(181)-JVS(1335)*X(182)-JVS(1336)*X(183)-JVS(1337)*X(186)&
             &-JVS(1338)*X(188)-JVS(1339)*X(189)-JVS(1340)*X(193)-JVS(1341)*X(195))/(JVS(1331))
  X(176) = (X(176)-JVS(1305)*X(177)-JVS(1306)*X(178)-JVS(1307)*X(180)-JVS(1308)*X(181)-JVS(1309)*X(182)-JVS(1310)*X(183)&
             &-JVS(1311)*X(186)-JVS(1312)*X(189)-JVS(1313)*X(190)-JVS(1314)*X(193)-JVS(1315)*X(195))/(JVS(1304))
  X(175) = (X(175)-JVS(1272)*X(177)-JVS(1273)*X(178)-JVS(1274)*X(180)-JVS(1275)*X(181)-JVS(1276)*X(182)-JVS(1277)*X(183)&
             &-JVS(1278)*X(186)-JVS(1279)*X(189)-JVS(1280)*X(193)-JVS(1281)*X(195))/(JVS(1271))
  X(174) = (X(174)-JVS(1254)*X(175)-JVS(1255)*X(177)-JVS(1256)*X(178)-JVS(1257)*X(180)-JVS(1258)*X(181)-JVS(1259)*X(182)&
             &-JVS(1260)*X(183)-JVS(1261)*X(186)-JVS(1262)*X(189)-JVS(1263)*X(190)-JVS(1264)*X(193)-JVS(1265)*X(195))&
             &/(JVS(1253))
  X(173) = (X(173)-JVS(1222)*X(177)-JVS(1223)*X(178)-JVS(1224)*X(180)-JVS(1225)*X(181)-JVS(1226)*X(182)-JVS(1227)*X(183)&
             &-JVS(1228)*X(186)-JVS(1229)*X(189)-JVS(1230)*X(193)-JVS(1231)*X(195))/(JVS(1221))
  X(172) = (X(172)-JVS(1203)*X(182)-JVS(1204)*X(183)-JVS(1205)*X(185)-JVS(1206)*X(186)-JVS(1207)*X(187)-JVS(1208)*X(188)&
             &-JVS(1209)*X(189)-JVS(1210)*X(190)-JVS(1211)*X(191)-JVS(1212)*X(192)-JVS(1213)*X(193)-JVS(1214)*X(194)&
             &-JVS(1215)*X(195))/(JVS(1202))
  X(171) = (X(171)-JVS(1180)*X(173)-JVS(1181)*X(174)-JVS(1182)*X(175)-JVS(1183)*X(177)-JVS(1184)*X(178)-JVS(1185)*X(180)&
             &-JVS(1186)*X(181)-JVS(1187)*X(182)-JVS(1188)*X(183)-JVS(1189)*X(186)-JVS(1190)*X(189)-JVS(1191)*X(190)&
             &-JVS(1192)*X(193)-JVS(1193)*X(195))/(JVS(1179))
  X(170) = (X(170)-JVS(1138)*X(174)-JVS(1139)*X(175)-JVS(1140)*X(178)-JVS(1141)*X(181)-JVS(1142)*X(182)-JVS(1143)*X(183)&
             &-JVS(1144)*X(186)-JVS(1145)*X(189)-JVS(1146)*X(190)-JVS(1147)*X(193)-JVS(1148)*X(195))/(JVS(1137))
  X(169) = (X(169)-JVS(1115)*X(170)-JVS(1116)*X(175)-JVS(1117)*X(178)-JVS(1118)*X(181)-JVS(1119)*X(182)-JVS(1120)*X(183)&
             &-JVS(1121)*X(186)-JVS(1122)*X(189)-JVS(1123)*X(193))/(JVS(1114))
  X(168) = (X(168)-JVS(1096)*X(175)-JVS(1097)*X(178)-JVS(1098)*X(181)-JVS(1099)*X(182)-JVS(1100)*X(183)-JVS(1101)*X(186)&
             &-JVS(1102)*X(189)-JVS(1103)*X(193))/(JVS(1095))
  X(167) = (X(167)-JVS(1080)*X(168)-JVS(1081)*X(175)-JVS(1082)*X(178)-JVS(1083)*X(181)-JVS(1084)*X(182)-JVS(1085)*X(183)&
             &-JVS(1086)*X(186)-JVS(1087)*X(189)-JVS(1088)*X(193))/(JVS(1079))
  X(166) = (X(166)-JVS(1057)*X(167)-JVS(1058)*X(168)-JVS(1059)*X(169)-JVS(1060)*X(170)-JVS(1061)*X(173)-JVS(1062)*X(174)&
             &-JVS(1063)*X(175)-JVS(1064)*X(177)-JVS(1065)*X(178)-JVS(1066)*X(180)-JVS(1067)*X(181)-JVS(1068)*X(182)&
             &-JVS(1069)*X(183)-JVS(1070)*X(186)-JVS(1071)*X(189)-JVS(1072)*X(190)-JVS(1073)*X(193)-JVS(1074)*X(195))&
             &/(JVS(1056))
  X(165) = (X(165)-JVS(1027)*X(178)-JVS(1028)*X(181)-JVS(1029)*X(182)-JVS(1030)*X(183)-JVS(1031)*X(186)-JVS(1032)*X(189)&
             &-JVS(1033)*X(193))/(JVS(1026))
  X(164) = (X(164)-JVS(1014)*X(178)-JVS(1015)*X(181)-JVS(1016)*X(182)-JVS(1017)*X(183)-JVS(1018)*X(186)-JVS(1019)*X(189)&
             &-JVS(1020)*X(193))/(JVS(1013))
  X(163) = (X(163)-JVS(1004)*X(178)-JVS(1005)*X(181)-JVS(1006)*X(182)-JVS(1007)*X(183)-JVS(1008)*X(186)-JVS(1009)*X(189)&
             &-JVS(1010)*X(193))/(JVS(1003))
  X(162) = (X(162)-JVS(977)*X(163)-JVS(978)*X(164)-JVS(979)*X(165)-JVS(980)*X(167)-JVS(981)*X(168)-JVS(982)*X(169)&
             &-JVS(983)*X(170)-JVS(984)*X(173)-JVS(985)*X(174)-JVS(986)*X(175)-JVS(987)*X(177)-JVS(988)*X(178)-JVS(989)&
             &*X(180)-JVS(990)*X(181)-JVS(991)*X(182)-JVS(992)*X(183)-JVS(993)*X(186)-JVS(994)*X(189)-JVS(995)*X(190)&
             &-JVS(996)*X(193)-JVS(997)*X(195))/(JVS(976))
  X(161) = (X(161)-JVS(945)*X(175)-JVS(946)*X(177)-JVS(947)*X(178)-JVS(948)*X(180)-JVS(949)*X(181)-JVS(950)*X(182)&
             &-JVS(951)*X(183)-JVS(952)*X(186)-JVS(953)*X(189)-JVS(954)*X(193)-JVS(955)*X(195))/(JVS(944))
  X(160) = (X(160)-JVS(932)*X(163)-JVS(933)*X(175)-JVS(934)*X(178)-JVS(935)*X(181)-JVS(936)*X(182)-JVS(937)*X(183)&
             &-JVS(938)*X(186)-JVS(939)*X(189)-JVS(940)*X(193)-JVS(941)*X(195))/(JVS(931))
  X(159) = (X(159)-JVS(896)*X(160)-JVS(897)*X(163)-JVS(898)*X(164)-JVS(899)*X(165)-JVS(900)*X(166)-JVS(901)*X(167)&
             &-JVS(902)*X(168)-JVS(903)*X(171)-JVS(904)*X(172)-JVS(905)*X(173)-JVS(906)*X(174)-JVS(907)*X(175)-JVS(908)&
             &*X(176)-JVS(909)*X(177)-JVS(910)*X(178)-JVS(911)*X(179)-JVS(912)*X(180)-JVS(913)*X(181)-JVS(914)*X(182)&
             &-JVS(915)*X(183)-JVS(916)*X(184)-JVS(917)*X(185)-JVS(918)*X(186)-JVS(919)*X(187)-JVS(920)*X(188)-JVS(921)&
             &*X(189)-JVS(922)*X(190)-JVS(923)*X(191)-JVS(924)*X(192)-JVS(925)*X(193)-JVS(926)*X(195))/(JVS(895))
  X(158) = (X(158)-JVS(875)*X(163)-JVS(876)*X(178)-JVS(877)*X(181)-JVS(878)*X(182)-JVS(879)*X(183)-JVS(880)*X(186)&
             &-JVS(881)*X(189))/(JVS(874))
  X(157) = (X(157)-JVS(864)*X(169)-JVS(865)*X(170)-JVS(866)*X(174)-JVS(867)*X(178)-JVS(868)*X(181)-JVS(869)*X(183)&
             &-JVS(870)*X(186)-JVS(871)*X(189)-JVS(872)*X(190)-JVS(873)*X(193))/(JVS(863))
  X(156) = (X(156)-JVS(846)*X(172)-JVS(847)*X(181)-JVS(848)*X(184)-JVS(849)*X(185)-JVS(850)*X(186)-JVS(851)*X(187)&
             &-JVS(852)*X(188)-JVS(853)*X(189)-JVS(854)*X(191)-JVS(855)*X(192)-JVS(856)*X(193)-JVS(857)*X(194)-JVS(858)&
             &*X(195))/(JVS(845))
  X(155) = (X(155)-JVS(829)*X(167)-JVS(830)*X(168)-JVS(831)*X(170)-JVS(832)*X(175)-JVS(833)*X(178)-JVS(834)*X(181)&
             &-JVS(835)*X(183)-JVS(836)*X(186)-JVS(837)*X(189)-JVS(838)*X(193))/(JVS(828))
  X(154) = (X(154)-JVS(818)*X(184)-JVS(819)*X(185)-JVS(820)*X(187)-JVS(821)*X(188)-JVS(822)*X(189)-JVS(823)*X(191)&
             &-JVS(824)*X(192)-JVS(825)*X(195))/(JVS(817))
  X(153) = (X(153)-JVS(809)*X(169)-JVS(810)*X(170)-JVS(811)*X(178)-JVS(812)*X(181)-JVS(813)*X(183)-JVS(814)*X(189)&
             &-JVS(815)*X(190)-JVS(816)*X(193))/(JVS(808))
  X(152) = (X(152)-JVS(800)*X(165)-JVS(801)*X(178)-JVS(802)*X(181)-JVS(803)*X(183)-JVS(804)*X(189)-JVS(805)*X(193))&
             &/(JVS(799))
  X(151) = (X(151)-JVS(781)*X(152)-JVS(782)*X(163)-JVS(783)*X(164)-JVS(784)*X(165)-JVS(785)*X(167)-JVS(786)*X(168)&
             &-JVS(787)*X(173)-JVS(788)*X(177)-JVS(789)*X(178)-JVS(790)*X(180)-JVS(791)*X(181)-JVS(792)*X(182)-JVS(793)&
             &*X(183)-JVS(794)*X(186)-JVS(795)*X(189)-JVS(796)*X(193)-JVS(797)*X(195))/(JVS(780))
  X(150) = (X(150)-JVS(761)*X(158)-JVS(762)*X(171)-JVS(763)*X(173)-JVS(764)*X(174)-JVS(765)*X(178)-JVS(766)*X(181)&
             &-JVS(767)*X(183)-JVS(768)*X(186)-JVS(769)*X(189)-JVS(770)*X(193)-JVS(771)*X(195))/(JVS(760))
  X(149) = (X(149)-JVS(750)*X(174)-JVS(751)*X(178)-JVS(752)*X(181)-JVS(753)*X(183)-JVS(754)*X(186)-JVS(755)*X(189)&
             &-JVS(756)*X(193))/(JVS(749))
  X(148) = (X(148)-JVS(737)*X(150)-JVS(738)*X(169)-JVS(739)*X(170)-JVS(740)*X(171)-JVS(741)*X(174)-JVS(742)*X(178)&
             &-JVS(743)*X(181)-JVS(744)*X(183)-JVS(745)*X(186)-JVS(746)*X(189)-JVS(747)*X(190)-JVS(748)*X(193))/(JVS(736))
  X(147) = (X(147)-JVS(726)*X(173)-JVS(727)*X(178)-JVS(728)*X(181)-JVS(729)*X(182)-JVS(730)*X(183)-JVS(731)*X(186)&
             &-JVS(732)*X(189)-JVS(733)*X(193))/(JVS(725))
  X(146) = (X(146)-JVS(715)*X(173)-JVS(716)*X(178)-JVS(717)*X(181)-JVS(718)*X(182)-JVS(719)*X(183)-JVS(720)*X(186)&
             &-JVS(721)*X(189)-JVS(722)*X(193))/(JVS(714))
  X(145) = (X(145)-JVS(706)*X(175)-JVS(707)*X(178)-JVS(708)*X(181)-JVS(709)*X(183)-JVS(710)*X(186)-JVS(711)*X(193))&
             &/(JVS(705))
  X(144) = (X(144)-JVS(699)*X(158)-JVS(700)*X(183)-JVS(701)*X(186)-JVS(702)*X(189)-JVS(703)*X(193)-JVS(704)*X(195))&
             &/(JVS(698))
  X(143) = (X(143)-JVS(685)*X(165)-JVS(686)*X(173)-JVS(687)*X(175)-JVS(688)*X(177)-JVS(689)*X(180)-JVS(690)*X(182)&
             &-JVS(691)*X(183)-JVS(692)*X(184)-JVS(693)*X(186)-JVS(694)*X(188)-JVS(695)*X(189)-JVS(696)*X(193)-JVS(697)&
             &*X(195))/(JVS(684))
  X(142) = (X(142)-JVS(668)*X(177)-JVS(669)*X(180)-JVS(670)*X(182)-JVS(671)*X(183)-JVS(672)*X(186)-JVS(673)*X(189)&
             &-JVS(674)*X(195))/(JVS(667))
  X(141) = (X(141)-JVS(658)*X(173)-JVS(659)*X(178)-JVS(660)*X(181)-JVS(661)*X(182)-JVS(662)*X(183)-JVS(663)*X(186)&
             &-JVS(664)*X(189)-JVS(665)*X(193))/(JVS(657))
  X(140) = (X(140)-JVS(646)*X(153)-JVS(647)*X(169)-JVS(648)*X(170)-JVS(649)*X(178)-JVS(650)*X(181)-JVS(651)*X(183)&
             &-JVS(652)*X(189)-JVS(653)*X(190)-JVS(654)*X(193))/(JVS(645))
  X(139) = (X(139)-JVS(636)*X(161)-JVS(637)*X(164)-JVS(638)*X(167)-JVS(639)*X(168)-JVS(640)*X(182)-JVS(641)*X(183)&
             &-JVS(642)*X(189)-JVS(643)*X(193))/(JVS(635))
  X(138) = (X(138)-JVS(627)*X(173)-JVS(628)*X(176)-JVS(629)*X(182)-JVS(630)*X(184)-JVS(631)*X(186)-JVS(632)*X(189)&
             &-JVS(633)*X(194))/(JVS(626))
  X(137) = (X(137)-JVS(619)*X(160)-JVS(620)*X(178)-JVS(621)*X(181)-JVS(622)*X(183)-JVS(623)*X(189)-JVS(624)*X(193))&
             &/(JVS(618))
  X(136) = (X(136)-JVS(606)*X(151)-JVS(607)*X(152)-JVS(608)*X(158)-JVS(609)*X(164)-JVS(610)*X(178)-JVS(611)*X(181)&
             &-JVS(612)*X(182)-JVS(613)*X(183)-JVS(614)*X(186)-JVS(615)*X(189)-JVS(616)*X(193))/(JVS(605))
  X(135) = (X(135)-JVS(590)*X(145)-JVS(591)*X(146)-JVS(592)*X(163)-JVS(593)*X(164)-JVS(594)*X(168)-JVS(595)*X(178)&
             &-JVS(596)*X(181)-JVS(597)*X(182)-JVS(598)*X(183)-JVS(599)*X(189)-JVS(600)*X(193))/(JVS(589))
  X(134) = (X(134)-JVS(579)*X(169)-JVS(580)*X(170)-JVS(581)*X(178)-JVS(582)*X(181)-JVS(583)*X(183)-JVS(584)*X(189)&
             &-JVS(585)*X(190)-JVS(586)*X(193))/(JVS(578))
  X(133) = (X(133)-JVS(567)*X(172)-JVS(568)*X(187)-JVS(569)*X(188)-JVS(570)*X(189)-JVS(571)*X(190)-JVS(572)*X(191)&
             &-JVS(573)*X(192)-JVS(574)*X(193)-JVS(575)*X(195))/(JVS(566))
  X(132) = (X(132)-JVS(558)*X(173)-JVS(559)*X(177)-JVS(560)*X(180)-JVS(561)*X(182)-JVS(562)*X(183)-JVS(563)*X(195))&
             &/(JVS(557))
  X(131) = (X(131)-JVS(552)*X(158)-JVS(553)*X(181)-JVS(554)*X(183)-JVS(555)*X(186)-JVS(556)*X(193))/(JVS(551))
  X(130) = (X(130)-JVS(547)*X(173)-JVS(548)*X(182)-JVS(549)*X(186)-JVS(550)*X(189))/(JVS(546))
  X(129) = (X(129)-JVS(539)*X(163)-JVS(540)*X(164)-JVS(541)*X(181)-JVS(542)*X(182)-JVS(543)*X(183)-JVS(544)*X(189))&
             &/(JVS(538))
  X(128) = (X(128)-JVS(535)*X(182)-JVS(536)*X(186)-JVS(537)*X(189))/(JVS(534))
  X(127) = (X(127)-JVS(526)*X(128)-JVS(527)*X(130)-JVS(528)*X(173)-JVS(529)*X(175)-JVS(530)*X(177)-JVS(531)*X(182)&
             &-JVS(532)*X(183)-JVS(533)*X(195))/(JVS(525))
  X(126) = (X(126)-JVS(518)*X(165)-JVS(519)*X(175)-JVS(520)*X(177)-JVS(521)*X(180)-JVS(522)*X(182)-JVS(523)*X(183)&
             &-JVS(524)*X(195))/(JVS(517))
  X(125) = (X(125)-JVS(512)*X(158)-JVS(513)*X(181)-JVS(514)*X(183)-JVS(515)*X(189)-JVS(516)*X(193))/(JVS(511))
  X(124) = (X(124)-JVS(502)*X(154)-JVS(503)*X(156)-JVS(504)*X(172)-JVS(505)*X(181)-JVS(506)*X(184)-JVS(507)*X(185)&
             &-JVS(508)*X(189)-JVS(509)*X(191)-JVS(510)*X(194))/(JVS(501))
  X(123) = (X(123)-JVS(494)*X(165)-JVS(495)*X(177)-JVS(496)*X(180)-JVS(497)*X(182)-JVS(498)*X(183)-JVS(499)*X(195))&
             &/(JVS(493))
  X(122) = (X(122)-JVS(483)*X(138)-JVS(484)*X(179)-JVS(485)*X(182)-JVS(486)*X(184)-JVS(487)*X(187)-JVS(488)*X(188)&
             &-JVS(489)*X(189)-JVS(490)*X(191)-JVS(491)*X(193)-JVS(492)*X(194))/(JVS(482))
  X(121) = (X(121)-JVS(477)*X(172)-JVS(478)*X(190)-JVS(479)*X(191)-JVS(480)*X(192)-JVS(481)*X(195))/(JVS(476))
  X(120) = (X(120)-JVS(469)*X(147)-JVS(470)*X(164)-JVS(471)*X(167)-JVS(472)*X(182)-JVS(473)*X(183)-JVS(474)*X(189)&
             &-JVS(475)*X(193))/(JVS(468))
  X(119) = (X(119)-JVS(461)*X(130)-JVS(462)*X(177)-JVS(463)*X(180)-JVS(464)*X(182)-JVS(465)*X(183)-JVS(466)*X(195))&
             &/(JVS(460))
  X(118) = (X(118)-JVS(454)*X(128)-JVS(455)*X(177)-JVS(456)*X(180)-JVS(457)*X(182)-JVS(458)*X(183)-JVS(459)*X(195))&
             &/(JVS(453))
  X(117) = (X(117)-JVS(448)*X(144)-JVS(449)*X(160)-JVS(450)*X(182)-JVS(451)*X(189)-JVS(452)*X(195))/(JVS(447))
  X(116) = (X(116)-JVS(443)*X(164)-JVS(444)*X(178)-JVS(445)*X(183)-JVS(446)*X(193))/(JVS(442))
  X(115) = (X(115)-JVS(436)*X(152)-JVS(437)*X(178)-JVS(438)*X(182)-JVS(439)*X(183)-JVS(440)*X(189)-JVS(441)*X(193))&
             &/(JVS(435))
  X(114) = (X(114)-JVS(420)*X(134)-JVS(421)*X(137)-JVS(422)*X(141)-JVS(423)*X(146)-JVS(424)*X(147)-JVS(425)*X(149)&
             &-JVS(426)*X(152)-JVS(427)*X(153)-JVS(428)*X(155)-JVS(429)*X(161)-JVS(430)*X(164)-JVS(431)*X(169)-JVS(432)&
             &*X(181)-JVS(433)*X(189))/(JVS(419))
  X(113) = (X(113)-JVS(414)*X(139)-JVS(415)*X(183)-JVS(416)*X(189)-JVS(417)*X(193)-JVS(418)*X(195))/(JVS(413))
  X(112) = (X(112)-JVS(408)*X(164)-JVS(409)*X(183)-JVS(410)*X(189)-JVS(411)*X(193))/(JVS(407))
  X(111) = (X(111)-JVS(401)*X(164)-JVS(402)*X(182)-JVS(403)*X(183)-JVS(404)*X(189))/(JVS(400))
  X(110) = (X(110)-JVS(395)*X(145)-JVS(396)*X(167)-JVS(397)*X(168)-JVS(398)*X(189)-JVS(399)*X(193))/(JVS(394))
  X(109) = (X(109)-JVS(389)*X(125)-JVS(390)*X(131)-JVS(391)*X(163)-JVS(392)*X(189)-JVS(393)*X(193))/(JVS(388))
  X(108) = (X(108)-JVS(384)*X(186)-JVS(385)*X(191)-JVS(386)*X(192)-JVS(387)*X(195))/(JVS(383))
  X(107) = (X(107)-JVS(378)*X(152)-JVS(379)*X(178)-JVS(380)*X(182)-JVS(381)*X(183)-JVS(382)*X(189))/(JVS(377))
  X(106) = (X(106)-JVS(372)*X(152)-JVS(373)*X(178)-JVS(374)*X(183)-JVS(375)*X(189)-JVS(376)*X(193))/(JVS(371))
  X(105) = (X(105)-JVS(367)*X(152)-JVS(368)*X(189)-JVS(369)*X(193))/(JVS(366))
  X(104) = (X(104)-JVS(363)*X(186)-JVS(364)*X(189)-JVS(365)*X(190))/(JVS(362))
  X(103) = (X(103)-JVS(358)*X(142)-JVS(359)*X(161)-JVS(360)*X(189)-JVS(361)*X(193))/(JVS(357))
  X(102) = (X(102)-JVS(354)*X(147)-JVS(355)*X(189)-JVS(356)*X(193))/(JVS(353))
  X(101) = (X(101)-JVS(350)*X(164)-JVS(351)*X(189)-JVS(352)*X(193))/(JVS(349))
  X(100) = (X(100)-JVS(345)*X(149)-JVS(346)*X(157)-JVS(347)*X(189)-JVS(348)*X(193))/(JVS(344))
  X(99) = (X(99)-JVS(339)*X(115)-JVS(340)*X(116)-JVS(341)*X(169)-JVS(342)*X(189)-JVS(343)*X(193))/(JVS(338))
  X(98) = (X(98)-JVS(332)*X(121)-JVS(333)*X(133)-JVS(334)*X(172)-JVS(335)*X(187)-JVS(336)*X(189)-JVS(337)*X(190))&
            &/(JVS(331))
  X(97) = (X(97)-JVS(328)*X(146)-JVS(329)*X(189)-JVS(330)*X(193))/(JVS(327))
  X(96) = (X(96)-JVS(324)*X(141)-JVS(325)*X(189)-JVS(326)*X(193))/(JVS(323))
  X(95) = (X(95)-JVS(317)*X(129)-JVS(318)*X(168)-JVS(319)*X(182)-JVS(320)*X(183)-JVS(321)*X(189)-JVS(322)*X(193))&
            &/(JVS(316))
  X(94) = (X(94)-JVS(307)*X(118)-JVS(308)*X(119)-JVS(309)*X(123)-JVS(310)*X(126)-JVS(311)*X(127)-JVS(312)*X(132)&
            &-JVS(313)*X(142)-JVS(314)*X(177)-JVS(315)*X(189))/(JVS(306))
  X(93) = (X(93)-JVS(302)*X(144)-JVS(303)*X(150)-JVS(304)*X(189)-JVS(305)*X(193))/(JVS(301))
  X(92) = (X(92)-JVS(293)*X(121)-JVS(294)*X(133)-JVS(295)*X(154)-JVS(296)*X(156)-JVS(297)*X(172)-JVS(298)*X(185)&
            &-JVS(299)*X(187)-JVS(300)*X(191))/(JVS(292))
  X(91) = (X(91)-JVS(289)*X(160)-JVS(290)*X(189)-JVS(291)*X(193))/(JVS(288))
  X(90) = (X(90)-JVS(285)*X(189)-JVS(286)*X(193)-JVS(287)*X(195))/(JVS(284))
  X(89) = (X(89)-JVS(281)*X(181)-JVS(282)*X(189)-JVS(283)*X(193))/(JVS(280))
  X(88) = (X(88)-JVS(275)*X(172)-JVS(276)*X(181)-JVS(277)*X(184)-JVS(278)*X(185)-JVS(279)*X(189))/(JVS(274))
  X(87) = (X(87)-JVS(271)*X(184)-JVS(272)*X(189))/(JVS(270))
  X(86) = (X(86)-JVS(268)*X(189)-JVS(269)*X(194))/(JVS(267))
  X(85) = (X(85)-JVS(264)*X(183)-JVS(265)*X(189)-JVS(266)*X(195))/(JVS(263))
  X(84) = (X(84)-JVS(260)*X(148)-JVS(261)*X(189)-JVS(262)*X(193))/(JVS(259))
  X(83) = (X(83)-JVS(257)*X(189)-JVS(258)*X(190))/(JVS(256))
  X(82) = (X(82)-JVS(254)*X(189)-JVS(255)*X(194))/(JVS(253))
  X(81) = (X(81)-JVS(251)*X(189)-JVS(252)*X(194))/(JVS(250))
  X(80) = (X(80)-JVS(248)*X(189)-JVS(249)*X(194))/(JVS(247))
  X(79) = (X(79)-JVS(245)*X(189)-JVS(246)*X(194))/(JVS(244))
  X(78) = (X(78)-JVS(241)*X(108)-JVS(242)*X(189)-JVS(243)*X(191))/(JVS(240))
  X(77) = (X(77)-JVS(237)*X(153)-JVS(238)*X(189)-JVS(239)*X(193))/(JVS(236))
  X(76) = (X(76)-JVS(233)*X(172)-JVS(234)*X(185)-JVS(235)*X(189))/(JVS(232))
  X(75) = (X(75)-JVS(229)*X(134)-JVS(230)*X(189)-JVS(231)*X(193))/(JVS(228))
  X(74) = (X(74)-JVS(226)*X(186)-JVS(227)*X(189))/(JVS(225))
  X(73) = (X(73)-JVS(223)*X(185)-JVS(224)*X(189))/(JVS(222))
  X(72) = (X(72)-JVS(220)*X(181)-JVS(221)*X(195))/(JVS(219))
  X(71) = (X(71)-JVS(217)*X(189)-JVS(218)*X(194))/(JVS(216))
  X(70) = (X(70)-JVS(214)*X(189)-JVS(215)*X(194))/(JVS(213))
  X(69) = (X(69)-JVS(210)*X(178)-JVS(211)*X(189)-JVS(212)*X(193))/(JVS(209))
  X(68) = (X(68)-JVS(207)*X(186)-JVS(208)*X(189))/(JVS(206))
  X(67) = (X(67)-JVS(204)*X(188)-JVS(205)*X(189))/(JVS(203))
  X(66) = (X(66)-JVS(201)*X(183)-JVS(202)*X(195))/(JVS(200))
  X(65) = (X(65)-JVS(198)*X(178)-JVS(199)*X(195))/(JVS(197))
  X(64) = (X(64)-JVS(196)*X(194))/(JVS(195))
  X(63) = (X(63)-JVS(192)*X(66)-JVS(193)*X(194)-JVS(194)*X(195))/(JVS(191))
  X(62) = (X(62)-JVS(188)*X(183)-JVS(189)*X(189)-JVS(190)*X(193))/(JVS(187))
  X(61) = (X(61)-JVS(183)*X(183)-JVS(184)*X(189)-JVS(185)*X(193))/(JVS(182))
  X(60) = (X(60)-JVS(178)*X(183)-JVS(179)*X(189)-JVS(180)*X(193))/(JVS(177))
  X(59) = (X(59)-JVS(175)*X(194))/(JVS(174))
  X(58) = (X(58)-JVS(173)*X(194))/(JVS(172))
  X(57) = (X(57)-JVS(171)*X(194))/(JVS(170))
  X(56) = (X(56)-JVS(169)*X(194))/(JVS(168))
  X(55) = (X(55)-JVS(167)*X(194))/(JVS(166))
  X(54) = (X(54)-JVS(165)*X(194))/(JVS(164))
  X(53) = (X(53)-JVS(163)*X(194))/(JVS(162))
  X(52) = (X(52)-JVS(161)*X(194))/(JVS(160))
  X(51) = (X(51)-JVS(158)*X(113)-JVS(159)*X(195))/(JVS(157))
  X(50) = (X(50)-JVS(155)*X(101)-JVS(156)*X(189))/(JVS(154))
  X(49) = (X(49)-JVS(152)*X(190)-JVS(153)*X(195))/(JVS(151))
  X(48) = (X(48)-JVS(149)*X(150)-JVS(150)*X(195))/(JVS(148))
  X(47) = (X(47)-JVS(147)*X(189))/(JVS(146))
  X(46) = (X(46)-JVS(145)*X(189))/(JVS(144))
  X(45) = (X(45)-JVS(143)*X(189))/(JVS(142))
  X(44) = (X(44)-JVS(141)*X(189))/(JVS(140))
  X(43) = (X(43)-JVS(112)*X(48)-JVS(113)*X(50)-JVS(114)*X(51)-JVS(115)*X(65)-JVS(116)*X(98)-JVS(117)*X(101)-JVS(118)&
            &*X(108)-JVS(119)*X(111)-JVS(120)*X(117)-JVS(121)*X(120)-JVS(122)*X(121)-JVS(123)*X(129)-JVS(124)*X(133)&
            &-JVS(125)*X(135)-JVS(126)*X(139)-JVS(127)*X(140)-JVS(128)*X(143)-JVS(129)*X(159)-JVS(130)*X(162)-JVS(131)&
            &*X(170)-JVS(132)*X(173)-JVS(133)*X(175)-JVS(134)*X(176)-JVS(135)*X(179)-JVS(136)*X(182)-JVS(137)*X(187)&
            &-JVS(138)*X(191)-JVS(139)*X(195))/(JVS(111))
  X(42) = (X(42)-JVS(91)*X(67)-JVS(92)*X(95)-JVS(93)*X(103)-JVS(94)*X(107)-JVS(95)*X(112)-JVS(96)*X(113)-JVS(97)*X(115)&
            &-JVS(98)*X(137)-JVS(99)*X(145)-JVS(100)*X(151)-JVS(101)*X(160)-JVS(102)*X(162)-JVS(103)*X(164)-JVS(104)*X(178)&
            &-JVS(105)*X(180)-JVS(106)*X(181)-JVS(107)*X(182)-JVS(108)*X(183)-JVS(109)*X(189)-JVS(110)*X(193))/(JVS(90))
  X(41) = (X(41)-JVS(89)*X(191))/(JVS(88))
  X(40) = X(40)/JVS(87)
  X(39) = X(39)/JVS(86)
  X(38) = (X(38)-JVS(85)*X(135))/(JVS(84))
  X(37) = (X(37)-JVS(83)*X(120))/(JVS(82))
  X(36) = (X(36)-JVS(81)*X(139))/(JVS(80))
  X(35) = (X(35)-JVS(79)*X(50))/(JVS(78))
  X(34) = (X(34)-JVS(77)*X(101))/(JVS(76))
  X(33) = (X(33)-JVS(75)*X(111))/(JVS(74))
  X(32) = (X(32)-JVS(73)*X(129))/(JVS(72))
  X(31) = (X(31)-JVS(70)*X(120)-JVS(71)*X(189))/(JVS(69))
  X(30) = (X(30)-JVS(68)*X(98))/(JVS(67))
  X(29) = (X(29)-JVS(66)*X(121))/(JVS(65))
  X(28) = (X(28)-JVS(64)*X(187))/(JVS(63))
  X(27) = (X(27)-JVS(62)*X(133))/(JVS(61))
  X(26) = (X(26)-JVS(59)*X(62)-JVS(60)*X(183))/(JVS(58))
  X(25) = (X(25)-JVS(56)*X(62)-JVS(57)*X(193))/(JVS(55))
  X(24) = (X(24)-JVS(53)*X(61)-JVS(54)*X(183))/(JVS(52))
  X(23) = (X(23)-JVS(50)*X(61)-JVS(51)*X(193))/(JVS(49))
  X(22) = (X(22)-JVS(47)*X(60)-JVS(48)*X(183))/(JVS(46))
  X(21) = (X(21)-JVS(44)*X(60)-JVS(45)*X(193))/(JVS(43))
  X(20) = (X(20)-JVS(41)*X(128)-JVS(42)*X(186))/(JVS(40))
  X(19) = (X(19)-JVS(38)*X(128)-JVS(39)*X(189))/(JVS(37))
  X(18) = (X(18)-JVS(35)*X(74)-JVS(36)*X(189))/(JVS(34))
  X(17) = (X(17)-JVS(33)*X(176))/(JVS(32))
  X(16) = (X(16)-JVS(31)*X(162))/(JVS(30))
  X(15) = (X(15)-JVS(29)*X(175))/(JVS(28))
  X(14) = (X(14)-JVS(27)*X(173))/(JVS(26))
  X(13) = (X(13)-JVS(25)*X(170))/(JVS(24))
  X(12) = (X(12)-JVS(23)*X(48))/(JVS(22))
  X(11) = (X(11)-JVS(21)*X(51))/(JVS(20))
  X(10) = (X(10)-JVS(19)*X(117))/(JVS(18))
  X(9) = (X(9)-JVS(17)*X(65))/(JVS(16))
  X(8) = (X(8)-JVS(15)*X(182))/(JVS(14))
  X(7) = (X(7)-JVS(13)*X(195))/(JVS(12))
  X(6) = (X(6)-JVS(11)*X(108))/(JVS(10))
  X(5) = (X(5)-JVS(9)*X(159))/(JVS(8))
  X(4) = (X(4)-JVS(7)*X(143))/(JVS(6))
  X(3) = (X(3)-JVS(5)*X(179))/(JVS(4))
  X(2) = (X(2)-JVS(3)*X(140))/(JVS(2))
  X(1) = X(1)/JVS(1)
      
END SUBROUTINE KppSolve

! End of KppSolve function
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~


! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
! 
! KppSolveTR - sparse, transposed back substitution
!   Arguments :
!      JVS       - sparse Jacobian of variables
!      X         - Vector for variables
!      XX        - Vector for output variables
! 
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

SUBROUTINE KppSolveTR ( JVS, X, XX )

! JVS - sparse Jacobian of variables
  REAL(kind=dp) :: JVS(LU_NONZERO)
! X - Vector for variables
  REAL(kind=dp) :: X(NVAR)
! XX - Vector for output variables
  REAL(kind=dp) :: XX(NVAR)

  XX(1) = X(1)/JVS(1)
  XX(2) = X(2)/JVS(2)
  XX(3) = X(3)/JVS(4)
  XX(4) = X(4)/JVS(6)
  XX(5) = X(5)/JVS(8)
  XX(6) = X(6)/JVS(10)
  XX(7) = X(7)/JVS(12)
  XX(8) = X(8)/JVS(14)
  XX(9) = X(9)/JVS(16)
  XX(10) = X(10)/JVS(18)
  XX(11) = X(11)/JVS(20)
  XX(12) = X(12)/JVS(22)
  XX(13) = X(13)/JVS(24)
  XX(14) = X(14)/JVS(26)
  XX(15) = X(15)/JVS(28)
  XX(16) = X(16)/JVS(30)
  XX(17) = X(17)/JVS(32)
  XX(18) = X(18)/JVS(34)
  XX(19) = X(19)/JVS(37)
  XX(20) = X(20)/JVS(40)
  XX(21) = X(21)/JVS(43)
  XX(22) = X(22)/JVS(46)
  XX(23) = X(23)/JVS(49)
  XX(24) = X(24)/JVS(52)
  XX(25) = X(25)/JVS(55)
  XX(26) = X(26)/JVS(58)
  XX(27) = X(27)/JVS(61)
  XX(28) = X(28)/JVS(63)
  XX(29) = X(29)/JVS(65)
  XX(30) = X(30)/JVS(67)
  XX(31) = X(31)/JVS(69)
  XX(32) = X(32)/JVS(72)
  XX(33) = X(33)/JVS(74)
  XX(34) = X(34)/JVS(76)
  XX(35) = X(35)/JVS(78)
  XX(36) = X(36)/JVS(80)
  XX(37) = X(37)/JVS(82)
  XX(38) = X(38)/JVS(84)
  XX(39) = X(39)/JVS(86)
  XX(40) = X(40)/JVS(87)
  XX(41) = X(41)/JVS(88)
  XX(42) = X(42)/JVS(90)
  XX(43) = X(43)/JVS(111)
  XX(44) = X(44)/JVS(140)
  XX(45) = X(45)/JVS(142)
  XX(46) = X(46)/JVS(144)
  XX(47) = X(47)/JVS(146)
  XX(48) = (X(48)-JVS(23)*XX(12)-JVS(112)*XX(43))/(JVS(148))
  XX(49) = X(49)/JVS(151)
  XX(50) = (X(50)-JVS(79)*XX(35)-JVS(113)*XX(43))/(JVS(154))
  XX(51) = (X(51)-JVS(21)*XX(11)-JVS(114)*XX(43))/(JVS(157))
  XX(52) = X(52)/JVS(160)
  XX(53) = X(53)/JVS(162)
  XX(54) = X(54)/JVS(164)
  XX(55) = X(55)/JVS(166)
  XX(56) = X(56)/JVS(168)
  XX(57) = X(57)/JVS(170)
  XX(58) = X(58)/JVS(172)
  XX(59) = X(59)/JVS(174)
  XX(60) = (X(60)-JVS(44)*XX(21)-JVS(47)*XX(22))/(JVS(177))
  XX(61) = (X(61)-JVS(50)*XX(23)-JVS(53)*XX(24))/(JVS(182))
  XX(62) = (X(62)-JVS(56)*XX(25)-JVS(59)*XX(26))/(JVS(187))
  XX(63) = X(63)/JVS(191)
  XX(64) = X(64)/JVS(195)
  XX(65) = (X(65)-JVS(17)*XX(9)-JVS(115)*XX(43))/(JVS(197))
  XX(66) = (X(66)-JVS(192)*XX(63))/(JVS(200))
  XX(67) = (X(67)-JVS(91)*XX(42))/(JVS(203))
  XX(68) = X(68)/JVS(206)
  XX(69) = X(69)/JVS(209)
  XX(70) = X(70)/JVS(213)
  XX(71) = X(71)/JVS(216)
  XX(72) = X(72)/JVS(219)
  XX(73) = X(73)/JVS(222)
  XX(74) = (X(74)-JVS(35)*XX(18))/(JVS(225))
  XX(75) = X(75)/JVS(228)
  XX(76) = X(76)/JVS(232)
  XX(77) = X(77)/JVS(236)
  XX(78) = X(78)/JVS(240)
  XX(79) = X(79)/JVS(244)
  XX(80) = X(80)/JVS(247)
  XX(81) = X(81)/JVS(250)
  XX(82) = X(82)/JVS(253)
  XX(83) = X(83)/JVS(256)
  XX(84) = X(84)/JVS(259)
  XX(85) = X(85)/JVS(263)
  XX(86) = X(86)/JVS(267)
  XX(87) = X(87)/JVS(270)
  XX(88) = X(88)/JVS(274)
  XX(89) = X(89)/JVS(280)
  XX(90) = X(90)/JVS(284)
  XX(91) = X(91)/JVS(288)
  XX(92) = X(92)/JVS(292)
  XX(93) = X(93)/JVS(301)
  XX(94) = X(94)/JVS(306)
  XX(95) = (X(95)-JVS(92)*XX(42))/(JVS(316))
  XX(96) = X(96)/JVS(323)
  XX(97) = X(97)/JVS(327)
  XX(98) = (X(98)-JVS(68)*XX(30)-JVS(116)*XX(43))/(JVS(331))
  XX(99) = X(99)/JVS(338)
  XX(100) = X(100)/JVS(344)
  XX(101) = (X(101)-JVS(77)*XX(34)-JVS(117)*XX(43)-JVS(155)*XX(50))/(JVS(349))
  XX(102) = X(102)/JVS(353)
  XX(103) = (X(103)-JVS(93)*XX(42))/(JVS(357))
  XX(104) = X(104)/JVS(362)
  XX(105) = X(105)/JVS(366)
  XX(106) = X(106)/JVS(371)
  XX(107) = (X(107)-JVS(94)*XX(42))/(JVS(377))
  XX(108) = (X(108)-JVS(11)*XX(6)-JVS(118)*XX(43)-JVS(241)*XX(78))/(JVS(383))
  XX(109) = X(109)/JVS(388)
  XX(110) = X(110)/JVS(394)
  XX(111) = (X(111)-JVS(75)*XX(33)-JVS(119)*XX(43))/(JVS(400))
  XX(112) = (X(112)-JVS(95)*XX(42))/(JVS(407))
  XX(113) = (X(113)-JVS(96)*XX(42)-JVS(158)*XX(51))/(JVS(413))
  XX(114) = X(114)/JVS(419)
  XX(115) = (X(115)-JVS(97)*XX(42)-JVS(339)*XX(99))/(JVS(435))
  XX(116) = (X(116)-JVS(340)*XX(99))/(JVS(442))
  XX(117) = (X(117)-JVS(19)*XX(10)-JVS(120)*XX(43))/(JVS(447))
  XX(118) = (X(118)-JVS(307)*XX(94))/(JVS(453))
  XX(119) = (X(119)-JVS(308)*XX(94))/(JVS(460))
  XX(120) = (X(120)-JVS(70)*XX(31)-JVS(83)*XX(37)-JVS(121)*XX(43))/(JVS(468))
  XX(121) = (X(121)-JVS(66)*XX(29)-JVS(122)*XX(43)-JVS(293)*XX(92)-JVS(332)*XX(98))/(JVS(476))
  XX(122) = X(122)/JVS(482)
  XX(123) = (X(123)-JVS(309)*XX(94))/(JVS(493))
  XX(124) = X(124)/JVS(501)
  XX(125) = (X(125)-JVS(389)*XX(109))/(JVS(511))
  XX(126) = (X(126)-JVS(310)*XX(94))/(JVS(517))
  XX(127) = (X(127)-JVS(311)*XX(94))/(JVS(525))
  XX(128) = (X(128)-JVS(38)*XX(19)-JVS(41)*XX(20)-JVS(454)*XX(118)-JVS(526)*XX(127))/(JVS(534))
  XX(129) = (X(129)-JVS(73)*XX(32)-JVS(123)*XX(43)-JVS(317)*XX(95))/(JVS(538))
  XX(130) = (X(130)-JVS(461)*XX(119)-JVS(527)*XX(127))/(JVS(546))
  XX(131) = (X(131)-JVS(390)*XX(109))/(JVS(551))
  XX(132) = (X(132)-JVS(312)*XX(94))/(JVS(557))
  XX(133) = (X(133)-JVS(62)*XX(27)-JVS(124)*XX(43)-JVS(294)*XX(92)-JVS(333)*XX(98))/(JVS(566))
  XX(134) = (X(134)-JVS(229)*XX(75)-JVS(420)*XX(114))/(JVS(578))
  XX(135) = (X(135)-JVS(85)*XX(38)-JVS(125)*XX(43))/(JVS(589))
  XX(136) = X(136)/JVS(605)
  XX(137) = (X(137)-JVS(98)*XX(42)-JVS(421)*XX(114))/(JVS(618))
  XX(138) = (X(138)-JVS(483)*XX(122))/(JVS(626))
  XX(139) = (X(139)-JVS(81)*XX(36)-JVS(126)*XX(43)-JVS(414)*XX(113))/(JVS(635))
  XX(140) = (X(140)-JVS(3)*XX(2)-JVS(127)*XX(43))/(JVS(645))
  XX(141) = (X(141)-JVS(324)*XX(96)-JVS(422)*XX(114))/(JVS(657))
  XX(142) = (X(142)-JVS(313)*XX(94)-JVS(358)*XX(103))/(JVS(667))
  XX(143) = (X(143)-JVS(7)*XX(4)-JVS(128)*XX(43))/(JVS(684))
  XX(144) = (X(144)-JVS(302)*XX(93)-JVS(448)*XX(117))/(JVS(698))
  XX(145) = (X(145)-JVS(99)*XX(42)-JVS(395)*XX(110)-JVS(590)*XX(135))/(JVS(705))
  XX(146) = (X(146)-JVS(328)*XX(97)-JVS(423)*XX(114)-JVS(591)*XX(135))/(JVS(714))
  XX(147) = (X(147)-JVS(354)*XX(102)-JVS(424)*XX(114)-JVS(469)*XX(120))/(JVS(725))
  XX(148) = (X(148)-JVS(260)*XX(84))/(JVS(736))
  XX(149) = (X(149)-JVS(345)*XX(100)-JVS(425)*XX(114))/(JVS(749))
  XX(150) = (X(150)-JVS(149)*XX(48)-JVS(303)*XX(93)-JVS(737)*XX(148))/(JVS(760))
  XX(151) = (X(151)-JVS(100)*XX(42)-JVS(606)*XX(136))/(JVS(780))
  XX(152) = (X(152)-JVS(367)*XX(105)-JVS(372)*XX(106)-JVS(378)*XX(107)-JVS(426)*XX(114)-JVS(436)*XX(115)-JVS(607)&
              &*XX(136)-JVS(781)*XX(151))/(JVS(799))
  XX(153) = (X(153)-JVS(237)*XX(77)-JVS(427)*XX(114)-JVS(646)*XX(140))/(JVS(808))
  XX(154) = (X(154)-JVS(295)*XX(92)-JVS(502)*XX(124))/(JVS(817))
  XX(155) = (X(155)-JVS(428)*XX(114))/(JVS(828))
  XX(156) = (X(156)-JVS(296)*XX(92)-JVS(503)*XX(124))/(JVS(845))
  XX(157) = (X(157)-JVS(346)*XX(100))/(JVS(863))
  XX(158) = (X(158)-JVS(512)*XX(125)-JVS(552)*XX(131)-JVS(608)*XX(136)-JVS(699)*XX(144)-JVS(761)*XX(150))/(JVS(874))
  XX(159) = (X(159)-JVS(9)*XX(5)-JVS(129)*XX(43))/(JVS(895))
  XX(160) = (X(160)-JVS(101)*XX(42)-JVS(289)*XX(91)-JVS(449)*XX(117)-JVS(619)*XX(137)-JVS(896)*XX(159))/(JVS(931))
  XX(161) = (X(161)-JVS(359)*XX(103)-JVS(429)*XX(114)-JVS(636)*XX(139))/(JVS(944))
  XX(162) = (X(162)-JVS(31)*XX(16)-JVS(102)*XX(42)-JVS(130)*XX(43))/(JVS(976))
  XX(163) = (X(163)-JVS(391)*XX(109)-JVS(539)*XX(129)-JVS(592)*XX(135)-JVS(782)*XX(151)-JVS(875)*XX(158)-JVS(897)&
              &*XX(159)-JVS(932)*XX(160)-JVS(977)*XX(162))/(JVS(1003))
  XX(164) = (X(164)-JVS(103)*XX(42)-JVS(350)*XX(101)-JVS(401)*XX(111)-JVS(408)*XX(112)-JVS(430)*XX(114)-JVS(443)*XX(116)&
              &-JVS(470)*XX(120)-JVS(540)*XX(129)-JVS(593)*XX(135)-JVS(609)*XX(136)-JVS(637)*XX(139)-JVS(783)*XX(151)&
              &-JVS(898)*XX(159)-JVS(978)*XX(162))/(JVS(1013))
  XX(165) = (X(165)-JVS(494)*XX(123)-JVS(518)*XX(126)-JVS(685)*XX(143)-JVS(784)*XX(151)-JVS(800)*XX(152)-JVS(899)&
              &*XX(159)-JVS(979)*XX(162))/(JVS(1026))
  XX(166) = (X(166)-JVS(900)*XX(159))/(JVS(1056))
  XX(167) = (X(167)-JVS(396)*XX(110)-JVS(471)*XX(120)-JVS(638)*XX(139)-JVS(785)*XX(151)-JVS(829)*XX(155)-JVS(901)&
              &*XX(159)-JVS(980)*XX(162)-JVS(1057)*XX(166))/(JVS(1079))
  XX(168) = (X(168)-JVS(318)*XX(95)-JVS(397)*XX(110)-JVS(594)*XX(135)-JVS(639)*XX(139)-JVS(786)*XX(151)-JVS(830)*XX(155)&
              &-JVS(902)*XX(159)-JVS(981)*XX(162)-JVS(1058)*XX(166)-JVS(1080)*XX(167))/(JVS(1095))
  XX(169) = (X(169)-JVS(341)*XX(99)-JVS(431)*XX(114)-JVS(579)*XX(134)-JVS(647)*XX(140)-JVS(738)*XX(148)-JVS(809)*XX(153)&
              &-JVS(864)*XX(157)-JVS(982)*XX(162)-JVS(1059)*XX(166))/(JVS(1114))
  XX(170) = (X(170)-JVS(25)*XX(13)-JVS(131)*XX(43)-JVS(580)*XX(134)-JVS(648)*XX(140)-JVS(739)*XX(148)-JVS(810)*XX(153)&
              &-JVS(831)*XX(155)-JVS(865)*XX(157)-JVS(983)*XX(162)-JVS(1060)*XX(166)-JVS(1115)*XX(169))/(JVS(1137))
  XX(171) = (X(171)-JVS(740)*XX(148)-JVS(762)*XX(150)-JVS(903)*XX(159))/(JVS(1179))
  XX(172) = (X(172)-JVS(233)*XX(76)-JVS(275)*XX(88)-JVS(297)*XX(92)-JVS(334)*XX(98)-JVS(477)*XX(121)-JVS(504)*XX(124)&
              &-JVS(567)*XX(133)-JVS(846)*XX(156)-JVS(904)*XX(159))/(JVS(1202))
  XX(173) = (X(173)-JVS(27)*XX(14)-JVS(132)*XX(43)-JVS(528)*XX(127)-JVS(547)*XX(130)-JVS(558)*XX(132)-JVS(627)*XX(138)&
              &-JVS(658)*XX(141)-JVS(686)*XX(143)-JVS(715)*XX(146)-JVS(726)*XX(147)-JVS(763)*XX(150)-JVS(787)*XX(151)&
              &-JVS(905)*XX(159)-JVS(984)*XX(162)-JVS(1061)*XX(166)-JVS(1180)*XX(171))/(JVS(1221))
  XX(174) = (X(174)-JVS(741)*XX(148)-JVS(750)*XX(149)-JVS(764)*XX(150)-JVS(866)*XX(157)-JVS(906)*XX(159)-JVS(985)&
              &*XX(162)-JVS(1062)*XX(166)-JVS(1138)*XX(170)-JVS(1181)*XX(171))/(JVS(1253))
  XX(175) = (X(175)-JVS(29)*XX(15)-JVS(133)*XX(43)-JVS(519)*XX(126)-JVS(529)*XX(127)-JVS(687)*XX(143)-JVS(706)*XX(145)&
              &-JVS(832)*XX(155)-JVS(907)*XX(159)-JVS(933)*XX(160)-JVS(945)*XX(161)-JVS(986)*XX(162)-JVS(1063)*XX(166)&
              &-JVS(1081)*XX(167)-JVS(1096)*XX(168)-JVS(1116)*XX(169)-JVS(1139)*XX(170)-JVS(1182)*XX(171)-JVS(1254)*XX(174))&
              &/(JVS(1271))
  XX(176) = (X(176)-JVS(33)*XX(17)-JVS(134)*XX(43)-JVS(628)*XX(138)-JVS(908)*XX(159))/(JVS(1304))
  XX(177) = (X(177)-JVS(314)*XX(94)-JVS(455)*XX(118)-JVS(462)*XX(119)-JVS(495)*XX(123)-JVS(520)*XX(126)-JVS(530)*XX(127)&
              &-JVS(559)*XX(132)-JVS(668)*XX(142)-JVS(688)*XX(143)-JVS(788)*XX(151)-JVS(909)*XX(159)-JVS(946)*XX(161)&
              &-JVS(987)*XX(162)-JVS(1064)*XX(166)-JVS(1183)*XX(171)-JVS(1222)*XX(173)-JVS(1255)*XX(174)-JVS(1272)*XX(175)&
              &-JVS(1305)*XX(176))/(JVS(1331))
  XX(178) = (X(178)-JVS(104)*XX(42)-JVS(198)*XX(65)-JVS(210)*XX(69)-JVS(373)*XX(106)-JVS(379)*XX(107)-JVS(437)*XX(115)&
              &-JVS(444)*XX(116)-JVS(581)*XX(134)-JVS(595)*XX(135)-JVS(610)*XX(136)-JVS(620)*XX(137)-JVS(649)*XX(140)&
              &-JVS(659)*XX(141)-JVS(707)*XX(145)-JVS(716)*XX(146)-JVS(727)*XX(147)-JVS(742)*XX(148)-JVS(751)*XX(149)&
              &-JVS(765)*XX(150)-JVS(789)*XX(151)-JVS(801)*XX(152)-JVS(811)*XX(153)-JVS(833)*XX(155)-JVS(867)*XX(157)&
              &-JVS(876)*XX(158)-JVS(910)*XX(159)-JVS(934)*XX(160)-JVS(947)*XX(161)-JVS(988)*XX(162)-JVS(1004)*XX(163)&
              &-JVS(1014)*XX(164)-JVS(1027)*XX(165)-JVS(1065)*XX(166)-JVS(1082)*XX(167)-JVS(1097)*XX(168)-JVS(1117)*XX(169)&
              &-JVS(1140)*XX(170)-JVS(1184)*XX(171)-JVS(1223)*XX(173)-JVS(1256)*XX(174)-JVS(1273)*XX(175)-JVS(1306)*XX(176)&
              &-JVS(1332)*XX(177))/(JVS(1384))
  XX(179) = (X(179)-JVS(5)*XX(3)-JVS(135)*XX(43)-JVS(484)*XX(122)-JVS(911)*XX(159))/(JVS(1455))
  XX(180) = (X(180)-JVS(105)*XX(42)-JVS(456)*XX(118)-JVS(463)*XX(119)-JVS(496)*XX(123)-JVS(521)*XX(126)-JVS(560)*XX(132)&
              &-JVS(669)*XX(142)-JVS(689)*XX(143)-JVS(790)*XX(151)-JVS(912)*XX(159)-JVS(948)*XX(161)-JVS(989)*XX(162)&
              &-JVS(1066)*XX(166)-JVS(1185)*XX(171)-JVS(1224)*XX(173)-JVS(1257)*XX(174)-JVS(1274)*XX(175)-JVS(1307)*XX(176)&
              &-JVS(1333)*XX(177)-JVS(1385)*XX(178)-JVS(1456)*XX(179))/(JVS(1516))
  XX(181) = (X(181)-JVS(106)*XX(42)-JVS(220)*XX(72)-JVS(276)*XX(88)-JVS(281)*XX(89)-JVS(432)*XX(114)-JVS(505)*XX(124)&
              &-JVS(513)*XX(125)-JVS(541)*XX(129)-JVS(553)*XX(131)-JVS(582)*XX(134)-JVS(596)*XX(135)-JVS(611)*XX(136)&
              &-JVS(621)*XX(137)-JVS(650)*XX(140)-JVS(660)*XX(141)-JVS(708)*XX(145)-JVS(717)*XX(146)-JVS(728)*XX(147)&
              &-JVS(743)*XX(148)-JVS(752)*XX(149)-JVS(766)*XX(150)-JVS(791)*XX(151)-JVS(802)*XX(152)-JVS(812)*XX(153)&
              &-JVS(834)*XX(155)-JVS(847)*XX(156)-JVS(868)*XX(157)-JVS(877)*XX(158)-JVS(913)*XX(159)-JVS(935)*XX(160)&
              &-JVS(949)*XX(161)-JVS(990)*XX(162)-JVS(1005)*XX(163)-JVS(1015)*XX(164)-JVS(1028)*XX(165)-JVS(1067)*XX(166)&
              &-JVS(1083)*XX(167)-JVS(1098)*XX(168)-JVS(1118)*XX(169)-JVS(1141)*XX(170)-JVS(1186)*XX(171)-JVS(1225)*XX(173)&
              &-JVS(1258)*XX(174)-JVS(1275)*XX(175)-JVS(1308)*XX(176)-JVS(1334)*XX(177)-JVS(1386)*XX(178)-JVS(1457)*XX(179)&
              &-JVS(1517)*XX(180))/(JVS(1576))
  XX(182) = (X(182)-JVS(15)*XX(8)-JVS(107)*XX(42)-JVS(136)*XX(43)-JVS(319)*XX(95)-JVS(380)*XX(107)-JVS(402)*XX(111)&
              &-JVS(438)*XX(115)-JVS(450)*XX(117)-JVS(457)*XX(118)-JVS(464)*XX(119)-JVS(472)*XX(120)-JVS(485)*XX(122)&
              &-JVS(497)*XX(123)-JVS(522)*XX(126)-JVS(531)*XX(127)-JVS(535)*XX(128)-JVS(542)*XX(129)-JVS(548)*XX(130)&
              &-JVS(561)*XX(132)-JVS(597)*XX(135)-JVS(612)*XX(136)-JVS(629)*XX(138)-JVS(640)*XX(139)-JVS(661)*XX(141)&
              &-JVS(670)*XX(142)-JVS(690)*XX(143)-JVS(718)*XX(146)-JVS(729)*XX(147)-JVS(792)*XX(151)-JVS(878)*XX(158)&
              &-JVS(914)*XX(159)-JVS(936)*XX(160)-JVS(950)*XX(161)-JVS(991)*XX(162)-JVS(1006)*XX(163)-JVS(1016)*XX(164)&
              &-JVS(1029)*XX(165)-JVS(1068)*XX(166)-JVS(1084)*XX(167)-JVS(1099)*XX(168)-JVS(1119)*XX(169)-JVS(1142)*XX(170)&
              &-JVS(1187)*XX(171)-JVS(1203)*XX(172)-JVS(1226)*XX(173)-JVS(1259)*XX(174)-JVS(1276)*XX(175)-JVS(1309)*XX(176)&
              &-JVS(1335)*XX(177)-JVS(1387)*XX(178)-JVS(1458)*XX(179)-JVS(1518)*XX(180)-JVS(1577)*XX(181))/(JVS(1614))
  XX(183) = (X(183)-JVS(48)*XX(22)-JVS(54)*XX(24)-JVS(60)*XX(26)-JVS(108)*XX(42)-JVS(178)*XX(60)-JVS(183)*XX(61)&
              &-JVS(188)*XX(62)-JVS(201)*XX(66)-JVS(264)*XX(85)-JVS(320)*XX(95)-JVS(374)*XX(106)-JVS(381)*XX(107)-JVS(403)&
              &*XX(111)-JVS(409)*XX(112)-JVS(415)*XX(113)-JVS(439)*XX(115)-JVS(445)*XX(116)-JVS(458)*XX(118)-JVS(465)&
              &*XX(119)-JVS(473)*XX(120)-JVS(498)*XX(123)-JVS(514)*XX(125)-JVS(523)*XX(126)-JVS(532)*XX(127)-JVS(543)&
              &*XX(129)-JVS(554)*XX(131)-JVS(562)*XX(132)-JVS(583)*XX(134)-JVS(598)*XX(135)-JVS(613)*XX(136)-JVS(622)&
              &*XX(137)-JVS(641)*XX(139)-JVS(651)*XX(140)-JVS(662)*XX(141)-JVS(671)*XX(142)-JVS(691)*XX(143)-JVS(700)&
              &*XX(144)-JVS(709)*XX(145)-JVS(719)*XX(146)-JVS(730)*XX(147)-JVS(744)*XX(148)-JVS(753)*XX(149)-JVS(767)&
              &*XX(150)-JVS(793)*XX(151)-JVS(803)*XX(152)-JVS(813)*XX(153)-JVS(835)*XX(155)-JVS(869)*XX(157)-JVS(879)&
              &*XX(158)-JVS(915)*XX(159)-JVS(937)*XX(160)-JVS(951)*XX(161)-JVS(992)*XX(162)-JVS(1007)*XX(163)-JVS(1017)&
              &*XX(164)-JVS(1030)*XX(165)-JVS(1069)*XX(166)-JVS(1085)*XX(167)-JVS(1100)*XX(168)-JVS(1120)*XX(169)-JVS(1143)&
              &*XX(170)-JVS(1188)*XX(171)-JVS(1204)*XX(172)-JVS(1227)*XX(173)-JVS(1260)*XX(174)-JVS(1277)*XX(175)-JVS(1310)&
              &*XX(176)-JVS(1336)*XX(177)-JVS(1388)*XX(178)-JVS(1459)*XX(179)-JVS(1519)*XX(180)-JVS(1578)*XX(181)-JVS(1615)&
              &*XX(182))/(JVS(1683))
  XX(184) = (X(184)-JVS(271)*XX(87)-JVS(277)*XX(88)-JVS(486)*XX(122)-JVS(506)*XX(124)-JVS(630)*XX(138)-JVS(692)*XX(143)&
              &-JVS(818)*XX(154)-JVS(848)*XX(156)-JVS(916)*XX(159)-JVS(1460)*XX(179)-JVS(1520)*XX(180)-JVS(1579)*XX(181)&
              &-JVS(1616)*XX(182)-JVS(1684)*XX(183))/(JVS(1733))
  XX(185) = (X(185)-JVS(223)*XX(73)-JVS(234)*XX(76)-JVS(278)*XX(88)-JVS(298)*XX(92)-JVS(507)*XX(124)-JVS(819)*XX(154)&
              &-JVS(849)*XX(156)-JVS(917)*XX(159)-JVS(1205)*XX(172)-JVS(1461)*XX(179)-JVS(1521)*XX(180)-JVS(1580)*XX(181)&
              &-JVS(1617)*XX(182)-JVS(1685)*XX(183)-JVS(1734)*XX(184))/(JVS(1767))
  XX(186) = (X(186)-JVS(42)*XX(20)-JVS(207)*XX(68)-JVS(226)*XX(74)-JVS(363)*XX(104)-JVS(384)*XX(108)-JVS(536)*XX(128)&
              &-JVS(549)*XX(130)-JVS(555)*XX(131)-JVS(614)*XX(136)-JVS(631)*XX(138)-JVS(663)*XX(141)-JVS(672)*XX(142)&
              &-JVS(693)*XX(143)-JVS(701)*XX(144)-JVS(710)*XX(145)-JVS(720)*XX(146)-JVS(731)*XX(147)-JVS(745)*XX(148)&
              &-JVS(754)*XX(149)-JVS(768)*XX(150)-JVS(794)*XX(151)-JVS(836)*XX(155)-JVS(850)*XX(156)-JVS(870)*XX(157)&
              &-JVS(880)*XX(158)-JVS(918)*XX(159)-JVS(938)*XX(160)-JVS(952)*XX(161)-JVS(993)*XX(162)-JVS(1008)*XX(163)&
              &-JVS(1018)*XX(164)-JVS(1031)*XX(165)-JVS(1070)*XX(166)-JVS(1086)*XX(167)-JVS(1101)*XX(168)-JVS(1121)*XX(169)&
              &-JVS(1144)*XX(170)-JVS(1189)*XX(171)-JVS(1206)*XX(172)-JVS(1228)*XX(173)-JVS(1261)*XX(174)-JVS(1278)*XX(175)&
              &-JVS(1311)*XX(176)-JVS(1337)*XX(177)-JVS(1389)*XX(178)-JVS(1462)*XX(179)-JVS(1522)*XX(180)-JVS(1581)*XX(181)&
              &-JVS(1618)*XX(182)-JVS(1686)*XX(183)-JVS(1735)*XX(184)-JVS(1768)*XX(185))/(JVS(1833))
  XX(187) = (X(187)-JVS(64)*XX(28)-JVS(137)*XX(43)-JVS(299)*XX(92)-JVS(335)*XX(98)-JVS(487)*XX(122)-JVS(568)*XX(133)&
              &-JVS(820)*XX(154)-JVS(851)*XX(156)-JVS(919)*XX(159)-JVS(1207)*XX(172)-JVS(1619)*XX(182)-JVS(1687)*XX(183)&
              &-JVS(1736)*XX(184)-JVS(1769)*XX(185)-JVS(1834)*XX(186))/(JVS(1865))
  XX(188) = (X(188)-JVS(204)*XX(67)-JVS(488)*XX(122)-JVS(569)*XX(133)-JVS(694)*XX(143)-JVS(821)*XX(154)-JVS(852)*XX(156)&
              &-JVS(920)*XX(159)-JVS(1208)*XX(172)-JVS(1338)*XX(177)-JVS(1390)*XX(178)-JVS(1463)*XX(179)-JVS(1523)*XX(180)&
              &-JVS(1582)*XX(181)-JVS(1620)*XX(182)-JVS(1688)*XX(183)-JVS(1737)*XX(184)-JVS(1770)*XX(185)-JVS(1835)*XX(186)&
              &-JVS(1866)*XX(187))/(JVS(1915))
  XX(189) = (X(189)-JVS(36)*XX(18)-JVS(39)*XX(19)-JVS(71)*XX(31)-JVS(109)*XX(42)-JVS(141)*XX(44)-JVS(143)*XX(45)&
              &-JVS(145)*XX(46)-JVS(147)*XX(47)-JVS(156)*XX(50)-JVS(179)*XX(60)-JVS(184)*XX(61)-JVS(189)*XX(62)-JVS(205)&
              &*XX(67)-JVS(208)*XX(68)-JVS(211)*XX(69)-JVS(214)*XX(70)-JVS(217)*XX(71)-JVS(224)*XX(73)-JVS(227)*XX(74)&
              &-JVS(230)*XX(75)-JVS(235)*XX(76)-JVS(238)*XX(77)-JVS(242)*XX(78)-JVS(245)*XX(79)-JVS(248)*XX(80)-JVS(251)&
              &*XX(81)-JVS(254)*XX(82)-JVS(257)*XX(83)-JVS(261)*XX(84)-JVS(265)*XX(85)-JVS(268)*XX(86)-JVS(272)*XX(87)&
              &-JVS(279)*XX(88)-JVS(282)*XX(89)-JVS(285)*XX(90)-JVS(290)*XX(91)-JVS(304)*XX(93)-JVS(315)*XX(94)-JVS(321)&
              &*XX(95)-JVS(325)*XX(96)-JVS(329)*XX(97)-JVS(336)*XX(98)-JVS(342)*XX(99)-JVS(347)*XX(100)-JVS(351)*XX(101)&
              &-JVS(355)*XX(102)-JVS(360)*XX(103)-JVS(364)*XX(104)-JVS(368)*XX(105)-JVS(375)*XX(106)-JVS(382)*XX(107)&
              &-JVS(392)*XX(109)-JVS(398)*XX(110)-JVS(404)*XX(111)-JVS(410)*XX(112)-JVS(416)*XX(113)-JVS(433)*XX(114)&
              &-JVS(440)*XX(115)-JVS(451)*XX(117)-JVS(474)*XX(120)-JVS(489)*XX(122)-JVS(508)*XX(124)-JVS(515)*XX(125)&
              &-JVS(537)*XX(128)-JVS(544)*XX(129)-JVS(550)*XX(130)-JVS(570)*XX(133)-JVS(584)*XX(134)-JVS(599)*XX(135)&
              &-JVS(615)*XX(136)-JVS(623)*XX(137)-JVS(632)*XX(138)-JVS(642)*XX(139)-JVS(652)*XX(140)-JVS(664)*XX(141)&
              &-JVS(673)*XX(142)-JVS(695)*XX(143)-JVS(702)*XX(144)-JVS(721)*XX(146)-JVS(732)*XX(147)-JVS(746)*XX(148)&
              &-JVS(755)*XX(149)-JVS(769)*XX(150)-JVS(795)*XX(151)-JVS(804)*XX(152)-JVS(814)*XX(153)-JVS(822)*XX(154)&
              &-JVS(837)*XX(155)-JVS(853)*XX(156)-JVS(871)*XX(157)-JVS(881)*XX(158)-JVS(921)*XX(159)-JVS(939)*XX(160)&
              &-JVS(953)*XX(161)-JVS(994)*XX(162)-JVS(1009)*XX(163)-JVS(1019)*XX(164)-JVS(1032)*XX(165)-JVS(1071)*XX(166)&
              &-JVS(1087)*XX(167)-JVS(1102)*XX(168)-JVS(1122)*XX(169)-JVS(1145)*XX(170)-JVS(1190)*XX(171)-JVS(1209)*XX(172)&
              &-JVS(1229)*XX(173)-JVS(1262)*XX(174)-JVS(1279)*XX(175)-JVS(1312)*XX(176)-JVS(1339)*XX(177)-JVS(1391)*XX(178)&
              &-JVS(1464)*XX(179)-JVS(1524)*XX(180)-JVS(1583)*XX(181)-JVS(1621)*XX(182)-JVS(1689)*XX(183)-JVS(1738)*XX(184)&
              &-JVS(1771)*XX(185)-JVS(1836)*XX(186)-JVS(1867)*XX(187)-JVS(1916)*XX(188))/(JVS(2047))
  XX(190) = (X(190)-JVS(152)*XX(49)-JVS(258)*XX(83)-JVS(337)*XX(98)-JVS(365)*XX(104)-JVS(478)*XX(121)-JVS(571)*XX(133)&
              &-JVS(585)*XX(134)-JVS(653)*XX(140)-JVS(747)*XX(148)-JVS(815)*XX(153)-JVS(872)*XX(157)-JVS(922)*XX(159)&
              &-JVS(995)*XX(162)-JVS(1072)*XX(166)-JVS(1146)*XX(170)-JVS(1191)*XX(171)-JVS(1210)*XX(172)-JVS(1263)*XX(174)&
              &-JVS(1313)*XX(176)-JVS(1392)*XX(178)-JVS(1465)*XX(179)-JVS(1525)*XX(180)-JVS(1584)*XX(181)-JVS(1622)*XX(182)&
              &-JVS(1690)*XX(183)-JVS(1739)*XX(184)-JVS(1772)*XX(185)-JVS(1837)*XX(186)-JVS(1868)*XX(187)-JVS(1917)*XX(188)&
              &-JVS(2048)*XX(189))/(JVS(2090))
  XX(191) = (X(191)-JVS(89)*XX(41)-JVS(138)*XX(43)-JVS(243)*XX(78)-JVS(300)*XX(92)-JVS(385)*XX(108)-JVS(479)*XX(121)&
              &-JVS(490)*XX(122)-JVS(509)*XX(124)-JVS(572)*XX(133)-JVS(823)*XX(154)-JVS(854)*XX(156)-JVS(923)*XX(159)&
              &-JVS(1211)*XX(172)-JVS(1623)*XX(182)-JVS(1691)*XX(183)-JVS(1740)*XX(184)-JVS(1773)*XX(185)-JVS(1838)*XX(186)&
              &-JVS(1869)*XX(187)-JVS(1918)*XX(188)-JVS(2049)*XX(189)-JVS(2091)*XX(190))/(JVS(2123))
  XX(192) = (X(192)-JVS(386)*XX(108)-JVS(480)*XX(121)-JVS(573)*XX(133)-JVS(824)*XX(154)-JVS(855)*XX(156)-JVS(924)&
              &*XX(159)-JVS(1212)*XX(172)-JVS(1692)*XX(183)-JVS(1741)*XX(184)-JVS(1774)*XX(185)-JVS(1839)*XX(186)-JVS(1870)&
              &*XX(187)-JVS(1919)*XX(188)-JVS(2050)*XX(189)-JVS(2092)*XX(190)-JVS(2124)*XX(191))/(JVS(2183))
  XX(193) = (X(193)-JVS(45)*XX(21)-JVS(51)*XX(23)-JVS(57)*XX(25)-JVS(110)*XX(42)-JVS(180)*XX(60)-JVS(185)*XX(61)&
              &-JVS(190)*XX(62)-JVS(212)*XX(69)-JVS(231)*XX(75)-JVS(239)*XX(77)-JVS(262)*XX(84)-JVS(283)*XX(89)-JVS(286)&
              &*XX(90)-JVS(291)*XX(91)-JVS(305)*XX(93)-JVS(322)*XX(95)-JVS(326)*XX(96)-JVS(330)*XX(97)-JVS(343)*XX(99)&
              &-JVS(348)*XX(100)-JVS(352)*XX(101)-JVS(356)*XX(102)-JVS(361)*XX(103)-JVS(369)*XX(105)-JVS(376)*XX(106)&
              &-JVS(393)*XX(109)-JVS(399)*XX(110)-JVS(411)*XX(112)-JVS(417)*XX(113)-JVS(441)*XX(115)-JVS(446)*XX(116)&
              &-JVS(475)*XX(120)-JVS(491)*XX(122)-JVS(516)*XX(125)-JVS(556)*XX(131)-JVS(574)*XX(133)-JVS(586)*XX(134)&
              &-JVS(600)*XX(135)-JVS(616)*XX(136)-JVS(624)*XX(137)-JVS(643)*XX(139)-JVS(654)*XX(140)-JVS(665)*XX(141)&
              &-JVS(696)*XX(143)-JVS(703)*XX(144)-JVS(711)*XX(145)-JVS(722)*XX(146)-JVS(733)*XX(147)-JVS(748)*XX(148)&
              &-JVS(756)*XX(149)-JVS(770)*XX(150)-JVS(796)*XX(151)-JVS(805)*XX(152)-JVS(816)*XX(153)-JVS(838)*XX(155)&
              &-JVS(856)*XX(156)-JVS(873)*XX(157)-JVS(925)*XX(159)-JVS(940)*XX(160)-JVS(954)*XX(161)-JVS(996)*XX(162)&
              &-JVS(1010)*XX(163)-JVS(1020)*XX(164)-JVS(1033)*XX(165)-JVS(1073)*XX(166)-JVS(1088)*XX(167)-JVS(1103)*XX(168)&
              &-JVS(1123)*XX(169)-JVS(1147)*XX(170)-JVS(1192)*XX(171)-JVS(1213)*XX(172)-JVS(1230)*XX(173)-JVS(1264)*XX(174)&
              &-JVS(1280)*XX(175)-JVS(1314)*XX(176)-JVS(1340)*XX(177)-JVS(1393)*XX(178)-JVS(1466)*XX(179)-JVS(1526)*XX(180)&
              &-JVS(1585)*XX(181)-JVS(1624)*XX(182)-JVS(1693)*XX(183)-JVS(1742)*XX(184)-JVS(1775)*XX(185)-JVS(1840)*XX(186)&
              &-JVS(1871)*XX(187)-JVS(1920)*XX(188)-JVS(2051)*XX(189)-JVS(2093)*XX(190)-JVS(2125)*XX(191)-JVS(2184)*XX(192))&
              &/(JVS(2276))
  XX(194) = (X(194)-JVS(161)*XX(52)-JVS(163)*XX(53)-JVS(165)*XX(54)-JVS(167)*XX(55)-JVS(169)*XX(56)-JVS(171)*XX(57)&
              &-JVS(173)*XX(58)-JVS(175)*XX(59)-JVS(193)*XX(63)-JVS(196)*XX(64)-JVS(215)*XX(70)-JVS(218)*XX(71)-JVS(246)&
              &*XX(79)-JVS(249)*XX(80)-JVS(252)*XX(81)-JVS(255)*XX(82)-JVS(269)*XX(86)-JVS(492)*XX(122)-JVS(510)*XX(124)&
              &-JVS(633)*XX(138)-JVS(857)*XX(156)-JVS(1214)*XX(172)-JVS(1467)*XX(179)-JVS(1527)*XX(180)-JVS(1586)*XX(181)&
              &-JVS(1625)*XX(182)-JVS(1694)*XX(183)-JVS(1743)*XX(184)-JVS(1776)*XX(185)-JVS(1841)*XX(186)-JVS(1872)*XX(187)&
              &-JVS(1921)*XX(188)-JVS(2052)*XX(189)-JVS(2094)*XX(190)-JVS(2126)*XX(191)-JVS(2185)*XX(192)-JVS(2277)*XX(193))&
              &/(JVS(2320))
  XX(195) = (X(195)-JVS(13)*XX(7)-JVS(139)*XX(43)-JVS(150)*XX(48)-JVS(153)*XX(49)-JVS(159)*XX(51)-JVS(194)*XX(63)&
              &-JVS(199)*XX(65)-JVS(202)*XX(66)-JVS(221)*XX(72)-JVS(266)*XX(85)-JVS(287)*XX(90)-JVS(387)*XX(108)-JVS(418)&
              &*XX(113)-JVS(452)*XX(117)-JVS(459)*XX(118)-JVS(466)*XX(119)-JVS(481)*XX(121)-JVS(499)*XX(123)-JVS(524)&
              &*XX(126)-JVS(533)*XX(127)-JVS(563)*XX(132)-JVS(575)*XX(133)-JVS(674)*XX(142)-JVS(697)*XX(143)-JVS(704)&
              &*XX(144)-JVS(771)*XX(150)-JVS(797)*XX(151)-JVS(825)*XX(154)-JVS(858)*XX(156)-JVS(926)*XX(159)-JVS(941)&
              &*XX(160)-JVS(955)*XX(161)-JVS(997)*XX(162)-JVS(1074)*XX(166)-JVS(1148)*XX(170)-JVS(1193)*XX(171)-JVS(1215)&
              &*XX(172)-JVS(1231)*XX(173)-JVS(1265)*XX(174)-JVS(1281)*XX(175)-JVS(1315)*XX(176)-JVS(1341)*XX(177)-JVS(1394)&
              &*XX(178)-JVS(1468)*XX(179)-JVS(1528)*XX(180)-JVS(1587)*XX(181)-JVS(1626)*XX(182)-JVS(1695)*XX(183)-JVS(1744)&
              &*XX(184)-JVS(1777)*XX(185)-JVS(1842)*XX(186)-JVS(1873)*XX(187)-JVS(1922)*XX(188)-JVS(2053)*XX(189)-JVS(2095)&
              &*XX(190)-JVS(2127)*XX(191)-JVS(2186)*XX(192)-JVS(2278)*XX(193)-JVS(2321)*XX(194))/(JVS(2409))
  XX(195) = XX(195)
  XX(194) = XX(194)-JVS(2408)*XX(195)
  XX(193) = XX(193)-JVS(2319)*XX(194)-JVS(2407)*XX(195)
  XX(192) = XX(192)-JVS(2275)*XX(193)-JVS(2318)*XX(194)-JVS(2406)*XX(195)
  XX(191) = XX(191)-JVS(2182)*XX(192)-JVS(2274)*XX(193)-JVS(2317)*XX(194)-JVS(2405)*XX(195)
  XX(190) = XX(190)-JVS(2122)*XX(191)-JVS(2181)*XX(192)-JVS(2273)*XX(193)-JVS(2316)*XX(194)-JVS(2404)*XX(195)
  XX(189) = XX(189)-JVS(2089)*XX(190)-JVS(2121)*XX(191)-JVS(2180)*XX(192)-JVS(2272)*XX(193)-JVS(2315)*XX(194)-JVS(2403)&
              &*XX(195)
  XX(188) = XX(188)-JVS(2046)*XX(189)-JVS(2088)*XX(190)-JVS(2120)*XX(191)-JVS(2179)*XX(192)-JVS(2271)*XX(193)-JVS(2314)&
              &*XX(194)-JVS(2402)*XX(195)
  XX(187) = XX(187)-JVS(1914)*XX(188)-JVS(2045)*XX(189)-JVS(2087)*XX(190)-JVS(2119)*XX(191)-JVS(2178)*XX(192)-JVS(2270)&
              &*XX(193)-JVS(2313)*XX(194)-JVS(2401)*XX(195)
  XX(186) = XX(186)-JVS(1864)*XX(187)-JVS(1913)*XX(188)-JVS(2044)*XX(189)-JVS(2086)*XX(190)-JVS(2118)*XX(191)-JVS(2177)&
              &*XX(192)-JVS(2269)*XX(193)-JVS(2312)*XX(194)-JVS(2400)*XX(195)
  XX(185) = XX(185)-JVS(1832)*XX(186)-JVS(1863)*XX(187)-JVS(1912)*XX(188)-JVS(2043)*XX(189)-JVS(2085)*XX(190)-JVS(2117)&
              &*XX(191)-JVS(2176)*XX(192)-JVS(2268)*XX(193)-JVS(2311)*XX(194)-JVS(2399)*XX(195)
  XX(184) = XX(184)-JVS(1766)*XX(185)-JVS(1831)*XX(186)-JVS(1862)*XX(187)-JVS(1911)*XX(188)-JVS(2042)*XX(189)-JVS(2084)&
              &*XX(190)-JVS(2116)*XX(191)-JVS(2175)*XX(192)-JVS(2267)*XX(193)-JVS(2310)*XX(194)-JVS(2398)*XX(195)
  XX(183) = XX(183)-JVS(1732)*XX(184)-JVS(1765)*XX(185)-JVS(1830)*XX(186)-JVS(1861)*XX(187)-JVS(1910)*XX(188)-JVS(2041)&
              &*XX(189)-JVS(2083)*XX(190)-JVS(2115)*XX(191)-JVS(2174)*XX(192)-JVS(2266)*XX(193)-JVS(2309)*XX(194)-JVS(2397)&
              &*XX(195)
  XX(182) = XX(182)-JVS(1682)*XX(183)-JVS(1731)*XX(184)-JVS(1764)*XX(185)-JVS(1829)*XX(186)-JVS(1860)*XX(187)-JVS(1909)&
              &*XX(188)-JVS(2040)*XX(189)-JVS(2082)*XX(190)-JVS(2114)*XX(191)-JVS(2173)*XX(192)-JVS(2265)*XX(193)-JVS(2308)&
              &*XX(194)-JVS(2396)*XX(195)
  XX(181) = XX(181)-JVS(1613)*XX(182)-JVS(1681)*XX(183)-JVS(1730)*XX(184)-JVS(1763)*XX(185)-JVS(1828)*XX(186)-JVS(1859)&
              &*XX(187)-JVS(1908)*XX(188)-JVS(2039)*XX(189)-JVS(2081)*XX(190)-JVS(2113)*XX(191)-JVS(2172)*XX(192)-JVS(2264)&
              &*XX(193)-JVS(2307)*XX(194)-JVS(2395)*XX(195)
  XX(180) = XX(180)-JVS(1575)*XX(181)-JVS(1612)*XX(182)-JVS(1680)*XX(183)-JVS(1729)*XX(184)-JVS(1827)*XX(186)-JVS(1858)&
              &*XX(187)-JVS(1907)*XX(188)-JVS(2038)*XX(189)-JVS(2080)*XX(190)-JVS(2112)*XX(191)-JVS(2171)*XX(192)-JVS(2263)&
              &*XX(193)-JVS(2306)*XX(194)-JVS(2394)*XX(195)
  XX(179) = XX(179)-JVS(1515)*XX(180)-JVS(1611)*XX(182)-JVS(1728)*XX(184)-JVS(1826)*XX(186)-JVS(1857)*XX(187)-JVS(1906)&
              &*XX(188)-JVS(2037)*XX(189)-JVS(2079)*XX(190)-JVS(2111)*XX(191)-JVS(2170)*XX(192)-JVS(2262)*XX(193)-JVS(2393)&
              &*XX(195)
  XX(178) = XX(178)-JVS(1454)*XX(179)-JVS(1514)*XX(180)-JVS(1574)*XX(181)-JVS(1610)*XX(182)-JVS(1679)*XX(183)-JVS(1727)&
              &*XX(184)-JVS(1825)*XX(186)-JVS(1856)*XX(187)-JVS(1905)*XX(188)-JVS(2036)*XX(189)-JVS(2078)*XX(190)-JVS(2110)&
              &*XX(191)-JVS(2169)*XX(192)-JVS(2261)*XX(193)-JVS(2305)*XX(194)-JVS(2392)*XX(195)
  XX(177) = XX(177)-JVS(1383)*XX(178)-JVS(1453)*XX(179)-JVS(1513)*XX(180)-JVS(1573)*XX(181)-JVS(1609)*XX(182)-JVS(1678)&
              &*XX(183)-JVS(1726)*XX(184)-JVS(1824)*XX(186)-JVS(1855)*XX(187)-JVS(1904)*XX(188)-JVS(2035)*XX(189)-JVS(2077)&
              &*XX(190)-JVS(2109)*XX(191)-JVS(2168)*XX(192)-JVS(2260)*XX(193)-JVS(2304)*XX(194)-JVS(2391)*XX(195)
  XX(176) = XX(176)-JVS(1382)*XX(178)-JVS(1452)*XX(179)-JVS(1512)*XX(180)-JVS(1572)*XX(181)-JVS(1608)*XX(182)-JVS(1725)&
              &*XX(184)-JVS(1823)*XX(186)-JVS(1854)*XX(187)-JVS(1903)*XX(188)-JVS(2034)*XX(189)-JVS(2076)*XX(190)-JVS(2108)&
              &*XX(191)-JVS(2167)*XX(192)-JVS(2259)*XX(193)-JVS(2303)*XX(194)-JVS(2390)*XX(195)
  XX(175) = XX(175)-JVS(1303)*XX(176)-JVS(1330)*XX(177)-JVS(1381)*XX(178)-JVS(1451)*XX(179)-JVS(1511)*XX(180)-JVS(1571)&
              &*XX(181)-JVS(1607)*XX(182)-JVS(1677)*XX(183)-JVS(1724)*XX(184)-JVS(1822)*XX(186)-JVS(1853)*XX(187)-JVS(1902)&
              &*XX(188)-JVS(2033)*XX(189)-JVS(2075)*XX(190)-JVS(2107)*XX(191)-JVS(2166)*XX(192)-JVS(2258)*XX(193)-JVS(2389)&
              &*XX(195)
  XX(174) = XX(174)-JVS(1302)*XX(176)-JVS(1380)*XX(178)-JVS(1450)*XX(179)-JVS(1510)*XX(180)-JVS(1570)*XX(181)-JVS(1606)&
              &*XX(182)-JVS(1676)*XX(183)-JVS(1821)*XX(186)-JVS(1852)*XX(187)-JVS(2032)*XX(189)-JVS(2074)*XX(190)-JVS(2165)&
              &*XX(192)-JVS(2257)*XX(193)-JVS(2388)*XX(195)
  XX(173) = XX(173)-JVS(1252)*XX(174)-JVS(1301)*XX(176)-JVS(1329)*XX(177)-JVS(1379)*XX(178)-JVS(1449)*XX(179)-JVS(1509)&
              &*XX(180)-JVS(1569)*XX(181)-JVS(1605)*XX(182)-JVS(1675)*XX(183)-JVS(1723)*XX(184)-JVS(1820)*XX(186)-JVS(1901)&
              &*XX(188)-JVS(2031)*XX(189)-JVS(2106)*XX(191)-JVS(2164)*XX(192)-JVS(2256)*XX(193)-JVS(2302)*XX(194)-JVS(2387)&
              &*XX(195)
  XX(172) = XX(172)-JVS(1674)*XX(183)-JVS(1722)*XX(184)-JVS(1762)*XX(185)-JVS(1819)*XX(186)-JVS(1851)*XX(187)-JVS(1900)&
              &*XX(188)-JVS(2030)*XX(189)-JVS(2073)*XX(190)-JVS(2105)*XX(191)-JVS(2163)*XX(192)-JVS(2255)*XX(193)-JVS(2301)&
              &*XX(194)-JVS(2386)*XX(195)
  XX(171) = XX(171)-JVS(1300)*XX(176)-JVS(1378)*XX(178)-JVS(1448)*XX(179)-JVS(1508)*XX(180)-JVS(1568)*XX(181)-JVS(1604)&
              &*XX(182)-JVS(1673)*XX(183)-JVS(1818)*XX(186)-JVS(2029)*XX(189)-JVS(2162)*XX(192)-JVS(2254)*XX(193)-JVS(2385)&
              &*XX(195)
  XX(170) = XX(170)-JVS(1178)*XX(171)-JVS(1251)*XX(174)-JVS(1299)*XX(176)-JVS(1377)*XX(178)-JVS(1447)*XX(179)-JVS(1507)&
              &*XX(180)-JVS(1567)*XX(181)-JVS(1672)*XX(183)-JVS(1817)*XX(186)-JVS(1850)*XX(187)-JVS(2028)*XX(189)-JVS(2072)&
              &*XX(190)-JVS(2161)*XX(192)-JVS(2253)*XX(193)-JVS(2384)*XX(195)
  XX(169) = XX(169)-JVS(1136)*XX(170)-JVS(1177)*XX(171)-JVS(1250)*XX(174)-JVS(1298)*XX(176)-JVS(1376)*XX(178)-JVS(1446)&
              &*XX(179)-JVS(1506)*XX(180)-JVS(1566)*XX(181)-JVS(1671)*XX(183)-JVS(1816)*XX(186)-JVS(1849)*XX(187)-JVS(2027)&
              &*XX(189)-JVS(2071)*XX(190)-JVS(2160)*XX(192)-JVS(2252)*XX(193)-JVS(2383)*XX(195)
  XX(168) = XX(168)-JVS(1113)*XX(169)-JVS(1135)*XX(170)-JVS(1176)*XX(171)-JVS(1249)*XX(174)-JVS(1297)*XX(176)-JVS(1375)&
              &*XX(178)-JVS(1445)*XX(179)-JVS(1505)*XX(180)-JVS(1565)*XX(181)-JVS(1670)*XX(183)-JVS(1815)*XX(186)-JVS(2026)&
              &*XX(189)-JVS(2159)*XX(192)-JVS(2251)*XX(193)-JVS(2382)*XX(195)
  XX(167) = XX(167)-JVS(1094)*XX(168)-JVS(1112)*XX(169)-JVS(1134)*XX(170)-JVS(1175)*XX(171)-JVS(1296)*XX(176)-JVS(1374)&
              &*XX(178)-JVS(1444)*XX(179)-JVS(1504)*XX(180)-JVS(1564)*XX(181)-JVS(1669)*XX(183)-JVS(1814)*XX(186)-JVS(2025)&
              &*XX(189)-JVS(2158)*XX(192)-JVS(2250)*XX(193)-JVS(2381)*XX(195)
  XX(166) = XX(166)-JVS(1373)*XX(178)-JVS(1503)*XX(180)-JVS(1813)*XX(186)-JVS(2024)*XX(189)-JVS(2157)*XX(192)-JVS(2249)&
              &*XX(193)-JVS(2380)*XX(195)
  XX(165) = XX(165)-JVS(1055)*XX(166)-JVS(1111)*XX(169)-JVS(1174)*XX(171)-JVS(1248)*XX(174)-JVS(1328)*XX(177)-JVS(1372)&
              &*XX(178)-JVS(1443)*XX(179)-JVS(1502)*XX(180)-JVS(1563)*XX(181)-JVS(1603)*XX(182)-JVS(1668)*XX(183)-JVS(1721)&
              &*XX(184)-JVS(1812)*XX(186)-JVS(1899)*XX(188)-JVS(2023)*XX(189)-JVS(2104)*XX(191)-JVS(2156)*XX(192)-JVS(2248)&
              &*XX(193)-JVS(2379)*XX(195)
  XX(164) = XX(164)-JVS(1025)*XX(165)-JVS(1054)*XX(166)-JVS(1078)*XX(167)-JVS(1093)*XX(168)-JVS(1110)*XX(169)-JVS(1173)&
              &*XX(171)-JVS(1220)*XX(173)-JVS(1247)*XX(174)-JVS(1270)*XX(175)-JVS(1371)*XX(178)-JVS(1442)*XX(179)-JVS(1501)&
              &*XX(180)-JVS(1562)*XX(181)-JVS(1602)*XX(182)-JVS(1667)*XX(183)-JVS(1811)*XX(186)-JVS(2022)*XX(189)-JVS(2155)&
              &*XX(192)-JVS(2247)*XX(193)-JVS(2378)*XX(195)
  XX(163) = XX(163)-JVS(1024)*XX(165)-JVS(1053)*XX(166)-JVS(1092)*XX(168)-JVS(1133)*XX(170)-JVS(1172)*XX(171)-JVS(1219)&
              &*XX(173)-JVS(1246)*XX(174)-JVS(1269)*XX(175)-JVS(1295)*XX(176)-JVS(1370)*XX(178)-JVS(1441)*XX(179)-JVS(1500)&
              &*XX(180)-JVS(1561)*XX(181)-JVS(1601)*XX(182)-JVS(1666)*XX(183)-JVS(1810)*XX(186)-JVS(2021)*XX(189)-JVS(2154)&
              &*XX(192)-JVS(2246)*XX(193)-JVS(2377)*XX(195)
  XX(162) = XX(162)-JVS(1052)*XX(166)-JVS(1369)*XX(178)-JVS(1440)*XX(179)-JVS(1499)*XX(180)-JVS(1560)*XX(181)-JVS(2020)&
              &*XX(189)-JVS(2245)*XX(193)
  XX(161) = XX(161)-JVS(975)*XX(162)-JVS(1051)*XX(166)-JVS(1171)*XX(171)-JVS(1245)*XX(174)-JVS(1368)*XX(178)-JVS(1439)&
              &*XX(179)-JVS(1498)*XX(180)-JVS(1559)*XX(181)-JVS(1665)*XX(183)-JVS(1809)*XX(186)-JVS(2019)*XX(189)-JVS(2244)&
              &*XX(193)-JVS(2376)*XX(195)
  XX(160) = XX(160)-JVS(974)*XX(162)-JVS(1170)*XX(171)-JVS(1244)*XX(174)-JVS(1367)*XX(178)-JVS(1438)*XX(179)-JVS(1497)&
              &*XX(180)-JVS(1558)*XX(181)-JVS(1600)*XX(182)-JVS(1664)*XX(183)-JVS(1808)*XX(186)-JVS(2018)*XX(189)-JVS(2153)&
              &*XX(192)-JVS(2243)*XX(193)-JVS(2375)*XX(195)
  XX(159) = XX(159)-JVS(1807)*XX(186)-JVS(2017)*XX(189)-JVS(2152)*XX(192)-JVS(2374)*XX(195)
  XX(158) = XX(158)-JVS(894)*XX(159)-JVS(930)*XX(160)-JVS(973)*XX(162)-JVS(1002)*XX(163)-JVS(1132)*XX(170)-JVS(1169)&
              &*XX(171)-JVS(1243)*XX(174)-JVS(1294)*XX(176)-JVS(1366)*XX(178)-JVS(1437)*XX(179)-JVS(1496)*XX(180)-JVS(1557)&
              &*XX(181)-JVS(1599)*XX(182)-JVS(1663)*XX(183)-JVS(1806)*XX(186)-JVS(2016)*XX(189)-JVS(2242)*XX(193)-JVS(2373)&
              &*XX(195)
  XX(157) = XX(157)-JVS(972)*XX(162)-JVS(1050)*XX(166)-JVS(1131)*XX(170)-JVS(1242)*XX(174)-JVS(1365)*XX(178)-JVS(1436)&
              &*XX(179)-JVS(1556)*XX(181)-JVS(1662)*XX(183)-JVS(2015)*XX(189)-JVS(2151)*XX(192)-JVS(2241)*XX(193)-JVS(2372)&
              &*XX(195)
  XX(156) = XX(156)-JVS(1720)*XX(184)-JVS(1761)*XX(185)-JVS(1848)*XX(187)-JVS(1898)*XX(188)-JVS(2014)*XX(189)-JVS(2070)&
              &*XX(190)-JVS(2103)*XX(191)-JVS(2150)*XX(192)-JVS(2300)*XX(194)
  XX(155) = XX(155)-JVS(1109)*XX(169)-JVS(1130)*XX(170)-JVS(1168)*XX(171)-JVS(1293)*XX(176)-JVS(1364)*XX(178)-JVS(1435)&
              &*XX(179)-JVS(1555)*XX(181)-JVS(1661)*XX(183)-JVS(2013)*XX(189)-JVS(2240)*XX(193)-JVS(2371)*XX(195)
  XX(154) = XX(154)-JVS(844)*XX(156)-JVS(893)*XX(159)-JVS(1719)*XX(184)-JVS(1760)*XX(185)-JVS(1805)*XX(186)-JVS(1847)&
              &*XX(187)-JVS(1897)*XX(188)-JVS(2012)*XX(189)-JVS(2069)*XX(190)-JVS(2102)*XX(191)-JVS(2149)*XX(192)-JVS(2299)&
              &*XX(194)-JVS(2370)*XX(195)
  XX(153) = XX(153)-JVS(862)*XX(157)-JVS(1167)*XX(171)-JVS(1363)*XX(178)-JVS(1434)*XX(179)-JVS(1495)*XX(180)-JVS(1554)&
              &*XX(181)-JVS(1660)*XX(183)-JVS(1846)*XX(187)-JVS(2011)*XX(189)-JVS(2068)*XX(190)-JVS(2148)*XX(192)-JVS(2239)&
              &*XX(193)-JVS(2369)*XX(195)
  XX(152) = XX(152)-JVS(892)*XX(159)-JVS(971)*XX(162)-JVS(1049)*XX(166)-JVS(1108)*XX(169)-JVS(1166)*XX(171)-JVS(1241)&
              &*XX(174)-JVS(1362)*XX(178)-JVS(1433)*XX(179)-JVS(1494)*XX(180)-JVS(1553)*XX(181)-JVS(1598)*XX(182)-JVS(1659)&
              &*XX(183)-JVS(1804)*XX(186)-JVS(2010)*XX(189)-JVS(2238)*XX(193)-JVS(2368)*XX(195)
  XX(151) = XX(151)-JVS(891)*XX(159)-JVS(1432)*XX(179)-JVS(1493)*XX(180)-JVS(1803)*XX(186)-JVS(2009)*XX(189)-JVS(2237)&
              &*XX(193)
  XX(150) = XX(150)-JVS(1292)*XX(176)-JVS(1361)*XX(178)-JVS(1431)*XX(179)-JVS(1552)*XX(181)-JVS(1597)*XX(182)-JVS(1658)&
              &*XX(183)-JVS(2008)*XX(189)-JVS(2236)*XX(193)-JVS(2367)*XX(195)
  XX(149) = XX(149)-JVS(861)*XX(157)-JVS(1048)*XX(166)-JVS(1129)*XX(170)-JVS(1165)*XX(171)-JVS(1240)*XX(174)-JVS(1291)&
              &*XX(176)-JVS(1360)*XX(178)-JVS(1430)*XX(179)-JVS(1551)*XX(181)-JVS(1657)*XX(183)-JVS(2007)*XX(189)-JVS(2147)&
              &*XX(192)-JVS(2235)*XX(193)-JVS(2366)*XX(195)
  XX(148) = XX(148)-JVS(1290)*XX(176)-JVS(1359)*XX(178)-JVS(1429)*XX(179)-JVS(1550)*XX(181)-JVS(1656)*XX(183)-JVS(2006)&
              &*XX(189)-JVS(2234)*XX(193)-JVS(2365)*XX(195)
  XX(147) = XX(147)-JVS(779)*XX(151)-JVS(1047)*XX(166)-JVS(1164)*XX(171)-JVS(1239)*XX(174)-JVS(1358)*XX(178)-JVS(1428)&
              &*XX(179)-JVS(1549)*XX(181)-JVS(1655)*XX(183)-JVS(1802)*XX(186)-JVS(2005)*XX(189)-JVS(2233)*XX(193)-JVS(2364)&
              &*XX(195)
  XX(146) = XX(146)-JVS(1046)*XX(166)-JVS(1163)*XX(171)-JVS(1289)*XX(176)-JVS(1357)*XX(178)-JVS(1427)*XX(179)-JVS(1492)&
              &*XX(180)-JVS(1548)*XX(181)-JVS(1654)*XX(183)-JVS(2004)*XX(189)-JVS(2232)*XX(193)-JVS(2363)*XX(195)
  XX(145) = XX(145)-JVS(827)*XX(155)-JVS(1045)*XX(166)-JVS(1077)*XX(167)-JVS(1091)*XX(168)-JVS(1128)*XX(170)-JVS(1162)&
              &*XX(171)-JVS(1356)*XX(178)-JVS(1426)*XX(179)-JVS(1491)*XX(180)-JVS(1547)*XX(181)-JVS(1653)*XX(183)-JVS(2003)&
              &*XX(189)-JVS(2231)*XX(193)-JVS(2362)*XX(195)
  XX(144) = XX(144)-JVS(759)*XX(150)-JVS(929)*XX(160)-JVS(970)*XX(162)-JVS(1127)*XX(170)-JVS(1288)*XX(176)-JVS(1355)&
              &*XX(178)-JVS(1425)*XX(179)-JVS(1490)*XX(180)-JVS(1596)*XX(182)-JVS(1652)*XX(183)-JVS(1801)*XX(186)-JVS(2002)&
              &*XX(189)-JVS(2230)*XX(193)-JVS(2361)*XX(195)
  XX(143) = XX(143)-JVS(1718)*XX(184)-JVS(1896)*XX(188)-JVS(2001)*XX(189)-JVS(2101)*XX(191)-JVS(2146)*XX(192)-JVS(2229)&
              &*XX(193)
  XX(142) = XX(142)-JVS(683)*XX(143)-JVS(943)*XX(161)-JVS(969)*XX(162)-JVS(1268)*XX(175)-JVS(1327)*XX(177)-JVS(1424)&
              &*XX(179)-JVS(1489)*XX(180)-JVS(1651)*XX(183)-JVS(1800)*XX(186)-JVS(2000)*XX(189)-JVS(2228)*XX(193)-JVS(2360)&
              &*XX(195)
  XX(141) = XX(141)-JVS(968)*XX(162)-JVS(1161)*XX(171)-JVS(1287)*XX(176)-JVS(1354)*XX(178)-JVS(1423)*XX(179)-JVS(1546)&
              &*XX(181)-JVS(1650)*XX(183)-JVS(1999)*XX(189)-JVS(2227)*XX(193)-JVS(2359)*XX(195)
  XX(140) = XX(140)-JVS(860)*XX(157)-JVS(1353)*XX(178)-JVS(1488)*XX(180)-JVS(1545)*XX(181)-JVS(1845)*XX(187)-JVS(1998)&
              &*XX(189)-JVS(2067)*XX(190)-JVS(2145)*XX(192)
  XX(139) = XX(139)-JVS(967)*XX(162)-JVS(1044)*XX(166)-JVS(1422)*XX(179)-JVS(1487)*XX(180)-JVS(1649)*XX(183)-JVS(1799)&
              &*XX(186)-JVS(1997)*XX(189)-JVS(2226)*XX(193)-JVS(2358)*XX(195)
  XX(138) = XX(138)-JVS(1421)*XX(179)-JVS(1544)*XX(181)-JVS(1595)*XX(182)-JVS(1717)*XX(184)-JVS(1895)*XX(188)-JVS(1996)&
              &*XX(189)-JVS(2100)*XX(191)-JVS(2144)*XX(192)-JVS(2225)*XX(193)-JVS(2298)*XX(194)
  XX(137) = XX(137)-JVS(890)*XX(159)-JVS(966)*XX(162)-JVS(1160)*XX(171)-JVS(1238)*XX(174)-JVS(1352)*XX(178)-JVS(1420)&
              &*XX(179)-JVS(1543)*XX(181)-JVS(1648)*XX(183)-JVS(1995)*XX(189)-JVS(2224)*XX(193)-JVS(2357)*XX(195)
  XX(136) = XX(136)-JVS(889)*XX(159)-JVS(1419)*XX(179)-JVS(1486)*XX(180)-JVS(1798)*XX(186)-JVS(1994)*XX(189)-JVS(2223)&
              &*XX(193)
  XX(135) = XX(135)-JVS(1043)*XX(166)-JVS(1418)*XX(179)-JVS(1485)*XX(180)-JVS(1542)*XX(181)-JVS(1993)*XX(189)-JVS(2356)&
              &*XX(195)
  XX(134) = XX(134)-JVS(1159)*XX(171)-JVS(1351)*XX(178)-JVS(1417)*XX(179)-JVS(1541)*XX(181)-JVS(1647)*XX(183)-JVS(1992)&
              &*XX(189)-JVS(2222)*XX(193)-JVS(2355)*XX(195)
  XX(133) = XX(133)-JVS(1201)*XX(172)-JVS(1716)*XX(184)-JVS(1894)*XX(188)-JVS(1991)*XX(189)-JVS(2066)*XX(190)-JVS(2099)&
              &*XX(191)-JVS(2143)*XX(192)
  XX(132) = XX(132)-JVS(682)*XX(143)-JVS(1042)*XX(166)-JVS(1326)*XX(177)-JVS(1350)*XX(178)-JVS(1484)*XX(180)-JVS(1540)&
              &*XX(181)-JVS(1646)*XX(183)-JVS(1797)*XX(186)-JVS(1990)*XX(189)-JVS(2354)*XX(195)
  XX(131) = XX(131)-JVS(604)*XX(136)-JVS(1001)*XX(163)-JVS(1126)*XX(170)-JVS(1158)*XX(171)-JVS(1237)*XX(174)-JVS(1416)&
              &*XX(179)-JVS(1539)*XX(181)-JVS(1645)*XX(183)-JVS(1796)*XX(186)-JVS(1989)*XX(189)-JVS(2221)*XX(193)-JVS(2353)&
              &*XX(195)
  XX(130) = XX(130)-JVS(625)*XX(138)-JVS(656)*XX(141)-JVS(681)*XX(143)-JVS(713)*XX(146)-JVS(1286)*XX(176)-JVS(1325)&
              &*XX(177)-JVS(1415)*XX(179)-JVS(1483)*XX(180)-JVS(1538)*XX(181)-JVS(1594)*XX(182)-JVS(1644)*XX(183)-JVS(1795)&
              &*XX(186)-JVS(1988)*XX(189)-JVS(2220)*XX(193)-JVS(2352)*XX(195)
  XX(129) = XX(129)-JVS(588)*XX(135)-JVS(778)*XX(151)-JVS(965)*XX(162)-JVS(1023)*XX(165)-JVS(1090)*XX(168)-JVS(1414)&
              &*XX(179)-JVS(1593)*XX(182)-JVS(1987)*XX(189)-JVS(2219)*XX(193)-JVS(2351)*XX(195)
  XX(128) = XX(128)-JVS(545)*XX(130)-JVS(666)*XX(142)-JVS(680)*XX(143)-JVS(724)*XX(147)-JVS(1000)*XX(163)-JVS(1012)&
              &*XX(164)-JVS(1218)*XX(173)-JVS(1267)*XX(175)-JVS(1324)*XX(177)-JVS(1349)*XX(178)-JVS(1413)*XX(179)-JVS(1482)&
              &*XX(180)-JVS(1537)*XX(181)-JVS(1592)*XX(182)-JVS(1643)*XX(183)-JVS(1794)*XX(186)-JVS(1986)*XX(189)-JVS(2218)&
              &*XX(193)-JVS(2350)*XX(195)
  XX(127) = XX(127)-JVS(679)*XX(143)-JVS(1323)*XX(177)-JVS(1412)*XX(179)-JVS(1642)*XX(183)-JVS(1793)*XX(186)-JVS(1985)&
              &*XX(189)-JVS(2349)*XX(195)
  XX(126) = XX(126)-JVS(678)*XX(143)-JVS(1041)*XX(166)-JVS(1322)*XX(177)-JVS(1481)*XX(180)-JVS(1641)*XX(183)-JVS(1792)&
              &*XX(186)-JVS(1984)*XX(189)-JVS(2348)*XX(195)
  XX(125) = XX(125)-JVS(603)*XX(136)-JVS(999)*XX(163)-JVS(1125)*XX(170)-JVS(1157)*XX(171)-JVS(1236)*XX(174)-JVS(1411)&
              &*XX(179)-JVS(1536)*XX(181)-JVS(1640)*XX(183)-JVS(1983)*XX(189)-JVS(2217)*XX(193)-JVS(2347)*XX(195)
  XX(124) = XX(124)-JVS(843)*XX(156)-JVS(1715)*XX(184)-JVS(1759)*XX(185)-JVS(1893)*XX(188)-JVS(1982)*XX(189)-JVS(2297)&
              &*XX(194)
  XX(123) = XX(123)-JVS(677)*XX(143)-JVS(777)*XX(151)-JVS(964)*XX(162)-JVS(1321)*XX(177)-JVS(1480)*XX(180)-JVS(1639)&
              &*XX(183)-JVS(1791)*XX(186)-JVS(1981)*XX(189)-JVS(2346)*XX(195)
  XX(122) = XX(122)-JVS(1591)*XX(182)-JVS(1892)*XX(188)-JVS(1980)*XX(189)-JVS(2142)*XX(192)-JVS(2216)*XX(193)
  XX(121) = XX(121)-JVS(565)*XX(133)-JVS(888)*XX(159)-JVS(1200)*XX(172)-JVS(1714)*XX(184)-JVS(1790)*XX(186)-JVS(1979)&
              &*XX(189)-JVS(2065)*XX(190)-JVS(2098)*XX(191)-JVS(2141)*XX(192)-JVS(2345)*XX(195)
  XX(120) = XX(120)-JVS(776)*XX(151)-JVS(1040)*XX(166)-JVS(1348)*XX(178)-JVS(1410)*XX(179)-JVS(1789)*XX(186)-JVS(1978)&
              &*XX(189)-JVS(2344)*XX(195)
  XX(119) = XX(119)-JVS(676)*XX(143)-JVS(1285)*XX(176)-JVS(1320)*XX(177)-JVS(1479)*XX(180)-JVS(1638)*XX(183)-JVS(1788)&
              &*XX(186)-JVS(1977)*XX(189)-JVS(2343)*XX(195)
  XX(118) = XX(118)-JVS(675)*XX(143)-JVS(1217)*XX(173)-JVS(1319)*XX(177)-JVS(1478)*XX(180)-JVS(1637)*XX(183)-JVS(1787)&
              &*XX(186)-JVS(1976)*XX(189)-JVS(2342)*XX(195)
  XX(117) = XX(117)-JVS(928)*XX(160)-JVS(963)*XX(162)-JVS(1347)*XX(178)-JVS(1409)*XX(179)-JVS(1477)*XX(180)-JVS(1590)&
              &*XX(182)-JVS(1786)*XX(186)-JVS(1975)*XX(189)-JVS(2341)*XX(195)
  XX(116) = XX(116)-JVS(602)*XX(136)-JVS(775)*XX(151)-JVS(887)*XX(159)-JVS(962)*XX(162)-JVS(1039)*XX(166)-JVS(1107)&
              &*XX(169)-JVS(1156)*XX(171)-JVS(1636)*XX(183)-JVS(1974)*XX(189)-JVS(2215)*XX(193)-JVS(2340)*XX(195)
  XX(115) = XX(115)-JVS(886)*XX(159)-JVS(1106)*XX(169)-JVS(1155)*XX(171)-JVS(1635)*XX(183)-JVS(1973)*XX(189)-JVS(2214)&
              &*XX(193)-JVS(2339)*XX(195)
  XX(114) = XX(114)-JVS(1154)*XX(171)-JVS(1972)*XX(189)-JVS(2213)*XX(193)
  XX(113) = XX(113)-JVS(961)*XX(162)-JVS(1038)*XX(166)-JVS(1408)*XX(179)-JVS(1634)*XX(183)-JVS(1785)*XX(186)-JVS(1971)&
              &*XX(189)-JVS(2212)*XX(193)-JVS(2338)*XX(195)
  XX(112) = XX(112)-JVS(601)*XX(136)-JVS(774)*XX(151)-JVS(960)*XX(162)-JVS(1037)*XX(166)-JVS(1407)*XX(179)-JVS(1476)&
              &*XX(180)-JVS(1633)*XX(183)-JVS(1970)*XX(189)-JVS(2211)*XX(193)-JVS(2337)*XX(195)
  XX(111) = XX(111)-JVS(467)*XX(120)-JVS(634)*XX(139)-JVS(1022)*XX(165)-JVS(1076)*XX(167)-JVS(1406)*XX(179)-JVS(1589)&
              &*XX(182)-JVS(1969)*XX(189)-JVS(2210)*XX(193)-JVS(2336)*XX(195)
  XX(110) = XX(110)-JVS(826)*XX(155)-JVS(1075)*XX(167)-JVS(1089)*XX(168)-JVS(1153)*XX(171)-JVS(1968)*XX(189)-JVS(2209)&
              &*XX(193)-JVS(2335)*XX(195)
  XX(109) = XX(109)-JVS(998)*XX(163)-JVS(1124)*XX(170)-JVS(1152)*XX(171)-JVS(1235)*XX(174)-JVS(1967)*XX(189)-JVS(2208)&
              &*XX(193)-JVS(2334)*XX(195)
  XX(108) = XX(108)-JVS(842)*XX(156)-JVS(885)*XX(159)-JVS(1713)*XX(184)-JVS(1784)*XX(186)-JVS(1966)*XX(189)-JVS(2097)&
              &*XX(191)-JVS(2140)*XX(192)-JVS(2333)*XX(195)
  XX(107) = XX(107)-JVS(434)*XX(115)-JVS(1234)*XX(174)-JVS(1588)*XX(182)-JVS(1965)*XX(189)-JVS(2207)*XX(193)
  XX(106) = XX(106)-JVS(959)*XX(162)-JVS(1233)*XX(174)-JVS(1475)*XX(180)-JVS(1964)*XX(189)-JVS(2206)*XX(193)
  XX(105) = XX(105)-JVS(370)*XX(106)-JVS(773)*XX(151)-JVS(798)*XX(152)-JVS(958)*XX(162)-JVS(1036)*XX(166)-JVS(1474)&
              &*XX(180)-JVS(1963)*XX(189)-JVS(2205)*XX(193)
  XX(104) = XX(104)-JVS(735)*XX(148)-JVS(884)*XX(159)-JVS(1473)*XX(180)-JVS(1783)*XX(186)-JVS(1844)*XX(187)-JVS(1962)&
              &*XX(189)-JVS(2064)*XX(190)-JVS(2139)*XX(192)
  XX(103) = XX(103)-JVS(942)*XX(161)-JVS(957)*XX(162)-JVS(1405)*XX(179)-JVS(1472)*XX(180)-JVS(1961)*XX(189)-JVS(2204)&
              &*XX(193)
  XX(102) = XX(102)-JVS(723)*XX(147)-JVS(772)*XX(151)-JVS(1035)*XX(166)-JVS(1232)*XX(174)-JVS(1346)*XX(178)-JVS(1404)&
              &*XX(179)-JVS(1960)*XX(189)-JVS(2203)*XX(193)
  XX(101) = XX(101)-JVS(406)*XX(112)-JVS(1011)*XX(164)-JVS(1021)*XX(165)-JVS(1216)*XX(173)-JVS(1266)*XX(175)-JVS(1403)&
              &*XX(179)-JVS(1959)*XX(189)-JVS(2202)*XX(193)
  XX(100) = XX(100)-JVS(859)*XX(157)-JVS(1034)*XX(166)-JVS(1345)*XX(178)-JVS(1402)*XX(179)-JVS(1958)*XX(189)-JVS(2138)&
              &*XX(192)
  XX(99) = XX(99)-JVS(1105)*XX(169)-JVS(1151)*XX(171)-JVS(1957)*XX(189)-JVS(2201)*XX(193)
  XX(98) = XX(98)-JVS(564)*XX(133)-JVS(1956)*XX(189)-JVS(2063)*XX(190)
  XX(97) = XX(97)-JVS(587)*XX(135)-JVS(712)*XX(146)-JVS(1150)*XX(171)-JVS(1955)*XX(189)-JVS(2200)*XX(193)-JVS(2332)&
             &*XX(195)
  XX(96) = XX(96)-JVS(655)*XX(141)-JVS(956)*XX(162)-JVS(1284)*XX(176)-JVS(1401)*XX(179)-JVS(1954)*XX(189)-JVS(2199)&
             &*XX(193)
  XX(95) = XX(95)-JVS(1400)*XX(179)-JVS(1953)*XX(189)-JVS(2331)*XX(195)
  XX(94) = XX(94)-JVS(1318)*XX(177)-JVS(1952)*XX(189)
  XX(93) = XX(93)-JVS(758)*XX(150)-JVS(1283)*XX(176)-JVS(1951)*XX(189)-JVS(2198)*XX(193)
  XX(92) = XX(92)-JVS(1712)*XX(184)-JVS(2062)*XX(190)
  XX(91) = XX(91)-JVS(617)*XX(137)-JVS(927)*XX(160)-JVS(1344)*XX(178)-JVS(1399)*XX(179)-JVS(1950)*XX(189)
  XX(90) = XX(90)-JVS(1782)*XX(186)-JVS(1949)*XX(189)-JVS(2137)*XX(192)-JVS(2197)*XX(193)-JVS(2330)*XX(195)
  XX(89) = XX(89)-JVS(1398)*XX(179)-JVS(1535)*XX(181)-JVS(1948)*XX(189)-JVS(2136)*XX(192)-JVS(2196)*XX(193)
  XX(88) = XX(88)-JVS(500)*XX(124)-JVS(1711)*XX(184)-JVS(1758)*XX(185)
  XX(87) = XX(87)-JVS(1471)*XX(180)-JVS(1534)*XX(181)-JVS(1710)*XX(184)-JVS(1947)*XX(189)-JVS(2096)*XX(191)-JVS(2135)&
             &*XX(192)-JVS(2195)*XX(193)
  XX(86) = XX(86)-JVS(1199)*XX(172)-JVS(1533)*XX(181)-JVS(1946)*XX(189)-JVS(2061)*XX(190)-JVS(2134)*XX(192)-JVS(2194)&
             &*XX(193)-JVS(2296)*XX(194)
  XX(85) = XX(85)-JVS(1632)*XX(183)-JVS(1945)*XX(189)-JVS(2133)*XX(192)-JVS(2329)*XX(195)
  XX(84) = XX(84)-JVS(734)*XX(148)-JVS(1282)*XX(176)-JVS(1944)*XX(189)-JVS(2193)*XX(193)
  XX(83) = XX(83)-JVS(577)*XX(134)-JVS(807)*XX(153)-JVS(1470)*XX(180)-JVS(1843)*XX(187)-JVS(1943)*XX(189)-JVS(2060)&
             &*XX(190)
  XX(82) = XX(82)-JVS(1709)*XX(184)-JVS(1757)*XX(185)-JVS(1891)*XX(188)-JVS(1942)*XX(189)-JVS(2132)*XX(192)-JVS(2295)&
             &*XX(194)
  XX(81) = XX(81)-JVS(1708)*XX(184)-JVS(1756)*XX(185)-JVS(1890)*XX(188)-JVS(1941)*XX(189)-JVS(2131)*XX(192)-JVS(2294)&
             &*XX(194)
  XX(80) = XX(80)-JVS(1707)*XX(184)-JVS(1755)*XX(185)-JVS(1889)*XX(188)-JVS(1940)*XX(189)-JVS(2130)*XX(192)-JVS(2293)&
             &*XX(194)
  XX(79) = XX(79)-JVS(1706)*XX(184)-JVS(1754)*XX(185)-JVS(1888)*XX(188)-JVS(1939)*XX(189)-JVS(2129)*XX(192)-JVS(2292)&
             &*XX(194)
  XX(78) = XX(78)-JVS(841)*XX(156)-JVS(1705)*XX(184)-JVS(1938)*XX(189)-JVS(2328)*XX(195)
  XX(77) = XX(77)-JVS(644)*XX(140)-JVS(806)*XX(153)-JVS(1937)*XX(189)-JVS(2192)*XX(193)
  XX(76) = XX(76)-JVS(840)*XX(156)-JVS(1753)*XX(185)-JVS(1887)*XX(188)-JVS(1936)*XX(189)
  XX(75) = XX(75)-JVS(576)*XX(134)-JVS(1149)*XX(171)-JVS(1935)*XX(189)-JVS(2191)*XX(193)
  XX(74) = XX(74)-JVS(883)*XX(159)-JVS(1317)*XX(177)-JVS(1397)*XX(179)-JVS(1532)*XX(181)-JVS(1781)*XX(186)-JVS(1934)&
             &*XX(189)
  XX(73) = XX(73)-JVS(273)*XX(88)-JVS(839)*XX(156)-JVS(1704)*XX(184)-JVS(1752)*XX(185)-JVS(1933)*XX(189)
  XX(72) = XX(72)-JVS(1396)*XX(179)-JVS(1531)*XX(181)-JVS(1780)*XX(186)-JVS(2190)*XX(193)-JVS(2327)*XX(195)
  XX(71) = XX(71)-JVS(1198)*XX(172)-JVS(1886)*XX(188)-JVS(1932)*XX(189)-JVS(2059)*XX(190)-JVS(2291)*XX(194)
  XX(70) = XX(70)-JVS(1197)*XX(172)-JVS(1885)*XX(188)-JVS(1931)*XX(189)-JVS(2058)*XX(190)-JVS(2290)*XX(194)
  XX(69) = XX(69)-JVS(1343)*XX(178)-JVS(1530)*XX(181)-JVS(1930)*XX(189)
  XX(68) = XX(68)-JVS(882)*XX(159)-JVS(1104)*XX(169)-JVS(1779)*XX(186)-JVS(1929)*XX(189)
  XX(67) = XX(67)-JVS(1316)*XX(177)-JVS(1469)*XX(180)-JVS(1884)*XX(188)-JVS(1928)*XX(189)
  XX(66) = XX(66)-JVS(1631)*XX(183)-JVS(1883)*XX(188)-JVS(2289)*XX(194)-JVS(2326)*XX(195)
  XX(65) = XX(65)-JVS(1342)*XX(178)-JVS(1529)*XX(181)-JVS(1778)*XX(186)-JVS(2325)*XX(195)
  XX(64) = XX(64)-JVS(1196)*XX(172)-JVS(1703)*XX(184)-JVS(1751)*XX(185)-JVS(1882)*XX(188)-JVS(2057)*XX(190)-JVS(2288)&
             &*XX(194)
  XX(63) = XX(63)-JVS(1630)*XX(183)-JVS(2287)*XX(194)
  XX(62) = XX(62)-JVS(1629)*XX(183)-JVS(2189)*XX(193)
  XX(61) = XX(61)-JVS(1628)*XX(183)-JVS(2188)*XX(193)
  XX(60) = XX(60)-JVS(1627)*XX(183)-JVS(2187)*XX(193)
  XX(59) = XX(59)-JVS(1195)*XX(172)-JVS(1881)*XX(188)-JVS(2056)*XX(190)-JVS(2286)*XX(194)
  XX(58) = XX(58)-JVS(1194)*XX(172)-JVS(1880)*XX(188)-JVS(2055)*XX(190)-JVS(2285)*XX(194)
  XX(57) = XX(57)-JVS(1702)*XX(184)-JVS(1750)*XX(185)-JVS(1879)*XX(188)-JVS(2284)*XX(194)
  XX(56) = XX(56)-JVS(1701)*XX(184)-JVS(1749)*XX(185)-JVS(1878)*XX(188)-JVS(2283)*XX(194)
  XX(55) = XX(55)-JVS(1700)*XX(184)-JVS(1748)*XX(185)-JVS(1877)*XX(188)-JVS(2282)*XX(194)
  XX(54) = XX(54)-JVS(1699)*XX(184)-JVS(1747)*XX(185)-JVS(1876)*XX(188)-JVS(2281)*XX(194)
  XX(53) = XX(53)-JVS(1698)*XX(184)-JVS(1746)*XX(185)-JVS(1875)*XX(188)-JVS(2280)*XX(194)
  XX(52) = XX(52)-JVS(1697)*XX(184)-JVS(1745)*XX(185)-JVS(1874)*XX(188)-JVS(2279)*XX(194)
  XX(51) = XX(51)-JVS(412)*XX(113)-JVS(2324)*XX(195)
  XX(50) = XX(50)-JVS(405)*XX(112)-JVS(1927)*XX(189)
  XX(49) = XX(49)-JVS(2054)*XX(190)-JVS(2323)*XX(195)
  XX(48) = XX(48)-JVS(757)*XX(150)-JVS(2322)*XX(195)
  XX(47) = XX(47)-JVS(1696)*XX(184)-JVS(1926)*XX(189)-JVS(2128)*XX(192)
  XX(46) = XX(46)-JVS(176)*XX(60)-JVS(1925)*XX(189)
  XX(45) = XX(45)-JVS(186)*XX(62)-JVS(1924)*XX(189)
  XX(44) = XX(44)-JVS(181)*XX(61)-JVS(1923)*XX(189)
  XX(43) = XX(43)
  XX(42) = XX(42)
  XX(41) = XX(41)
  XX(40) = XX(40)
  XX(39) = XX(39)
  XX(38) = XX(38)
  XX(37) = XX(37)
  XX(36) = XX(36)
  XX(35) = XX(35)
  XX(34) = XX(34)
  XX(33) = XX(33)
  XX(32) = XX(32)
  XX(31) = XX(31)
  XX(30) = XX(30)
  XX(29) = XX(29)
  XX(28) = XX(28)
  XX(27) = XX(27)
  XX(26) = XX(26)
  XX(25) = XX(25)
  XX(24) = XX(24)
  XX(23) = XX(23)
  XX(22) = XX(22)
  XX(21) = XX(21)
  XX(20) = XX(20)
  XX(19) = XX(19)
  XX(18) = XX(18)
  XX(17) = XX(17)
  XX(16) = XX(16)
  XX(15) = XX(15)
  XX(14) = XX(14)
  XX(13) = XX(13)
  XX(12) = XX(12)
  XX(11) = XX(11)
  XX(10) = XX(10)
  XX(9) = XX(9)
  XX(8) = XX(8)
  XX(7) = XX(7)
  XX(6) = XX(6)
  XX(5) = XX(5)
  XX(4) = XX(4)
  XX(3) = XX(3)
  XX(2) = XX(2)
  XX(1) = XX(1)-JVS(1395)*XX(179)
      
END SUBROUTINE KppSolveTR

! End of KppSolveTR function
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~


! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
! 
! BLAS_UTIL - BLAS-LIKE utility functions
!   Arguments :
! 
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

!--------------------------------------------------------------
!
! BLAS/LAPACK-like subroutines used by the integration algorithms
! It is recommended to replace them by calls to the optimized
!      BLAS/LAPACK library for your machine
!
!  (C) Adrian Sandu, Aug. 2004
!      Virginia Polytechnic Institute and State University
!--------------------------------------------------------------


!--------------------------------------------------------------
      SUBROUTINE WCOPY(N,X,incX,Y,incY)
!--------------------------------------------------------------
!     copies a vector, x, to a vector, y:  y <- x
!     only for incX=incY=1
!     after BLAS
!     replace this by the function from the optimized BLAS implementation:
!         CALL  SCOPY(N,X,1,Y,1)   or   CALL  DCOPY(N,X,1,Y,1)
!--------------------------------------------------------------
!     USE gckpp_Precision
      
      INTEGER  :: i,incX,incY,M,MP1,N
      REAL(kind=dp) :: X(N),Y(N)

      IF (N.LE.0) RETURN

      M = MOD(N,8)
      IF( M .NE. 0 ) THEN
        DO i = 1,M
          Y(i) = X(i)
        END DO
        IF( N .LT. 8 ) RETURN
      END IF    
      MP1 = M+1
      DO i = MP1,N,8
        Y(i) = X(i)
        Y(i + 1) = X(i + 1)
        Y(i + 2) = X(i + 2)
        Y(i + 3) = X(i + 3)
        Y(i + 4) = X(i + 4)
        Y(i + 5) = X(i + 5)
        Y(i + 6) = X(i + 6)
        Y(i + 7) = X(i + 7)
      END DO

      END SUBROUTINE WCOPY


!--------------------------------------------------------------
      SUBROUTINE WAXPY(N,Alpha,X,incX,Y,incY)
!--------------------------------------------------------------
!     constant times a vector plus a vector: y <- y + Alpha*x
!     only for incX=incY=1
!     after BLAS
!     replace this by the function from the optimized BLAS implementation:
!         CALL SAXPY(N,Alpha,X,1,Y,1) or  CALL DAXPY(N,Alpha,X,1,Y,1)
!--------------------------------------------------------------

      INTEGER  :: i,incX,incY,M,MP1,N
      REAL(kind=dp) :: X(N),Y(N),Alpha
      REAL(kind=dp), PARAMETER :: ZERO = 0.0_dp

      IF (Alpha .EQ. ZERO) RETURN
      IF (N .LE. 0) RETURN

      M = MOD(N,4)
      IF( M .NE. 0 ) THEN
        DO i = 1,M
          Y(i) = Y(i) + Alpha*X(i)
        END DO
        IF( N .LT. 4 ) RETURN
      END IF
      MP1 = M + 1
      DO i = MP1,N,4
        Y(i) = Y(i) + Alpha*X(i)
        Y(i + 1) = Y(i + 1) + Alpha*X(i + 1)
        Y(i + 2) = Y(i + 2) + Alpha*X(i + 2)
        Y(i + 3) = Y(i + 3) + Alpha*X(i + 3)
      END DO
      
      END SUBROUTINE WAXPY



!--------------------------------------------------------------
      SUBROUTINE WSCAL(N,Alpha,X,incX)
!--------------------------------------------------------------
!     constant times a vector: x(1:N) <- Alpha*x(1:N) 
!     only for incX=incY=1
!     after BLAS
!     replace this by the function from the optimized BLAS implementation:
!         CALL SSCAL(N,Alpha,X,1) or  CALL DSCAL(N,Alpha,X,1)
!--------------------------------------------------------------

      INTEGER  :: i,incX,M,MP1,N
      REAL(kind=dp)  :: X(N),Alpha
      REAL(kind=dp), PARAMETER  :: ZERO=0.0_dp, ONE=1.0_dp

      IF (Alpha .EQ. ONE) RETURN
      IF (N .LE. 0) RETURN

      M = MOD(N,5)
      IF( M .NE. 0 ) THEN
        IF (Alpha .EQ. (-ONE)) THEN
          DO i = 1,M
            X(i) = -X(i)
          END DO
        ELSEIF (Alpha .EQ. ZERO) THEN
          DO i = 1,M
            X(i) = ZERO
          END DO
        ELSE
          DO i = 1,M
            X(i) = Alpha*X(i)
          END DO
        END IF
        IF( N .LT. 5 ) RETURN
      END IF
      MP1 = M + 1
      IF (Alpha .EQ. (-ONE)) THEN
        DO i = MP1,N,5
          X(i)     = -X(i)
          X(i + 1) = -X(i + 1)
          X(i + 2) = -X(i + 2)
          X(i + 3) = -X(i + 3)
          X(i + 4) = -X(i + 4)
        END DO
      ELSEIF (Alpha .EQ. ZERO) THEN
        DO i = MP1,N,5
          X(i)     = ZERO
          X(i + 1) = ZERO
          X(i + 2) = ZERO
          X(i + 3) = ZERO
          X(i + 4) = ZERO
        END DO
      ELSE
        DO i = MP1,N,5
          X(i)     = Alpha*X(i)
          X(i + 1) = Alpha*X(i + 1)
          X(i + 2) = Alpha*X(i + 2)
          X(i + 3) = Alpha*X(i + 3)
          X(i + 4) = Alpha*X(i + 4)
        END DO
      END IF

      END SUBROUTINE WSCAL

!--------------------------------------------------------------
      REAL(kind=dp) FUNCTION WLAMCH( C )
!--------------------------------------------------------------
!     returns epsilon machine
!     after LAPACK
!     replace this by the function from the optimized LAPACK implementation:
!          CALL SLAMCH('E') or CALL DLAMCH('E')
!--------------------------------------------------------------
!      USE gckpp_Precision

      CHARACTER ::  C
      INTEGER    :: i
      REAL(kind=dp), SAVE  ::  Eps
      REAL(kind=dp)  ::  Suma
      REAL(kind=dp), PARAMETER  ::  ONE=1.0_dp, HALF=0.5_dp
      LOGICAL, SAVE   ::  First=.TRUE.
      
!$OMP THREADPRIVATE( Eps, First )

      IF (First) THEN
        First = .FALSE.
        Eps = HALF**(16)
        DO i = 17, 80
          Eps = Eps*HALF
          CALL WLAMCH_ADD(ONE,Eps,Suma)
          IF (Suma.LE.ONE) GOTO 10
        END DO
        PRINT*,'ERROR IN WLAMCH. EPS < ',Eps
        RETURN
10      Eps = Eps*2
        i = i-1      
      END IF

      WLAMCH = Eps

      END FUNCTION WLAMCH
     
      SUBROUTINE WLAMCH_ADD( A, B, Suma )
!      USE gckpp_Precision
      
      REAL(kind=dp) A, B, Suma
      Suma = A + B

      END SUBROUTINE WLAMCH_ADD
!--------------------------------------------------------------


!--------------------------------------------------------------
      SUBROUTINE SET2ZERO(N,Y)
!--------------------------------------------------------------
!     copies zeros into the vector y:  y <- 0
!     after BLAS
!--------------------------------------------------------------
      
      INTEGER ::  i,M,MP1,N
      REAL(kind=dp) ::  Y(N)
      REAL(kind=dp), PARAMETER :: ZERO = 0.0d0

      IF (N.LE.0) RETURN

      M = MOD(N,8)
      IF( M .NE. 0 ) THEN
        DO i = 1,M
          Y(i) = ZERO
        END DO
        IF( N .LT. 8 ) RETURN
      END IF    
      MP1 = M+1
      DO i = MP1,N,8
        Y(i)     = ZERO
        Y(i + 1) = ZERO
        Y(i + 2) = ZERO
        Y(i + 3) = ZERO
        Y(i + 4) = ZERO
        Y(i + 5) = ZERO
        Y(i + 6) = ZERO
        Y(i + 7) = ZERO
      END DO

      END SUBROUTINE SET2ZERO


!--------------------------------------------------------------
      REAL(kind=dp) FUNCTION WDOT (N, DX, incX, DY, incY) 
!--------------------------------------------------------------
!     dot produce: wdot = x(1:N)*y(1:N) 
!     only for incX=incY=1
!     after BLAS
!     replace this by the function from the optimized BLAS implementation:
!         CALL SDOT(N,X,1,Y,1) or  CALL DDOT(N,X,1,Y,1)
!--------------------------------------------------------------
!      USE messy_mecca_kpp_Precision
!--------------------------------------------------------------
      IMPLICIT NONE
      INTEGER :: N, incX, incY
      REAL(kind=dp) :: DX(N), DY(N) 

      INTEGER :: i, IX, IY, M, MP1, NS
                                 
      WDOT = 0.0D0 
      IF (N .LE. 0) RETURN 
      IF (incX .EQ. incY) IF (incX-1) 5,20,60 
!                                                                       
!     Code for unequal or nonpositive increments.                       
!                                                                       
    5 IX = 1 
      IY = 1 
      IF (incX .LT. 0) IX = (-N+1)*incX + 1 
      IF (incY .LT. 0) IY = (-N+1)*incY + 1 
      DO i = 1,N 
        WDOT = WDOT + DX(IX)*DY(IY) 
        IX = IX + incX 
        IY = IY + incY 
      END DO 
      RETURN 
!                                                                       
!     Code for both increments equal to 1.                              
!                                                                       
!     Clean-up loop so remaining vector length is a multiple of 5.      
!                                                                       
   20 M = MOD(N,5) 
      IF (M .EQ. 0) GO TO 40 
      DO i = 1,M 
         WDOT = WDOT + DX(i)*DY(i) 
      END DO 
      IF (N .LT. 5) RETURN 
   40 MP1 = M + 1 
      DO i = MP1,N,5 
          WDOT = WDOT + DX(i)*DY(i) + DX(i+1)*DY(i+1) + DX(i+2)*DY(i+2) +  &
                   DX(i+3)*DY(i+3) + DX(i+4)*DY(i+4)                   
      END DO 
      RETURN 
!                                                                       
!     Code for equal, positive, non-unit increments.                    
!                                                                       
   60 NS = N*incX 
      DO i = 1,NS,incX 
        WDOT = WDOT + DX(i)*DY(i) 
      END DO 

      END FUNCTION WDOT                                          


!--------------------------------------------------------------
      SUBROUTINE WADD(N,X,Y,Z)
!--------------------------------------------------------------
!     adds two vectors: z <- x + y
!     BLAS - like
!--------------------------------------------------------------
!     USE gckpp_Precision
      
      INTEGER :: i, M, MP1, N
      REAL(kind=dp) :: X(N),Y(N),Z(N)

      IF (N.LE.0) RETURN

      M = MOD(N,5)
      IF( M /= 0 ) THEN
         DO i = 1,M
            Z(i) = X(i) + Y(i)
         END DO
         IF( N < 5 ) RETURN
      END IF    
      MP1 = M+1
      DO i = MP1,N,5
         Z(i)     = X(i)     + Y(i)
         Z(i + 1) = X(i + 1) + Y(i + 1)
         Z(i + 2) = X(i + 2) + Y(i + 2)
         Z(i + 3) = X(i + 3) + Y(i + 3)
         Z(i + 4) = X(i + 4) + Y(i + 4)
      END DO

      END SUBROUTINE WADD
      
      
      
!--------------------------------------------------------------
      SUBROUTINE WGEFA(N,A,Ipvt,info)
!--------------------------------------------------------------
!     WGEFA FACTORS THE MATRIX A (N,N) BY
!           GAUSS ELIMINATION WITH PARTIAL PIVOTING
!     LINPACK - LIKE 
!--------------------------------------------------------------
!
      INTEGER       :: N,Ipvt(N),info
      REAL(kind=dp) :: A(N,N)
      REAL(kind=dp) :: t, dmax, da
      INTEGER       :: j,k,l
      REAL(kind=dp), PARAMETER :: ZERO = 0.0, ONE = 1.0

      info = 0

size: IF (n > 1) THEN
      
col:  DO k = 1, n-1

!        find l = pivot index
!        l = idamax(n-k+1,A(k,k),1) + k - 1
         l = k; dmax = abs(A(k,k))
         DO j = k+1,n
            da = ABS(A(j,k))
            IF (da > dmax) THEN
              l = j; dmax = da
            END IF
         END DO
         Ipvt(k) = l

!        zero pivot implies this column already triangularized
         IF (ABS(A(l,k)) < TINY(ZERO)) THEN
            info = k
            return
         ELSE   
            IF (l /= k) THEN
               t = A(l,k); A(l,k) = A(k,k); A(k,k) = t
            END IF
            t = -ONE/A(k,k)
            CALL WSCAL(n-k,t,A(k+1,k),1)
            DO j = k+1, n
               t = A(l,j)
               IF (l /= k) THEN
                  A(l,j) = A(k,j); A(k,j) = t
               END IF
               CALL WAXPY(n-k,t,A(k+1,k),1,A(k+1,j),1)
            END DO         
         END IF
         
       END DO col
       
      END IF size
      
      Ipvt(N) = N
      IF (ABS(A(N,N)) == ZERO) info = N
      
      END SUBROUTINE WGEFA


!--------------------------------------------------------------
      SUBROUTINE WGESL(Trans,N,A,Ipvt,b)
!--------------------------------------------------------------
!     WGESL solves the system
!     a * x = b  or  trans(a) * x = b
!     using the factors computed by WGEFA.
!
!     Trans      = 'N'   to solve  A*x = b ,
!                = 'T'   to solve  transpose(A)*x = b
!     LINPACK - LIKE 
!--------------------------------------------------------------

      INTEGER       :: N,Ipvt(N)
      CHARACTER     :: trans
      REAL(kind=dp) :: A(N,N),b(N)
      REAL(kind=dp) :: t
      INTEGER       :: k,kb,l

      
      SELECT CASE (Trans)

      CASE ('n','N')  !  Solve  A * x = b

!        first solve  L*y = b
         IF (n >= 2) THEN
          DO k = 1, n-1
            l = Ipvt(k)
            t = b(l)
            IF (l /= k) THEN
               b(l) = b(k)
               b(k) = t
            END IF
            CALL WAXPY(n-k,t,a(k+1,k),1,b(k+1),1)
          END DO
         END IF
!        now solve  U*x = y
         DO kb = 1, n
            k = n + 1 - kb
            b(k) = b(k)/a(k,k)
            t = -b(k)
            CALL WAXPY(k-1,t,a(1,k),1,b(1),1)
         END DO
      
      CASE ('t','T')  !  Solve transpose(A) * x = b

!        first solve  trans(U)*y = b
         DO k = 1, n
            t = WDOT(k-1,a(1,k),1,b(1),1)
            b(k) = (b(k) - t)/a(k,k)
         END DO
!        now solve trans(L)*x = y
         IF (n >= 2) THEN
         DO kb = 1, n-1
            k = n - kb
            b(k) = b(k) + WDOT(n-k,a(k+1,k),1,b(k+1),1)
            l = Ipvt(k)
            IF (l /= k) THEN
               t = b(l); b(l) = b(k); b(k) = t
            END IF
         END DO
         END IF
   
      END SELECT

      END SUBROUTINE WGESL
! End of BLAS_UTIL function
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~



END MODULE gckpp_LinearAlgebra

